﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Xml.Serialization;
using MyGeometry;


namespace ShapeGraph
{
    public partial class FormMain : Form
    {
        // constructor
        public FormMain()
        {
            InitializeComponent();

			this.crossMatchViewPanel.Hide();

			this.corelatedbuttons.Add(toolStripButtonViewTool);

			this.meshView.isGraphView = false;
			this.meshView.drawModelGraph = false;

			// for a pair of models
			this.sourceGraphView.isGraphView = true;
			this.targetGraphView.isGraphView = true;
			this.sourceGraphView.drawmodel = true;
			this.targetGraphView.drawmodel = true;
			this.sourceGraphView.ShowComponentProxy = true;
			this.targetGraphView.ShowComponentProxy = true;
			this.sourceGraphView.ShowSolidMesh = false;
			this.targetGraphView.ShowSolidMesh = false;
			this.sourceComponentView.isGraphView = false;
			this.targetComponentView.isGraphView = false;
			this.sourceComponentView.drawGroundGrids = false;
			this.targetComponentView.drawGroundGrids = false;
			this.sourceComponentView.ShowSolidMesh = true;
			this.targetComponentView.ShowSolidMesh = true;
			this.sourceComponentView.ShowComponentProxy = false;
			this.targetComponentView.ShowComponentProxy = false;

			this.propertyGrid.SelectedObject = this.meshView.colorpalette;

			this.splitContainer2.Panel2Collapsed = true;

			this.propertyGrid.Hide();
			this.selectTypePanel.Hide();
			nodeTypeColoredGroup.Hide();

        }

		private List<ToolStripButton> corelatedbuttons = new List<ToolStripButton>();
		public void SetMode(MeshView.EnumOperationMode mode)
		{
			foreach (ToolStripButton button in this.corelatedbuttons)
				button.Checked = false;
			meshView.CurrentMode = mode;
			switch (mode)
			{
				case MeshView.EnumOperationMode.Viewing:
					toolStripButtonViewTool.Checked = true;
					break;
			}

			this.meshView.Refresh();
		}

        // NEW_ADD
        public MeshView GetMeshView()
        {
            return meshView;
        }
		public MeshView GetSourceGraphView()
		{
			return sourceGraphView;
		}
		public MeshView GetTargetGraphView()
		{
			return targetGraphView;
		}
		public MeshView GetSourceComponentView()
		{
			return sourceComponentView;
		}
		public MeshView GetTargetComponentView()
		{
			return targetComponentView;
		}

        public void UpdataSegProcess(int step)
        {
            this.SegProcess.Increment(step);
        }
        public void EndSegProcess()
        {
            this.SegProcess.Value = this.SegProcess.Maximum;
            this.SegProcess.Hide();
        }
        public void BeginProcess()
        {
            this.SegProcess.Value = this.SegProcess.Minimum;
            this.SegProcess.Show();
        }

		public void SetPropertyGridObject(MeshRecord rec)
		{
			
		}

		public void AddTabObject(MeshRecord rec)
		{
			string filename = rec.Filename;
			TabPage p = new TabPage(filename);
			p.Tag = rec;
			rec.TabIndex = (byte)this.tabControlMain.TabPages.Count;
			this.tabControlMain.TabPages.Add(p);
			this.tabControlMain.SelectedTab = p;
		}

		public void SetTabObject(MeshRecord rec)
		{
			if (rec == null) return;
			int index = rec.TabIndex;
			if (index < 0) return;
			TabPage page = this.tabControlMain.TabPages[index];
			this.tabControlMain.SelectedTab = page;
		}
		

        private void toolStripButtonViewTool_Click(object sender, EventArgs e)
        {
			this.SetMode(MeshView.EnumOperationMode.Viewing);
			//this.contextMenuStrip.Hide();
        }
        private void toolStripButtonPickTool_Click(object sender, EventArgs e)
        {
			this.SetMode(MeshView.EnumOperationMode.Sketching);
		}

		private void tabControlMain_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (this.tabControlMain.SelectedTab != null)
			{
				MeshRecord rec = (MeshRecord)this.tabControlMain.SelectedTab.Tag;

				if (meshView.meshes.Contains(rec))
					this.meshView.MeshRecord = rec;

				foreach (MeshView.IMeshDisplay obj in this.meshView.MeshRecord.AlgorithmObjects)
				{
					obj.SetData();
				}
			}
			else
			{
				this.meshView.MeshRecord = null;
			}
		}
	
		private void buttonShowHideRightPanel_Click(object sender, EventArgs e)
        {
            this.splitContainer1.Panel2Collapsed =
                !this.splitContainer1.Panel2Collapsed;
        }
		private void buttonShowHideDownPanel_Click(object sender, EventArgs e)
		{
			this.splitContainer2.Panel2Collapsed =
				!this.splitContainer2.Panel2Collapsed;
		}
		private void buttonClearOutputText_Click(object sender, EventArgs e)
		{
			this.textBoxOutputText.Text = "";
		}
		private void buttonCopyOutputText_Click(object sender, EventArgs e)
		{
			if (this.textBoxOutputText.Text != "")
				Clipboard.SetText(this.textBoxOutputText.Text);
		}
		
        // private functions
		private string objectName = null;
		private bool hasIO = false;
		private Mesh LoadMesh(bool normalize)
		{
			OpenFileDialog d = new OpenFileDialog();
			d.InitialDirectory = @".\data\";
			d.FileName = "";
			d.Filter = "Wavefront Object File (*.obj;*.off;*.ply)|*.obj;*.off;*.ply|All files (*.*)|*.*";
			d.CheckFileExists = true;
			DialogResult ret = d.ShowDialog(this);

			if (ret == DialogResult.OK)
			{
				string fullPath = d.FileName;
				char[] delimiters = { '.' };
				string[] tokens = fullPath.Split(delimiters);
				//string extension = fullPath.Substring(fullPath.Length - 3, 3);
				string extension;
				if (tokens.Length > 0)
					extension = tokens[tokens.Length - 1];
				else
					extension = "obj";

				string filename = Path.GetFileName(fullPath);

				int index = fullPath.LastIndexOf('\\');
				this.objectName = fullPath.Substring(index + 1, fullPath.Length - index - 1);

				// load mesh model
				StreamReader sr = new StreamReader(fullPath);
				Mesh m = new Mesh(sr, extension, normalize);
				sr.Close();

				// add new tabpage and mesh record
				MeshRecord rec = new MeshRecord(m);
				rec.Filename = fullPath;// filename;
				rec.objName = this.objectName;

				this.meshView.SetModel(rec);

				// try loading segmentation file
				if (!meshView.LoadSegDetail())
				{
					this.meshView.meshComponents = this.meshView.DividMeshIntoCompnents(m);
					this.meshView.GetComponentMeshes();
				}
				else
				{
					this.meshView.DecomposeBySeg();
				}

				this.meshView.drawModelGraph = true;

				this.meshView.modelGraph = this.meshView.LoadModelGraphInfo(0);

				this.meshView.modelGraph.CalculateNodeLvelByDirections();

				hasIO = false;
				foreach(ModelGraphNode mn in this.meshView.modelGraph.graphNodes)
				{
					if(mn.name.Contains("input"))
					{
						hasIO = true;
						break;
					}
				}
				this.meshView.mcIdx = -1;
				this.meshView.iterIdx = 0;

				return m;
			}

			return null;
		}

        private void AddMesh()
        {
            OpenFileDialog d = new OpenFileDialog();
            d.FileName = "";
            //d.Filter = "Wavefront Object File (*.obj)|*.obj|All files (*.*)|*.*";
            d.Filter = "Wavefront Object File (*.obj;*.off;*.ply)|*.obj;*.off;*.ply|All files (*.*)|*.*";
            d.CheckFileExists = true;
			d.Multiselect = true;
            DialogResult ret = d.ShowDialog(this);
            if (ret == DialogResult.OK)
            {
				string[] allfullPaths = d.FileNames;
               
                char[] delimiters = { '.'};
				for (int nfile = 0; nfile < allfullPaths.Length; nfile++)
				{
					string fullPath = allfullPaths[nfile]; 
					string[] tokens = fullPath.Split(delimiters);
					//string extension = fullPath.Substring(fullPath.Length - 3, 3);
					string extension;
					if (tokens.Length > 0)
						extension = tokens[tokens.Length - 1];
					else
						extension = "obj";

					string filename = Path.GetFileName(fullPath);

					int index = fullPath.LastIndexOf('\\');
					this.objectName = fullPath.Substring(index + 1, fullPath.Length - index - 1);

					// load mesh model
					StreamReader sr = new StreamReader(fullPath);
					Mesh m = new Mesh(sr, extension, false);
					sr.Close();

					// add new tabpage and mesh record
					List<Mesh> meshes = new List<Mesh>();
					MeshRecord rec = null;
					int currentVertexCount = 0, currentFaceCount = 0;
					if (this.meshView.MeshRecord == null)
					{
						rec = new MeshRecord(m);
						this.meshView.meshComponents = new List<MeshComponent>();
						this.meshView.SetModel(rec);
					}
					else
					{
						rec = this.meshView.MeshRecord;
						meshes.Add(rec.Mesh);
						currentVertexCount = rec.Mesh.VertexCount;
						currentFaceCount = rec.Mesh.FaceCount;
					}
					rec.Filename = fullPath;// filename;
					rec.objName = this.objectName;
					meshes.Add(m);
					rec.Mesh = Mesh.CreateMesh(meshes);

					// try loading segmentation file
					List<MeshComponent> mcs = new List<MeshComponent>();

					mcs = this.meshView.DividMeshIntoCompnents(m);

					MeshComponent gmc = null;
					if (mcs.Count > 1)
					{
						gmc = this.meshView.GroupComponents(mcs, m);
					}
					else
					{
						gmc = mcs[0];
						this.meshView.meshComponents.AddRange(mcs);
					}
					if (currentVertexCount > 0)
					{
						for (int i = 0; i < gmc.faceIndex.Length; ++i)
							gmc.faceIndex[i] += currentFaceCount;
					}
					
					//this.AddTabObject(rec);
				}//end-for
				this.meshView.GetComponentMeshes();
            }
		}//End-Add-Mesh

		// public functions
		public void OutputText(string s, bool newLine)
		{
			if (newLine)
				this.textBoxOutputText.AppendText(s + System.Environment.NewLine);
			else
				this.textBoxOutputText.AppendText(s);
		}
		public void DisplayData(double[] data1, double[] data2, int index)
		{


		}

		private void toolStripButtonGroupingTool_Click(object sender, EventArgs e)
		{
			this.SetMode(MeshView.EnumOperationMode.ComponentGrouping);
			this.meshView.ShowComponentProxy = true;
		}

		private void groupComponentsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.meshView.GroupComponents(this.meshView.selectedMeshComponents,
				this.meshView.MeshRecord.Mesh);
			this.meshView.Refresh();
		}

		private void toolStripBTest_Click(object sender, EventArgs e)
		{
			string filename = this.objectName + ".obj";
			StreamWriter sw = new StreamWriter(filename);
			this.meshView.MeshRecord.Mesh.Write(sw);
			sw.Close();

			Program.OutputText("saved file to " + filename, true);
		}

		private void SwitchXY_Click(object sender, EventArgs e)
		{
			MeshView view = this.meshView;

			if (view == null || view.MeshRecord == null || view.MeshRecord.Mesh == null)
				return;

			Mesh m = view.MeshRecord.Mesh;
			for (int i = 0, j = 0; i < m.VertexCount; i++, j += 3)
			{
				double y = m.VertexPos[j + 1];
				double x = m.VertexPos[j];
				m.VertexPos[j + 1] = x;
				m.VertexPos[j] = -y;
			}
			m.ComputeFaceNormal();
			m.ComputeVertexNormal();

			if (view.meshComponents != null)
			{
				foreach (MeshComponent mc in view.meshComponents)
				{
					Mesh m2 = mc.mesh;
					for (int i = 0, j = 0; i < m2.VertexCount; i++, j += 3)
					{
						double y = m2.VertexPos[j + 1];
						double x = m2.VertexPos[j];
						m2.VertexPos[j + 1] = x;
						m2.VertexPos[j] = -y;
					}
					m2.ComputeFaceNormal();
					m2.ComputeVertexNormal();
				}
			}

			view.Refresh();
		}

		private void SwitchYZ_Click(object sender, EventArgs e)
		{
			MeshView view = this.meshView;

			if (view == null || view.MeshRecord == null || view.MeshRecord.Mesh == null)
				return;

			Mesh m = view.MeshRecord.Mesh;
			for (int i = 0, j = 0; i < m.VertexCount; i++, j += 3)
			{
				double z = m.VertexPos[j + 2];
				double y = m.VertexPos[j + 1];
				m.VertexPos[j + 2] = y;
				m.VertexPos[j + 1] = -z;
			}
			m.ComputeFaceNormal();
			m.ComputeVertexNormal();


			if (view.meshComponents != null)
			{
				foreach (MeshComponent mc in view.meshComponents)
				{
					Mesh m2 = mc.mesh;
					for (int i = 0, j = 0; i < m2.VertexCount; i++, j += 3)
					{
						double z = m2.VertexPos[j + 2];
						double y = m2.VertexPos[j + 1];
						m2.VertexPos[j + 2] = y;
						m2.VertexPos[j + 1] = -z;
					}
					m2.ComputeFaceNormal();
					m2.ComputeVertexNormal();
				}
			}


			view.Refresh();
		}

		private void SwitchXZ_Click(object sender, EventArgs e)
		{
			MeshView view = this.meshView;

			if (view == null || view.MeshRecord == null || view.MeshRecord.Mesh == null)
				return;

			Mesh m = view.MeshRecord.Mesh;
			for (int i = 0, j = 0; i < m.VertexCount; i++, j += 3)
			{
				double z = m.VertexPos[j + 2];
				double x = m.VertexPos[j];
				m.VertexPos[j + 2] = x;
				m.VertexPos[j] = -z;
			}
			m.ComputeFaceNormal();
			m.ComputeVertexNormal();

			if (view.meshComponents != null)
			{
				foreach (MeshComponent mc in view.meshComponents)
				{
					Mesh m2 = mc.mesh;
					for (int i = 0, j = 0; i < m2.VertexCount; i++, j += 3)
					{
						double z = m2.VertexPos[j + 2];
						double x = m2.VertexPos[j];
						m2.VertexPos[j + 2] = x;
						m2.VertexPos[j] = -z;
					}
					m2.ComputeFaceNormal();
					m2.ComputeVertexNormal();
				}
			}

			view.Refresh();
		}

		private void meshToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SaveFileDialog d = new SaveFileDialog();
            d.FileName = "";
            d.Filter = "Obj File (*.obj;)|*.obj;";
            DialogResult ret = d.ShowDialog(this);

			if (ret == DialogResult.OK)
			{
				StreamWriter sw = new StreamWriter(d.FileName);

				//this.meshView.SaveCurrentMesh();

				this.meshView.MeshRecord.Mesh.Write(sw);
				sw.Close();

				Program.OutputText("saved file to " + d.FileName, true);

				this.meshView.MeshRecord.Filename = d.FileName;
			}
		}		

		private void meshFIleToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.LoadMesh(false);
		}

		//***Select different family of models***//
		private void showChairModels_Click(object sender, EventArgs e)
		{
			this.meshView.modelType = 0;
			LoadModelImage(0);
			LoadModels(0);
			this.meshView.ConstructGraphGroup();
			SwitchModelIndex(0);
		}

		private void showFacadeModels_Click(object sender, EventArgs e)
		{
			this.meshView.modelType = 1;
			LoadModelImage(1);
			LoadModels(1);
			this.meshView.ConstructGraphGroup();
			SwitchModelIndex(0);
		}
		
		string filePath = null;
		string camName = null;
		private void LoadModels(int modelType)
		{
			// load all model in specified type, e.g. chairs			

			string[] files = Directory.GetFiles(filePath, "*.obj");
			this.meshView.graphGroup = new GraphGroup();
			this.meshView.modelGraphs = new List<ModelGraph>();
			this.meshView.meshComponents = new List<MeshComponent>();
			int gidx = 0;
			foreach (string file in files)
			{
				LoadMesh(file);
				// try loading segmentation file
				if (!meshView.LoadSegDetail())
				{
					Program.OutputText("Component file not existed!", true);
					return;
				}
				this.meshView.DecomposeBySeg();
				// try load graph file
				ModelGraph mg = null;
				mg = this.meshView.LoadModelGraphInfo(gidx);
				mg.modelType = modelType;
				this.meshView.ExtractSupport(ref mg);
				this.meshView.modelGraphs.Add(mg);
				++gidx;
			}
			if (File.Exists(camName))
			{
				camName = filePath + @"\camera.cam";
				StreamReader sr = new StreamReader(camName);
				this.meshView.LoadCamera(sr);
				sr.Close();
			}
			this.Refresh();
		}

		private void LoadModelImage(int modelType)
		{
			string modelTypeName = null;
			filePath = @".\data\crossMatch\";
			switch (modelType)
			{
				case 0:
					modelTypeName = @"bicycles";
					break;
				case 1:
					modelTypeName = @"playgrounds";
					break;
				case 2:
					modelTypeName = @"tractors";
					break;
				default:
					break;
			}
			string imagePath = filePath + modelTypeName + @"\images\";
			string[] files = Directory.GetFiles(imagePath, "*.png");
			filePath += modelTypeName;

			List<Image> thumbs = new List<Image>();
			int index = 0;
			foreach (string file in files)
			{
				Bitmap img = new Bitmap(file);
				thumbs.Add(img);

				int s = file.LastIndexOf('\\');
				int t = file.LastIndexOf('.');
				string name = file.Substring(s + 1, t - s - 1);
				index++;
			}

			this.MakeThumnails(thumbs);
		}

		public void MakeThumnails(List<Image> images)
		{
			this.modelList.StopThumbnailsGeneration();
			this.modelList.ShowThumbnails(images);
		}

		private void constructGraph_Click(object sender, EventArgs e)
		{
			this.meshView.drawModelGraph = true;
			this.showGraph.Checked = this.meshView.drawModelGraph;

			//this.meshView.InitializeModelGraph();
			//this.meshView.drawModelGraph = true;
			this.meshView.Refresh();
		}

		private void saveGraphInfo_Click(object sender, EventArgs e)
		{
			// save single graph info -- grouped component + graph(edge)
			this.meshView.SaveSegDetail(null);
			this.meshView.SaveModelGraphInfo(null);
		}

		private void loadGraphInfo_Click(object sender, EventArgs e)
		{
			// load single graph
			this.meshView.modelGraph = this.meshView.LoadModelGraphInfo_old(0);
			this.Refresh();
		}

		private Mesh LoadMesh(string fullPath)
		{
			char[] delimiters = { '.' };
			string[] tokens = fullPath.Split(delimiters);
			string extension;
			if (tokens.Length > 0)
				extension = tokens[tokens.Length - 1];
			else
				extension = "obj";

			string filename = Path.GetFileName(fullPath);

			int index = fullPath.LastIndexOf('\\');
			this.objectName = fullPath.Substring(index + 1, fullPath.Length - index - 1);

			// load mesh model
			StreamReader sr = new StreamReader(fullPath);
			Mesh m = new Mesh(sr, extension, true);
			sr.Close();

			//// add new tabpage and mesh record
			MeshRecord rec = new MeshRecord(m);
			rec.Filename = fullPath;// filename;
			rec.objName = this.objectName;

			//Vector3d minCoord = m.MinCoord();
			//Matrix4d trans = Matrix4d.TranslationMatrix(-1.0 * minCoord);
			//m.Transform(trans);
			//m.UpdateVertexPos();

			this.meshView.SetModel(rec);
			//this.AddTabObject(rec);

			return m;
		}

		public void SwitchModelIndex(int mIndex)
		{
			//trackBar1.Value = 0;
			//trackBar2.Value = 0;
			this.meshView.SwitchModel(mIndex);
		}
		
		private void userLabel_Click(object sender, EventArgs e)
		{
			this.functionTab.SelectedIndex = 1;
			this.meshView.showNodeTypeGroup = !this.meshView.showNodeTypeGroup;
			this.propertyGrid.Show();
			this.selectTypePanel.Show();		
		}

		private void showGraph_Click(object sender, EventArgs e)
		{
			this.meshView.drawModelGraph = !this.meshView.drawModelGraph;
			this.showGraph.Checked = this.meshView.drawModelGraph;
			this.meshView.Refresh();
		}

		private void delAEdge_Click(object sender, EventArgs e)
		{
			this.meshView.DeleteAEdge();
			this.meshView.Refresh();
		}

		private void showVertex_Click(object sender, EventArgs e)
		{
			this.meshView.showSelectedBoxVertex = true;
			this.meshView.selectedboxVertices = new List<Vector3d>();

			this.SetMode(MeshView.EnumOperationMode.UpdateContact);
			this.Refresh();
		}

		private void updateContact_Click(object sender, EventArgs e)
		{
			//this.meshView.UpdateContactPoint();
			//this.meshView.UpdateEdges();
			this.meshView.LocateContactFaces();
			this.meshView.Refresh();
		}

		private void edgeBuild_Click(object sender, EventArgs e)
		{
			this.meshView.AddAEdge();
			this.meshView.Refresh();
		}		

		private void decomposeGroupToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.meshView.DeGroupComponents(this.meshView.selectedMeshComponents);
			this.Refresh();
		}

		private void buttonCloseModel_Click(object sender, EventArgs e)
		{
			this.meshView.Clear();

			if (this.tabControlMain.SelectedTab == null) return;

			MeshRecord rec = (MeshRecord)this.tabControlMain.SelectedTab.Tag;
			if (rec != null)
			{
				this.tabControlMain.TabPages.Remove(this.tabControlMain.SelectedTab);

				// update the tab index for all meshrecords
				int index = 0;
				foreach (TabPage page in this.tabControlMain.TabPages)
				{
					MeshRecord r = (MeshRecord)page.Tag;
					r.TabIndex = (byte)index++;
				}

				if (this.meshView.meshes.Contains(rec))
				{
					this.meshView.meshes.Remove(rec);
					if (this.meshView.meshes.Count < 1)
						this.meshView.MeshRecord = null;
					this.meshView.Refresh();
				}

			}
		}

		private void showProxy_Click(object sender, EventArgs e)
		{
			this.meshView.ShowComponentProxy = !this.meshView.ShowComponentProxy;
			showProxy.Checked = this.meshView.ShowComponentProxy;
			this.meshView.Refresh();
		}

		private void showSolidMesh_Click(object sender, EventArgs e)
		{
			this.meshView.ShowSolidMesh = !this.meshView.ShowSolidMesh;
			showSolidMesh.Checked = this.meshView.ShowSolidMesh;

			this.meshView.drawModelGraph = false;
			this.showGraph.Checked = this.meshView.drawModelGraph;

			this.meshView.ShowComponentProxy = false;
			showProxy.Checked = this.meshView.ShowComponentProxy;

			this.meshView.Refresh();
		}

		private void OK_Click(object sender, EventArgs e)
		{
			this.DialogResult = DialogResult.OK;
			string selectedItem = this.propertyGrid.SelectedGridItem.Label;
			int length = selectedItem.Length - 2;
			string number = selectedItem.Substring(1, length);
			int index = Int32.Parse(number);
			this.meshView.SetNodeType(index);
		}

		private void Cancel_Click(object sender, EventArgs e)
		{
			this.DialogResult = DialogResult.Cancel;
		}

		public bool selectSourceGraph = false, selectTargetGraph = false;
		private void selSourceGraph_Click(object sender, EventArgs e)
		{
			this.selectSourceGraph = true;
			this.selectTargetGraph = false;
		}

		private void selTargetGraph_Click(object sender, EventArgs e)
		{
			this.selectTargetGraph = true;
			this.selectSourceGraph = false;
		}

		private void beginMatch_Click(object sender, EventArgs e)
		{
			bool isvalid = this.meshView.VerifyPath(this.meshView.modelGraph);
			//if (this.sourceGraphView.modelGraph != null && this.targetGraphView.modelGraph != null)
			//    this.meshView.InitializeGraphMatch(this.sourceGraphView.modelGraph, this.targetGraphView.modelGraph);
			//this.sourceGraphView.Refresh();
			//this.targetGraphView.Refresh();
		}

		public void SelectAGraph(int gIndex)
		{
			if (selectSourceGraph)
			{
				this.sourceGraphView.modelGraph =
					this.meshView.graphGroup.kidModelGraphs[gIndex].Clone() as ModelGraph;
				this.sourceGraphView.modelGraph.CalculateNodeLvelByDirections();
				this.sourceGraphView.modelGraph.CompleteAdjEdgesByCycles(true);

				if (File.Exists(camName))
				{
					StreamReader sr = new StreamReader(camName);
					this.sourceGraphView.LoadCamera(sr);
					sr.Close();
				}
				this.sourceGraphView.Refresh();
				this.selectSourceGraph = false;
			}
			if (selectTargetGraph)
			{
				this.targetGraphView.modelGraph =
					this.meshView.graphGroup.kidModelGraphs[gIndex].Clone() as ModelGraph;
				this.targetGraphView.modelGraph.CalculateNodeLvelByDirections();
				this.targetGraphView.modelGraph.CompleteAdjEdgesByCycles(true);				
				//this.targetGraphView.drawModelGraph = false;
				if (File.Exists(camName))
				{
					StreamReader sr = new StreamReader(camName);
					this.targetGraphView.LoadCamera(sr);
					sr.Close();
				}
				this.targetGraphView.Refresh();
				this.selectTargetGraph = false;
			}
		}

		private void searchSubgraphMatch_Click(object sender, EventArgs e)
		{
			if (this.sourceGraphView.modelGraph != null && this.targetGraphView.modelGraph != null)
			{
				this.sourceGraphView.RemoveDummyEdges();
				this.targetGraphView.RemoveDummyEdges();
				this.meshView.InitializeGraphMatch(this.sourceGraphView.modelGraph, this.targetGraphView.modelGraph);
				// min #cutEdges & dummy edges & height
				if(this.meshView.modelType == 0)
					this.meshView.SearchEquivalentSubgraph(2, 2, false, false);
				else if (this.meshView.modelType == 2)
					this.meshView.SearchEquivalentSubgraph(0, 1, true, false);
				else 
					this.meshView.SearchEquivalentSubgraph(0, 1, false, true);					
			}
            this.meshView.searchMODE = 1;
			this.meshView.mcIdx = -1;
			this.sourceGraphView.mcIdx = -1;
			this.targetGraphView.mcIdx = -1;
			if (this.meshView.sourceGraph != null && this.sourceGraphView != null)
			{
				if (this.meshView.sourceGraph.equCutpairs != null &&
					this.meshView.sourceGraph.equCutpairs.Count > 0)
				{
					npairs = this.meshView.sourceGraph.equCutpairs.Count;
					this.meshView.mcIdx = 0;
					this.UpdatePageCross();
				}
			}
			this.sourceGraphView.Refresh();
			this.targetGraphView.Refresh();
		}//searchSubgraphMatch

		private void addMeshToolStripMenuItem_Click(object sender, EventArgs e)
		{
			//this.AddMesh();
			FolderBrowserDialog dialog = new FolderBrowserDialog();
			dialog.SelectedPath = @"F:\Projects\GraphEditing\models";
			DialogResult ret = dialog.ShowDialog(this);
			
			if (ret == DialogResult.OK)
			{
			    string foldername = dialog.SelectedPath;
			    this.meshView.LoadComponentDetail(foldername, 1);
			}
			this.meshView.Refresh();
		}

		private void showBicycleModel_Click(object sender, EventArgs e)
		{
			this.selectAModelType(2);
		}

		private void showIrregularFacadesModel_Click(object sender, EventArgs e)
		{
			this.selectAModelType(3);
		}

		private void showRobotsModel_Click(object sender, EventArgs e)
		{
			this.selectAModelType(4);
		}

		private void showTractorsModel_Click(object sender, EventArgs e)
		{
			this.selectAModelType(5);
		}

		private void showFurnitureLayoutModel_Click(object sender, EventArgs e)
		{
			this.selectAModelType(6);
		}

		private void showPlanesModel_Click(object sender, EventArgs e)
		{
			this.selectAModelType(7);
		}

		private void showPlaygroundsModel_Click(object sender, EventArgs e)
		{
			this.selectAModelType(8);
		}

		private void playareasToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.selectAModelType(9);
		}

		private void mixModelsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.selectAModelType(10);
		}


		private void selectAModelType(int modelType)
		{
			this.meshView.modelType = modelType;
			LoadModelImage(modelType);
			LoadModels(modelType);
			this.meshView.ConstructGraphGroup();
			SwitchModelIndex(0);
		}

		private void drawRefGrids_Click(object sender, EventArgs e)
		{
			this.meshView.drawGroundGrids = !this.meshView.drawGroundGrids;
			this.drawRefGrids.Checked = this.meshView.drawGroundGrids;
			this.sourceGraphView.drawGroundGrids = !this.sourceGraphView.drawGroundGrids;
			this.targetGraphView.drawGroundGrids = !this.targetGraphView.drawGroundGrids;
			this.sourceComponentView.drawGroundGrids = !this.sourceComponentView.drawGroundGrids;
			this.targetComponentView.drawGroundGrids = !this.targetComponentView.drawGroundGrids;
			this.meshView.Refresh();
			this.sourceComponentView.Refresh();
			this.targetComponentView.Refresh();
			this.sourceGraphView.Refresh();
			this.targetGraphView.Refresh();
		}

		private void entryLabel_Click(object sender, EventArgs e)
		{
			this.meshView.MarkEntry();
		}

		private void exitLabel_Click(object sender, EventArgs e)
		{
			this.meshView.MarkExit();
		}

		private void coloredMeshToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SaveFileDialog d = new SaveFileDialog();
			d.FileName = "";
			d.Filter = "Obj File (*.obj;)|*.obj;";
			DialogResult ret = d.ShowDialog(this);

			if (ret == DialogResult.OK)
			{
				StreamWriter sw = new StreamWriter(d.FileName);

				// mtl file
				string mtl = d.FileName.Substring(0, d.FileName.Length - 4);
				mtl += ".mtl";
				StreamWriter sw_mtl = new StreamWriter(mtl);
				int index = mtl.LastIndexOf('\\');
				string mtlname = mtl.Substring(index+1, mtl.Length - index-1);
				sw.WriteLine("mtllib " + mtlname);
				this.meshView.WriteMeshFile(sw);
				this.meshView.WriteMTLFile(sw_mtl);
				sw.Close();
				sw_mtl.Close();

				Program.OutputText("saved file to " + d.FileName, true);

				this.meshView.MeshRecord.Filename = d.FileName;

			}
		}

		private void subgraphsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			FolderBrowserDialog dialog = new FolderBrowserDialog();
			dialog.SelectedPath = @"F:\Projects\GraphEditing\models\playgrounds\output\playground\iter_1\";
			//"F:\Projects\GraphEditing\models\";
			DialogResult ret = dialog.ShowDialog(this);
			if (ret == DialogResult.OK)
			{
			    string meshFileFolder = dialog.SelectedPath + @"\";
				string subgraphPathName = meshFileFolder;
				//@"F:\Projects\GraphEditing\models\raceTrack\output\iter_1\subgraphs\";
				//if (filePath != null)
				//{
				//    filePath.LastIndexOf('\\');
				//    subgraphPathName = filePath.Substring(0, index);
				//}
				//else if (this.meshView.modelGraph != null)
				//{
				//    filePath = this.meshView.modelGraph.meshfile;
				//    subgraphPathName = filePath;
				//}
				//else return;

				//string modelTypeName = filePath.Substring(index, filePath.Length-index);
				//subgraphPathName += @"\replaceableSubgraphs";
				//subgraphPathName += modelTypeName + @"\";
				//this.meshView.ExportSubgraphs(subgraphPathName);
				this.meshView.ExportSelfSubgraphs(subgraphPathName);
			}
		}	

		private void searchwithDummyEdges_Click(object sender, EventArgs e)
		{
			//if (this.sourceGraphView.modelGraph != null && this.targetGraphView.modelGraph != null)
			//{
			//    this.sourceGraphView.RemoveDummyEdges();
			//    this.targetGraphView.RemoveDummyEdges();
			//    this.meshView.InitializeGraphMatch(this.sourceGraphView.modelGraph, this.targetGraphView.modelGraph);
			//    this.meshView.SearchEquivalentSubgraph(int.Parse(this.maxdistanceOfdummyEdge.Text),
			//        int.Parse(this.numOfMinCuts.Text), this.allowSym.Checked, this.useheight.Checked);
			//}
			this.UpdateIteration();
		}

		private void UpdateIteration()
		{
			this.meshView.mcIdx = -1;
			this.sourceGraphView.mcIdx = -1;
			this.targetGraphView.mcIdx = -1;
			if (this.meshView.sourceGraph != null && this.targetGraphView != null)
			{
				if (this.meshView.sourceGraph.equCutpairs != null)
				{
					int npairs = this.meshView.sourceGraph.equCutpairs.Count;
					this.numOfMatch.Text = npairs.ToString();
				}
			}
			this.sourceGraphView.Refresh();
			this.targetGraphView.Refresh();
		}

		private void loadGraphNew_Click(object sender, EventArgs e)
		{
			// load single graph
			this.meshView.modelGraph = this.meshView.LoadModelGraphInfo(0);
			if(this.meshView.modelGraph.meshfile.Contains("ball"))
				this.meshView.modelGraph.CalculateNodeLvelByDirections();
			this.meshView.mcIdx = -1;
			this.meshView.iterIdx = 0;
			//this.meshView.modelGraph.ReDefEdgeDir();

			//string fullPath = @"F:\Projects\GraphEditing\models\castle\castle.cam";
			//StreamReader sr = new StreamReader(fullPath);

			//    this.meshView.LoadCamera(sr);
			//    sr = new StreamReader(fullPath);
			//sr.Close();

			this.meshView.Refresh();


			inputIndex = 0;
			this.Refresh();
		}

		private void assignSgIdex_Click(object sender, EventArgs e)
		{
			int npairs = 0;
			if (this.meshView.sourceGraph != null && this.sourceGraphView != null)
			{
				if (this.meshView.sourceGraph.equCutpairs != null)
				{
					npairs = this.meshView.sourceGraph.equCutpairs.Count;
					this.sourceGraphView.cutPairs = this.meshView.sourceGraph.equCutpairs;
					this.targetGraphView.cutPairs = this.meshView.targetGraph.equCutpairs;
				}
				int sgIndex = (int)double.Parse(this.sgIndex.Text);
				if (npairs == 0 || sgIndex >= npairs)
					this.meshView.mcIdx = -1;
				else
					this.meshView.mcIdx = sgIndex;
				this.sourceGraphView.mcIdx = this.meshView.mcIdx;
				this.targetGraphView.mcIdx = this.meshView.mcIdx;
				this.meshView.SetComponentView(0);
				this.meshView.SetComponentView(1);
			}
			this.sourceGraphView.Refresh();
			this.targetGraphView.Refresh();
			this.sourceComponentView.Refresh();
			this.targetComponentView.Refresh();
		}


		private void replace_G1_Click(object sender, EventArgs e)
		{
			if(this.meshView.mcIdx < 0) return;
			this.meshView.UpdateGraph(0);
			this.sourceGraphView.Refresh();
			this.targetGraphView.Refresh();
			this.sourceComponentView.Refresh();
			this.targetComponentView.Refresh();
		}

		private void replace_G2_Click(object sender, EventArgs e)
		{
			if (this.meshView.mcIdx < 0) return;
			this.meshView.UpdateGraph(1);
			this.sourceGraphView.Refresh();
			this.targetGraphView.Refresh();
			this.sourceComponentView.Refresh();
			this.targetComponentView.Refresh();
		}

		private void edgeOrdersToolStripMenuItem_Click(object sender, EventArgs e)
		{
			string edgePathName = @"F:\Projects\GraphEditing\models\playgrounds\edgeOrders\";
			this.meshView.ExportEdgeOrder(edgePathName);
		}

		private void loadSubgraphsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			// load saved matching subgraphs
            if (this.sourceGraphView.modelGraph == null ||
				this.targetGraphView.modelGraph == null)
                return;
			FolderBrowserDialog dialog = new FolderBrowserDialog();
			dialog.SelectedPath = @"F:\Projects\GraphEditing\models\replaceableSubgraphs\playgrounds";
			DialogResult ret = dialog.ShowDialog(this);
			
			if (ret == DialogResult.OK)
			{
				string foldername = dialog.SelectedPath;
				string[] objfiles = Directory.GetFiles(foldername, "*.subgraph");
				this.meshView.InitializeGraphMatch(this.sourceGraphView.modelGraph, this.targetGraphView.modelGraph);
				this.meshView.LoadSubgraphs(objfiles, 1);
				if (this.meshView.sourceGraph != null && this.sourceGraphView != null)
				{
					if (this.meshView.sourceGraph.equCutpairs != null)
					{
						int npairs = this.meshView.sourceGraph.equCutpairs.Count;
						this.numOfMatch.Text = npairs.ToString();
					}
				}
			}
		}

		public string iterationFolderName = null;
		private void saveCurrent_Click(object sender, EventArgs e)
		{
			SaveReplacedSubgraph(1);
		}

		private void saveCurrent_2_Click(object sender, EventArgs e)
		{
			SaveReplacedSubgraph(2);
		}

		private void SaveReplacedSubgraph(int repGraph)
		{
			if (this.meshView.iterationIndex == 0)
			{
				int index = filePath.LastIndexOf('\\');
				iterationFolderName = filePath.Substring(0, index);
				string modelTypeName = filePath.Substring(index, filePath.Length - index);
				iterationFolderName += @"\subgraphSequence";
				iterationFolderName += modelTypeName + @"\";
				if (this.meshView.sourceGraph != null)
					iterationFolderName += this.meshView.sourceGraph.objfilename + @"_";
				if (this.meshView.targetGraph != null)
					iterationFolderName += this.meshView.targetGraph.objfilename;
			}
			//index = 0;
			string wholefolderName = iterationFolderName + @"_" + DateTime.Now.ToString("d_M") + @"\";
			//string wholefolderName = subgraphPathName + @"_" + index.ToString() + @"\";
			//while (Directory.Exists(wholefolderName))
			//{
			//    ++index;
			//    wholefolderName = subgraphPathName + @"_" + index.ToString() + @"\";
			//}
			Directory.CreateDirectory(wholefolderName);
			this.meshView.SaveCurrentSubgraphReplacementInfo(wholefolderName, repGraph);
			this.UpdateIteration();
		}

		private void replaceableNodesToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.meshView.FindCorrespondingNodes(this.sourceGraphView.modelGraph,
				this.targetGraphView.modelGraph);
		}

		private void exportGraphToolStripMenuItem_Click(object sender, EventArgs e)
		{
            int index = 0;
            string subgraphPathName = null;
            if (filePath != null)
            {
                index = filePath.LastIndexOf('\\');
                subgraphPathName = filePath.Substring(0, index);
                string modelTypeName = filePath.Substring(index, filePath.Length - index);
                subgraphPathName += @"\graphPdfs";
                subgraphPathName += modelTypeName + @"\";
            }

			this.meshView.ExportSubgGraphPairToPSFiles(subgraphPathName, this.meshView.mcIdx);
		}

		private void showModelInGraphView_Click(object sender, EventArgs e)
		{
			this.sourceGraphView.drawmodel = !this.sourceGraphView.drawmodel;
			this.targetGraphView.drawmodel = !this.targetGraphView.drawmodel;
			this.sourceGraphView.drawModelGraph = !this.sourceGraphView.drawModelGraph;
			this.targetGraphView.drawModelGraph = !this.targetGraphView.drawModelGraph;
			//this.showModelInGraphView.Checked = this.sourceGraphView.drawmodel;
			this.sourceGraphView.Refresh();
			this.targetGraphView.Refresh();
		}

		private void exportPDFToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SaveFileDialog d = new SaveFileDialog();
			
			d.FileName = "";
			d.Filter = "pdf File (*.ps;)|*.ps;";
			DialogResult ret = d.ShowDialog(this);

			if (ret == DialogResult.OK)
			{
				this.meshView.ExportGraphPDFs(d.FileName);

			}
		}

		private void motion_Click(object sender, EventArgs e)
		{
			this.meshView.drawModelGraph = false;
			this.meshView.ShowComponentProxy = false;

			string fullPath = @".\data\ball.obj";
			char[] delimiters = { '.' };
			string[] tokens = fullPath.Split(delimiters);
			//string extension = fullPath.Substring(fullPath.Length - 3, 3);
			string extension = "obj";
			string filename = Path.GetFileName(fullPath);

			int index = fullPath.LastIndexOf('\\');
			this.objectName = fullPath.Substring(index + 1, fullPath.Length - index - 1);

			// load mesh model
			StreamReader sr = new StreamReader(fullPath);
			this.meshView.trackball = new Mesh(sr, extension, true);
			sr.Close();

			SelectARandomBallPos();

			if (inputIndex != -1)
			{
				this.meshView.RollingBall();
			}
		}		

		int inputIndex = 0;
		private void LoadBall_Click(object sender, EventArgs e)
		{
			string fullPath = @".\data\ball.obj";
			char[] delimiters = { '.' };
			string[] tokens = fullPath.Split(delimiters);
			//string extension = fullPath.Substring(fullPath.Length - 3, 3);
			string extension = "obj";
			string filename = Path.GetFileName(fullPath);

			int index = fullPath.LastIndexOf('\\');
			this.objectName = fullPath.Substring(index + 1, fullPath.Length - index - 1);

			// load mesh model
			StreamReader sr = new StreamReader(fullPath);
			this.meshView.trackball = new Mesh(sr, extension, true);
			sr.Close();
			SelectARandomBallPos();
			
		}

		private void SelectARandomBallPos()
		{
			double smallest = double.MaxValue;
			Vector3d pos = new Vector3d();
			if (this.meshView.modelGraph == null)
			{
				MessageBox.Show("Please load graph first.");
				return;
			}
			List<ModelGraphNode> inputs = new List<ModelGraphNode>();
			foreach (ModelGraphNode n in this.meshView.modelGraph.graphNodes)
			{
				if (n.name.Contains("input"))
					inputs.Add(n);
			}
			if (inputs.Count == 0)
			{
				inputIndex = -1;
				return;
			}
			Random rand = new Random();
			inputIndex = rand.Next(inputs.Count);
			ModelGraphNode mn = inputs[inputIndex];
			smallest = mn.meshComponent.MaxCoord().x - mn.meshComponent.MinCoord().x < smallest ?
				mn.meshComponent.MaxCoord().x - mn.meshComponent.MinCoord().x : smallest;
			smallest = mn.meshComponent.MaxCoord().y - mn.meshComponent.MinCoord().y < smallest ?
			mn.meshComponent.MaxCoord().y - mn.meshComponent.MinCoord().y : smallest;
			smallest = mn.meshComponent.MaxCoord().z - mn.meshComponent.MinCoord().z < smallest ?
			mn.meshComponent.MaxCoord().z - mn.meshComponent.MinCoord().z : smallest;
			pos = mn.meshComponent.boundingBox.center.pos;
			inputIndex = (inputIndex + 1) % inputs.Count;

			Vector3d inputPos = pos;
			inputPos.y += 0.05; 
			smallest /= 2;
			double ratio = smallest / (this.meshView.trackball.MaxCoord() - this.meshView.trackball.MinCoord()).Length();
			Vector3d scale = new Vector3d(ratio, ratio, ratio);
			Matrix4d scaleMat = Matrix4d.ScalingMatrix(scale);
			this.meshView.trackball.Transform(scaleMat);
			Vector3d center = (this.meshView.trackball.MaxCoord() + this.meshView.trackball.MinCoord()) / 2;
			Vector3d translation = inputPos - center;
			Matrix4d transMat = Matrix4d.TranslationMatrix(translation);
			this.meshView.trackball.Transform(transMat);
			this.meshView.Refresh();
		}		

		private void importMeshToolStripMenuItem_Click(object sender, EventArgs e)
		{
			string foldername = @"F:\Projects\GraphEditing\models\raceTrack\output\iter_1_race_track";
				//@"F:\Projects\GraphEditing\models\trackingballs\synthesis\symmetry_2";
				//@"F:\Projects\GraphEditing\models\raceTrack\iter1_race_track01";
               // @"F:\Projects\GraphEditing\models\raceTrack\race_track01";
			//string foldername = @"F:\Projects\GraphEditing\models\subgraphSequence\trackingballs\mesh\trackingball02_1_1";
			//@"F:\Projects\GraphEditing\models\input format\for_han\playground01";

			//FolderBrowserDialog dialog = new FolderBrowserDialog();
			//dialog.SelectedPath = @"F:\Projects\GraphEditing\models";
			//DialogResult ret = dialog.ShowDialog(this);
			//if (ret == DialogResult.OK)
			//{
			//    string foldername = dialog.SelectedPath;
			this.meshView.LoadComponentDetail(foldername, 2);

			//this.meshView.drawModelGraph = true;
			this.meshView.Refresh();
			//}
		}


		private void correctEdgeFaceIdx_Click(object sender, EventArgs e)
		{
			//this.SetMode(MeshView.EnumOperationMode.SelectBoundingboxFace);
			this.meshView.SelectBoxFacesByComponent();
			this.Refresh();
		}

		private void findContactFace_Click(object sender, EventArgs e)
		{
			this.meshView.CalculateContactFace();
			this.meshView.Refresh();
		}

		private void correctContactToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.meshView.CorrectContactFaceIndex();
			this.Refresh();
		}

		private void saveContactToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.meshView.SaveContactInfo(null);
		}

		private void a_nextROI_Click(object sender, EventArgs e)
		{
			int npairs = 0;
			if (this.meshView.sourceGraph != null)
			{
				if (this.meshView.sourceGraph.equSubgraphs != null)
				{
					npairs = this.meshView.sourceGraph.equSubgraphs.Count;
					this.sourceGraphView.cutPairs = this.meshView.sourceGraph.equCutpairs;
					
				}
				if (npairs == 0)
					this.meshView.mcIdx = -1;
				else
					this.meshView.mcIdx = (this.meshView.mcIdx + 1) % npairs;
				if (this.meshView.mcIdx == npairs - 1)
					MessageBox.Show("End of ROI Now.");
				this.sourceGraphView.mcIdx = this.meshView.mcIdx;
				this.meshView.SetComponentView(0);
			}

			this.meshView.Refresh();
			this.sourceGraphView.Refresh();
			this.sourceComponentView.Refresh();
		}

		private void b_nextROI_Click(object sender, EventArgs e)
		{
			int npairs = 0;
			if (this.meshView.targetGraph != null)
			{
				if (this.meshView.targetGraph.equSubgraphs != null)
				{
					npairs = this.meshView.targetGraph.equSubgraphs.Count;
					this.targetGraphView.cutPairs = this.meshView.targetGraph.equCutpairs;
				}
				if (npairs == 0)
					this.meshView.mcIdx = -1;
				else
					this.meshView.mcIdx = (this.meshView.mcIdx + 1) % npairs;
				if (this.meshView.mcIdx == npairs - 1)
					MessageBox.Show("End of ROI Now.");
				this.targetGraphView.mcIdx = this.meshView.mcIdx;
				this.meshView.SetComponentView(1);
			}

			this.meshView.Refresh();
			this.targetGraphView.Refresh();
			this.targetComponentView.Refresh();
		}

       

		private void saveNewGraphFiles(string foldername)
		{
			string graphfile = foldername;
			string matfile = graphfile.Clone() as string;
            graphfile += @"graph_structure.txt";
            matfile += @"labels.txt";
            //graphfile += @"trackingball.graph";
            //matfile += @"trackingball.material";

			StreamWriter sw = new StreamWriter(matfile);
			sw.WriteLine("#node number --- node index/color index/node type index");
			sw.WriteLine(this.sourceGraphView.modelGraph.graphNodes.Count);
			foreach (ModelGraphNode mn in this.sourceGraphView.modelGraph.graphNodes)
			{
				sw.WriteLine(mn.name + " " + mn.nodeType);
			}
			sw.Close();
			Program.OutputText("saved node material file to" + matfile, true);
			sw = new StreamWriter(graphfile);
			sw.WriteLine("#edge number --- node 1/node 2");
			sw.WriteLine(this.sourceGraphView.modelGraph.graphEdges.Count);
			foreach (ModelGraphEdge me in this.sourceGraphView.modelGraph.graphEdges)
			{
				if (me.conntectingNodes.Count != 2)
				{
					MessageBox.Show("Wrong number of contacts components!");
					return;
				}
				ModelGraphNode mn1 = me.conntectingNodes[0];
				ModelGraphNode mn2 = me.conntectingNodes[1];
				sw.Write(mn1.name + " " + mn2.name + " ");
				sw.WriteLine();
			}
			sw.Close();
		}


        private void labelCycle_Click(object sender, EventArgs e)
        {
            this.meshView.Labelcycle();
            //WriteLabelFile();
        }

        private void WriteLabelFile()
        {
            string filename = @"F:\Projects\GraphEditing\models\raceTrack\raceTrack01\labels.txt";
            string path = @"F:\Projects\GraphEditing\models\raceTrack\raceTrack01\mesh\";
            StreamWriter sw = new StreamWriter(filename);
            string[] files = Directory.GetFiles(path, "*.obj");
            foreach (string file in files)
            {
                int start = file.LastIndexOf('\\')+1;
                int index = file.IndexOf('.');
                int type = -1;
                if (file.Contains("arc"))
                    type = 0;
                else if (file.Contains("base"))
                    type = 1;
                else if (file.Contains("plane") && !file.Contains("upplane"))
                    type = 2;
                else if (file.Contains("upplane"))
                    type = 3;
                sw.WriteLine(file.Substring(start, index - start)+" " +type.ToString());
            }
            sw.Close();
        }

        private List<SubgraphView> subgraphVeiws = null;
        private void self_search_Click(object sender, EventArgs e)
        {
			this.meshView.modelGraph.SearchMatchingsubgraphs(20);

            SetSubgraphView(0);

			this.meshView.selectedSGIndex = -1;

			this.replaced = false;
			this.page = 0;
			
			if (this.meshView.modelGraph.equCutpairs != null)
				npairs = this.meshView.modelGraph.equCutpairs.Count / 2;

			if(npairs==0)
				this.pageNumber.Text = (this.page).ToString() + @"/" + npairs.ToString();
			else
				this.pageNumber.Text = (this.page + 1).ToString() + @"/" + npairs.ToString();

			this.iterIndex.Text = this.meshView.iterIdx.ToString();

        }//self_search_Click

		int nSGWindows = 1;
		int npairs = 0;
        private void SetSubgraphView(int page)
        {
            ModelGraph mg = this.meshView.modelGraph;
            if (mg.equCutpairs != null)
            {
                npairs = mg.equCutpairs.Count / 2;
            }

            this.functionTab.SelectedIndex = 0;
            
            int col_width = this.subgraphViewPanel.Width-15;
			int height = col_width;
			int width = col_width;
			Point startLoc = this.subgraphViewPanel.Location;
            String camPath = mg.meshfile + @"cam.cam";

			int start = nSGWindows * page;

			if (page == 0) start = 0;

            if(this.subgraphVeiws == null || this.subgraphVeiws.Count == 0)
            { 
                this.subgraphVeiws = new List<SubgraphView>(); 

					for (int j = 0; j < 2; ++j)
					{
						Point loc = new Point(startLoc.X, startLoc.Y + height * j);
						SubgraphView sgView = new SubgraphView();
						sgView.subStructure1 = new List<MeshComponent>();
						sgView.subStructure2 = new List<MeshComponent>();
						sgView.viewIndex = page * 2 + j;
						
						this.subgraphVeiws.Add(sgView);

						sgView.SetBounds(loc.X, loc.Y, width, height);
						sgView.BorderStyle = BorderStyle.FixedSingle;
						if (camPath != null && File.Exists(camPath))
						{
							StreamReader sr = new StreamReader(camPath);
							sgView.LoadCamera(sr);
							sr.Close();
						}
						sgView.Anchor = (AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right);
						if (j == 1)
						{
							sgView.colorIndex = 1;
						}
						//sgView.Anchor = (AnchorStyles.Top | AnchorStyles.Bottom| AnchorStyles.Left | AnchorStyles.Right);
                        sgView.CurrentMode = SubgraphView.EnumOperationMode.Viewing;
						this.subgraphViewPanel.Controls.Add(sgView);						
						sgView.Show();
						sgView.Refresh();
					}
            }

           

				SubgraphView sgView1 = this.subgraphVeiws[0],
					sgView2 = this.subgraphVeiws[1];
				sgView1.subStructure1 = new List<MeshComponent>();
				sgView1.subStructure2 = new List<MeshComponent>();
				sgView2.subStructure1 = new List<MeshComponent>();
				sgView2.subStructure2 = new List<MeshComponent>();

				if (mg.equCutpairs != null && mg.equCutpairs.Count > 0)
				{
					foreach (ModelGraphNode mn in mg.equSubgraphs[start * 2].nodeSets)
					{
						sgView1.subStructure1.Add(mn.meshComponent);
						sgView2.subStructure2.Add(mn.meshComponent);
					}
					foreach (ModelGraphNode mn in mg.equSubgraphs[start * 2 + 1].nodeSets)
					{
						sgView2.subStructure1.Add(mn.meshComponent);
						sgView1.subStructure2.Add(mn.meshComponent);
					}
				}
				sgView1.SetTransformation(this.meshView.currTransformation);
				sgView2.SetTransformation(this.meshView.currTransformation);
			
                sgView1.Show();
                sgView1.Refresh();
				sgView2.Show();
				sgView2.Refresh();

            this.Refresh();

			if (npairs == 0)
			{
				MessageBox.Show("No non-trivial results found, or it is used for cross match.");

			}
        }//SetSubgraphView

		public void RefreshSubgraphViews(Matrix4d transformation)
		{
			for (int i = 0; i < this.subgraphVeiws.Count; ++i)
			{
				this.subgraphVeiws[i].SetTransformation(transformation);
				this.subgraphVeiws[i].Refresh();
			}
		}

        private void save_self_match_Click(object sender, EventArgs e)
        {
            String sgPath = this.meshView.modelGraph.meshfile + @"subgraphs\";
            this.meshView.ExportSelfmatchingSubgraphs(sgPath);
        }

        private void load_self_match_Click(object sender, EventArgs e)
        {
            this.subgraphVeiws = new List<SubgraphView>();
            String sgPath = this.meshView.modelGraph.meshfile + @"subgraphs\";
            string[] objfiles = Directory.GetFiles(sgPath, "*.subgraph");
            this.meshView.LoadSubgraphs(objfiles, 2);
            SetSubgraphView(0);
        }

        private void subgraphViewPanel_Paint(object sender, PaintEventArgs e)
        {
            if (this.subgraphVeiws == null) return;
            for (int i = 0; i < this.subgraphVeiws.Count; ++i)
                this.subgraphVeiws[i].Refresh();
        }

		public bool replaced = false;
        public void ReplaceASubgraph(int selectedSGIndex)
        {
			if ( !inDemo || selectedSGIndex == -1 || 
				this.meshView.modelGraph.equSubgraphs == null) return;

			if (npairs == 0) return;

			if (replaced)
			{
				MessageBox.Show("Replacement happens once at each iteration.");
				return;
			}

			bool expand = (selectedSGIndex % 2 == 0);

			selectedSGIndex /= 2;

			selectedSGIndex += page * nSGWindows;

			bool isThreeD = true;


			if (expand)
					this.meshView.ReplaceASubgraph(this.meshView.modelGraph, this.meshView.modelGraph,
					this.meshView.modelGraph.equSubgraphs[2 * selectedSGIndex],
					this.meshView.modelGraph.equSubgraphs[2 * selectedSGIndex + 1], isThreeD);
				else
					this.meshView.ReplaceASubgraph(this.meshView.modelGraph, this.meshView.modelGraph,
					this.meshView.modelGraph.equSubgraphs[2 * selectedSGIndex + 1],
					this.meshView.modelGraph.equSubgraphs[2 * selectedSGIndex], isThreeD);

            this.meshView.meshComponents = new List<MeshComponent>();
            foreach (ModelGraphNode mn in this.meshView.modelGraph.graphNodes)
                this.meshView.meshComponents.Add(mn.meshComponent);

			for (int i = 0; i < this.subgraphVeiws.Count; ++i)
				this.subgraphVeiws[i].CurrentMode = SubgraphView.EnumOperationMode.Viewing;

			if (hasIO)
			{
				this.meshView.CheckValidity();
			}
			this.meshView.selectedSGIndex = selectedSGIndex;

			this.replaced = true;

			this.meshView.Refresh();
            this.Refresh();
        }

        private void saveGraphToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string filename = @"F:\Projects\GraphEditing\models\raceTrack\graphfiles\iter_2_raceTrack.seg";
            this.meshView.SaveSegDetail(filename);
            this.meshView.SaveModelGraphInfo(filename);
            this.meshView.SaveContactInfo(filename);
        }

		private void selectSG_Click(object sender, EventArgs e)
		{
			if (this.subgraphVeiws == null) return;
			for (int i = 0; i < this.subgraphVeiws.Count; ++i)
				this.subgraphVeiws[i].CurrentMode = SubgraphView.EnumOperationMode.Select;
		}

		private void saveCycleToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.meshView.SaveCycleInfo(null);
		}

		private void saveCurrentGraphToolStripMenuItem_Click(object sender, EventArgs e)
		{
			string folder = @"F:\Projects\GraphEditing\models\playgrounds\output\playground\iter_1\graphfiles\";
			this.meshView.SaveSegDetail(folder);
			this.meshView.SaveModelGraphInfo(folder);
			this.meshView.SaveCycleInfo(folder);
		}

		private void correctGraph_Click(object sender, EventArgs e)
		{
			this.meshView.CorrectGraph();
            this.meshView.Refresh();
		}

		private int page = 0;
		private void nextPage_Click(object sender, EventArgs e)
		{
			if (page + 1 >= npairs) return;
			++page;
			if (npairs == 0)
				this.pageNumber.Text = (this.page).ToString() + @"/" + npairs.ToString();
			else
				this.pageNumber.Text = (this.page + 1).ToString() + @"/" + npairs.ToString();
			this.SetSubgraphView(page);
		}

		private void prevPage_Click(object sender, EventArgs e)
		{
			if (page - 1 < 0) return;
			--page;
			if (npairs == 0)
				this.pageNumber.Text = (this.page).ToString() + @"/" + npairs.ToString();
			else
				this.pageNumber.Text = (this.page + 1).ToString() + @"/" + npairs.ToString();
			this.SetSubgraphView(page);
		}

		private void castleToolStripMenuItem_Click(object sender, EventArgs e)
		{
			// load mesh model
			string fullPath = @"F:\Projects\GraphEditing\models\castle\castle03.obj";
			StreamReader sr = new StreamReader(fullPath);
			Mesh m = new Mesh(sr, "obj", true);
			sr.Close();

			// add new tabpage and mesh record
			MeshRecord rec = new MeshRecord(m);
			rec.Filename = fullPath;// filename;
			rec.objName = this.objectName;

			this.meshView.SetModel(rec);

			// try loading segmentation file
			if (!meshView.LoadSegDetail())
			{
				this.meshView.meshComponents = this.meshView.DividMeshIntoCompnents(m);
				// if #components is large, merge manually
				//if (this.meshView.meshComponents != null && this.meshView.meshComponents.Count < 2000)
				//    this.meshView.MergeCoherentPatches();
				this.meshView.GetComponentMeshes();
			}
			else
			{
				this.meshView.DecomposeBySeg();
			}

			this.meshView.drawModelGraph = true;

			this.meshView.modelGraph = this.meshView.LoadModelGraphInfo(0);
			if (this.meshView.modelGraph.meshfile.Contains("ball"))
				this.meshView.modelGraph.CalculateNodeLvelByDirections();
			this.meshView.mcIdx = -1;
			this.meshView.iterIdx = 0;
			fullPath = @"F:\Projects\GraphEditing\models\castle\castle.cam";
			sr = new StreamReader(fullPath);

			this.meshView.LoadCamera(sr);
			sr = new StreamReader(fullPath);
			sr.Close();

			this.meshView.Refresh();


			inputIndex = 0;

			this.meshView.drawGroundGrids = !this.meshView.drawGroundGrids;
			

			this.meshView.Refresh();

		}

		private void button4_Click(object sender, EventArgs e)
		{
			int[] iters = new int[8] { 6, 1, 25, 0, 7, 8, 50, 28};
			//int[] sum_iters = new int[8] {  };
			string path = @"F:\Projects\GraphEditing\models\castle\iterations\";
			string camPath = @"F:\Projects\GraphEditing\models\castle\castle.cam";
			StreamReader cam_sr;
			inputIndex = 0;

			int num = 8;

			for (int i = 0; i < num; ++i)
			{
				string filename = path + @"iter_0" + (i+1).ToString() + @".obj";
				StreamReader sr = new StreamReader(filename);
				Mesh m = new Mesh(sr, "obj", true);
				sr.Close();

				// add new tabpage and mesh record
				MeshRecord rec = new MeshRecord(m);
				rec.Filename = filename;// filename;
				rec.objName = this.objectName;

				this.meshView.SetModel(rec);

				if (!meshView.LoadSegDetail())
				{
					this.meshView.meshComponents = this.meshView.DividMeshIntoCompnents(m);
					// if #components is large, merge manually
					//if (this.meshView.meshComponents != null && this.meshView.meshComponents.Count < 2000)
					//    this.meshView.MergeCoherentPatches();
					this.meshView.GetComponentMeshes();
				}
				else
				{
					this.meshView.DecomposeBySeg();
				}
				this.meshView.modelGraph = this.meshView.LoadModelGraphInfo(0);

				cam_sr = new StreamReader(camPath);
				this.meshView.LoadCamera(cam_sr);
				cam_sr.Close();

				this.showIteration.Text = @"Iteration " + (i + 1).ToString() + @": Subgraph#"
					+ iters[i].ToString();

				this.meshView.Refresh();
				this.Refresh();
			}

		}

		private bool inDemo = false;
		private void openDemoToolStripMenuItem_Click(object sender, EventArgs e)
		{
			string fullPath = @".\data\mesh.obj";
			StreamReader sr = new StreamReader(fullPath);
			Mesh m = new Mesh(sr, "obj", true);
			sr.Close();

			// add new tab page and mesh record
			MeshRecord rec = new MeshRecord(m);
			rec.Filename = fullPath;// filename;
			rec.objName = this.objectName;

			this.meshView.SetModel(rec);

			// try loading segmentation file
			if (!meshView.LoadSegDetail())
			{
				this.meshView.meshComponents = this.meshView.DividMeshIntoCompnents(m);
				this.meshView.GetComponentMeshes();
			}
			else
			{
				this.meshView.DecomposeBySeg();
			}

			this.meshView.drawModelGraph = true;

			this.meshView.modelGraph = this.meshView.LoadModelGraphInfo(0);

			this.meshView.modelGraph.CalculateNodeLvelByDirections();
			this.meshView.mcIdx = -1;
			this.meshView.iterIdx = 0;
			fullPath = @".\data\graphfiles\cam.cam";
			if (File.Exists(fullPath))
			{
				sr = new StreamReader(fullPath);

				this.meshView.LoadCamera(sr);
				sr = new StreamReader(fullPath);
				sr.Close();
			}

			inputIndex = -1;
			inDemo = true;
			this.functionTab.SelectedIndex = 0;

			this.crossMatchViewPanel.Hide();
			this.meshView.Show();

			this.meshView.Refresh();
		}

		private void openModelToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.LoadMesh(true);
			inDemo = false;
			this.crossMatchViewPanel.Hide();
			this.meshView.Show();
			this.functionTab.SelectedIndex = 0;
		}

		private void bikeToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.meshView.Hide();
			this.crossMatchViewPanel.Show();
			this.selectAModelType(0);
			this.functionTab.SelectedIndex = 2;
		}

		private void playgroundToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.meshView.Hide();
			this.crossMatchViewPanel.Show();
			this.selectAModelType(1);
			this.functionTab.SelectedIndex = 2;
		}

		private void toyTractorToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.meshView.Hide();
			this.crossMatchViewPanel.Show();
			this.selectAModelType(2);
			this.functionTab.SelectedIndex = 2;
		}

		private void nextPage_cross_Click(object sender, EventArgs e)
		{
			this.meshView.mcIdx = (this.meshView.mcIdx+1)%npairs;
			this.UpdatePageCross();
		}

		private void prevPage_cross_Click(object sender, EventArgs e)
		{
			this.meshView.mcIdx = (this.meshView.mcIdx - 1 + npairs) % npairs;
			this.UpdatePageCross();
		}

		private void UpdatePageCross()
		{
			this.meshView.SetComponentView(0);
			this.meshView.SetComponentView(1);
			this.page_cross.Text = (this.meshView.mcIdx + 1).ToString() + "/" + npairs.ToString();
			this.sourceComponentView.Refresh();
			this.targetComponentView.Refresh();
		}

		

	} // class
} // namespace
