﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using Tao.OpenGl;
using Tao.Platform.Windows;
using MyGeometry;
using System.IO;

using System.Diagnostics;
using System.Runtime.InteropServices;



using MyCholmodSolver;

using HeuristicLab.Analysis;
using HeuristicLab.Data;

using MRFWrapper;

using System.Threading;
using System.ComponentModel;

using Accord.Statistics.Analysis;

using NumericalRecipes;


namespace ShapeGraph
{
	[TypeConverterAttribute(typeof(DeformerConverter))]


	public unsafe class SubgraphView : SimpleOpenGlControl
	{

		[DllImport("opengl32", EntryPoint = "wglUseFontBitmaps", CallingConvention=CallingConvention.Winapi)]
		public static extern bool wglUseFontBitmaps(
		IntPtr hDC,
		[MarshalAs(UnmanagedType.U4)] UInt32 first,
		[MarshalAs(UnmanagedType.U4)] UInt32 count,
		[MarshalAs(UnmanagedType.U4)] UInt32 listBase
		);

		[DllImport("GDI32.DLL", EntryPoint = "SelectObject",
		CallingConvention = CallingConvention.Winapi)]
		public static extern IntPtr SelectObject(
		[In] IntPtr hDC,
		[In] IntPtr font
		);


		public interface IMeshDisplay
		{
			void Display();
			void SetData();
		}
		public enum EnumOperationMode { Viewing, Select};
		public delegate void OnPickingHandler(object sender, int index);


		// private fields
		private EnumOperationMode currentMode = EnumOperationMode.Viewing;

		// font
		//private bool initFont = false;
		public UInt32 fontBase = 0;
		private void BuildFont(PaintEventArgs pe)
		{
			IntPtr dc = pe.Graphics.GetHdc();
			IntPtr oldFontH = IntPtr.Zero;
			System.Drawing.Font font =
				new Font(
				"Arial",
				10F,
				System.Drawing.FontStyle.Bold,
				System.Drawing.GraphicsUnit.Point,
				((System.Byte)(0)));

			fontBase = (uint)Gl.glGenLists(128);

			IntPtr fontH = font.ToHfont();
			oldFontH = SelectObject(dc, fontH);

			bool ret = wglUseFontBitmaps(
				dc,
				0,
				128,
				fontBase);

			SelectObject(dc, oldFontH);						// Selects The Font We Want

			pe.Graphics.ReleaseHdc(dc);

			if (!ret) throw new Exception();
		}

		// display properties
		private Color pointColor = Color.Red;
		private Color lineColor = Color.Black;
		public static int DEFAULTCOLORINDEX = 14;

		public Rectangle screenCapRect = new Rectangle(0,0,800,600);

        private ModelGraph modelGraph = null;
		public bool ShowCuboidComponent = false;
		public bool ShowComponent = false;
		public bool showWireFrame = false;
		public bool ShowSolidMesh = false;

		public static Color[] colorMall = null;
		public static float[][] materialMall = null;

        public int viewIndex = -1;
		private Vector2d currMousePosition = new Vector2d();
		private Vector2d mouseDownPosition = new Vector2d();
		private bool isMouseDown = false;

		protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs e)
		{
			base.OnMouseDown(e);

			this.currMousePosition = new Vector2d(e.X, this.Height - e.Y);
			this.mouseDownPosition = currMousePosition;
			this.isMouseDown = true;

			switch (this.currentMode)
			{
				case EnumOperationMode.Viewing:
					switch (e.Button)
					{
						case MouseButtons.Left: ball.Click(currMousePosition, Trackball.MotionType.Rotation); break;
						case MouseButtons.Middle: ball.Click(currMousePosition / this.scaleRatio, Trackball.MotionType.Pan); break;
						case MouseButtons.Right: ball.Click(currMousePosition, Trackball.MotionType.Scale); break;
					}
					break;
				default: break;
			}
		}
		protected override void OnMouseMove(System.Windows.Forms.MouseEventArgs e)
		{
			base.OnMouseMove(e);

			this.currMousePosition = new Vector2d(e.X, this.Height - e.Y);

			switch (this.currentMode)
			{
				case EnumOperationMode.Viewing:
					if (isMouseDown)
					{
						switch (e.Button)
						{
							case MouseButtons.Left: ball.Drag(currMousePosition); break;
							case MouseButtons.Middle: ball.Drag(currMousePosition / this.scaleRatio); break;
							case MouseButtons.Right: ball.Drag(currMousePosition); break;
						}
						this.Refresh();
					}
					break;
				default: break;
			}
		}


        protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs e)
        {
            base.OnMouseUp(e);

			this.currMousePosition = new Vector2d(e.X, this.Height - e.Y);

			switch (this.currentMode)
			{
				case EnumOperationMode.Viewing:
					if (currMousePosition == mouseDownPosition) break;
					Matrix4d m = ball.GetMatrix();
					this.currTransformation = m * this.currTransformation;
					Program.GetFormMain().RefreshSubgraphViews(this.currTransformation);
					this.ball.End();
					this.Refresh();
					break;

				case EnumOperationMode.Select:
					Program.GetFormMain().ReplaceASubgraph(this.viewIndex);
					break;

				default:
					break;
			}
        }

		public void SetTransformation(Matrix4d transformation)
		{
			this.currTransformation = new Matrix4d(transformation);
		}

		public void InitialRandomColors(int K)
		{
			Random rand = new Random();
			colorMall = new Color[K];
			for (int i = 0; i < K; ++i)
			{
				colorMall[i] = Color.FromArgb(
					rand.Next(0, 255),
					rand.Next(0, 255),
					rand.Next(0, 255)
				);
			}

			// 8-data, color scheme --add to 10--2013-12-27
			if (true)
			{
				int off_set = 0;
				colorMall[0 + off_set] = Color.FromArgb(153,255,153);
				colorMall[1 + off_set] = Color.FromArgb(255,153,153);
				colorMall[2 + off_set] = Color.FromArgb(255, 255, 51);
				colorMall[3 + off_set] = Color.FromArgb(152, 78, 163);
				colorMall[4 + off_set] = Color.FromArgb(219, 147, 112);
				colorMall[5 + off_set] = Color.FromArgb(255, 255, 51);
				colorMall[6 + off_set] = Color.FromArgb(193,255,193);
				colorMall[7 + off_set] = Color.FromArgb(52, 68, 211);
				colorMall[8 + off_set] = Color.FromArgb(255, 182, 193);
				colorMall[9 + off_set] = Color.FromArgb(122, 221, 173);
			}

			colorMall[DEFAULTCOLORINDEX] = Color.Gray;

		}

		public void InitialRandomMaterials(int K)
		{
			Random rand = new Random();
			materialMall = new float[K][];
			for (int i = 0; i < K; ++i)
			{
				int j = i%K;
				materialMall[i] = new float[4] {
					colorMall[j].R / 255.0f,
					colorMall[j].G / 255.0f,
					colorMall[j].B / 255.0f,
					1.0f
				};
			}
		}

		// for viewing
		private Trackball ball = new Trackball(100, 100);
		private Trackball editball = new Trackball(100, 100);
		private double scaleRatio;

		public bool showModel = true;
		public int colorIndex = 0;

		private Matrix4d currTransformation = Matrix4d.IdentityMatrix();
		

		public EnumOperationMode CurrentMode
		{
			get { return currentMode; }
			set { currentMode = value; }
		}

		public SubgraphView()
		{			
			this.AutoMakeCurrent = true;
			this.InitializeContexts();
			this.InitOpenGL();
			this.InitialRandomColors(53);
			this.InitialRandomMaterials(53);
			foreach (Control aControl in this.Controls)
			{
				aControl.MouseDoubleClick += new MouseEventHandler(SubgraphView_MouseDoubleClick);
			}
			this.MouseDoubleClick += new System.Windows.Forms.MouseEventHandler(SubgraphView_MouseDoubleClick);
		}
		private void InitializeComponent()
		{
			this.SuspendLayout();
			// 
			// SubgraphView
			// 
		//	this.Name = "SubgraphView";
			this.ResumeLayout(false);
		}
		protected override void OnSizeChanged(EventArgs e)
		{
			base.OnSizeChanged(e);

			Size s = this.Size;
			if (s.Width == 0 || s.Height == 0) return;

			this.scaleRatio = (s.Width > s.Height) ? s.Height : s.Width;
		//	this.InitMatrix();
			
			this.ball.SetBounds(s.Width * 1.0, s.Height * 1.0);
			this.editball.SetBounds(s.Width * 1.0, s.Height * 1.0);

			this.Refresh();
		}

		public bool drawmodel = false;
		protected override void OnPaint(PaintEventArgs e)
		{
			//if (!initFont)
			//{
			//    BuildFont(e);
			//    initFont = true;
			//}


			if (this.DesignMode == true)
			{
				base.OnPaint(e); 
				return;
			}

			this.MakeCurrent();

			Gl.glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
			Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);

			
			// draw 3d scene
			this.Render3dScene();

			this.SwapBuffers();
		}

		private void SubgraphView_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				Program.GetFormMain().ReplaceASubgraph(this.viewIndex);
				this.Refresh();
			}
		}

		private void Render3dGraph()
		{

			this.InitMatrix();

			// -------------------------------------------------------------------------
			// draw 3d sense
			Matrix4d m = ball.GetMatrix() * currTransformation;

			Gl.glMatrixMode(Gl.GL_MODELVIEW);

			Gl.glPushMatrix();
			Gl.glMultMatrixd(m.Transpose().ToArray());

			Gl.glEnable(Gl.GL_POLYGON_OFFSET_FILL);

            this.DrawMatchedModelGraph();

			Gl.glPopMatrix();

			Gl.glPopMatrix();
		}
		
		private void Render2dScene()
		{
			Gl.glPushMatrix();

			Gl.glViewport(0, 0, this.Size.Width, this.Size.Height);

			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glLoadIdentity();

			Glu.gluOrtho2D(0, this.Size.Width, this.Size.Height, 0);

			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glLoadIdentity();

			Gl.glPushMatrix();

			Gl.glColor3f(1.0f, 1.0f, 1.0f);

			DrawModelGraph();

			Gl.glPopMatrix();

			Gl.glPopMatrix();

		}
		
		private void Render3dScene()
		{

			this.InitMatrix();

			// -------------------------------------------------------------------------
			// draw 3d sense
			Matrix4d m = ball.GetMatrix() * currTransformation;


			Gl.glMatrixMode(Gl.GL_MODELVIEW);

			Gl.glPushMatrix();
			Gl.glMultMatrixd(m.Transpose().ToArray());

			Gl.glEnable(Gl.GL_POLYGON_OFFSET_FILL);
			
			// draw here --

			if (this.subStructure1 != null)
			{
                foreach (MeshComponent mc in this.subStructure1)
				{
					//if (!this.ShowSolidMesh)
					//    this.DrawTransparent(mc.mesh, colorMall[0]);
					//else
					//this.DrawTransparent(mc.mesh, colorMall[colorIndex]);
					this.DrawFlatShaded(mc.mesh, colorMall[colorIndex]);
				}
			}
			//if (this.subStructure2 != null)
			//{
			//    foreach (MeshComponent mc in this.subStructure2)
			//    {
			//        if (!this.ShowSolidMesh)
			//            this.DrawTransparent(mc.mesh, colorMall[1]);
			//        else
			//            this.DrawFlatShaded(mc.mesh, colorMall[1]);
			//    }
			//}


			Gl.glPopMatrix();
			
			Gl.glPopMatrix();
		}


		private bool useGLOrthoProjection = false;
		private OpenGLProjector glProjector = null;
		public void ObtainGLProjector()
		{
			if (!this.useGLOrthoProjection)
			{
				Gl.glPushMatrix();
				Matrix4d mat = ball.GetMatrix() * currTransformation;
				Gl.glMultMatrixd(mat.Transpose().ToArray());
				this.glProjector = new OpenGLProjector();
				Gl.glPopMatrix();
			}
			else
			{
				this.glProjector = new OpenGLProjector();
			}
		}
		public void ScreenCaptureDefault()
		{
			this.ScreenCapture(this.screenCapRect, @"d:\\cap1.jpg");
		}
		public void ScreenCapture(Rectangle rect, string file)
		{
			int w = rect.Width, h = rect.Height;

			w -= w % 4; h -= h % 4;

			this.Refresh();
			this.Refresh();

			// generate suggestion images
			byte[] bytes = new byte[w * h * 3];


			Gl.glReadPixels(rect.X, rect.Y, w, h, Gl.GL_BGR, Gl.GL_UNSIGNED_BYTE, bytes);


			//Image<Bgr, Byte> image = new Image<Bgr, Byte>(w, h); //specify the width and height here
			//image.Bytes = bytes; //your byte array
			//image = image.Flip(Emgu.CV.CvEnum.FLIP.VERTICAL);

			//image.Save(file);

			Program.OutputText("saved screen cap file to " + file, true);
		}

		private void InitOpenGL()
		{
            // OpenGl.GL state
            Gl.glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
            Gl.glDisable(Gl.GL_DITHER);
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            Gl.glEnable(Gl.GL_POINT_SMOOTH);
            Gl.glEnable(Gl.GL_LINE_SMOOTH);


            // Material
            SetDefaultMaterial();


            // Lighting
            Gl.glLoadIdentity();
            SetDefaultLight();


            // Fog
            float[] fogColor = new float[] { 0.3f, 0.3f, 0.4f, 1.0f };

            Gl.glFogi(Gl.GL_FOG_MODE, Gl.GL_LINEAR);
            Gl.glFogfv(Gl.GL_FOG_COLOR, fogColor);
            Gl.glFogf(Gl.GL_FOG_DENSITY, 0.35f);
            Gl.glHint(Gl.GL_FOG_HINT, Gl.GL_DONT_CARE);
            Gl.glFogf(Gl.GL_FOG_START, 5.0f);
            Gl.glFogf(Gl.GL_FOG_END, 25.0f);


            Gl.glClearColor(0, 0, 0, 0);
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();
            Gl.glOrtho(0.0, 1.0, 0.0, 1.0, -1.0, 1.0);
		}
        private void SetDefaultLight()
        {
            float[] pos1 = new float[4] { 0.1f, 0.1f, -0.02f, 0.0f };
            float[] pos2 = new float[4] { -0.1f, 0.1f, -0.02f, 0.0f };
            float[] pos3 = new float[4] { 0.0f, 0.0f, 0.1f, 0.0f };
            float[] col1 = new float[4] { 0.7f, 0.7f, 0.8f, 1.0f };
            float[] col2 = new float[4] { 0.8f, 0.7f, 0.7f, 1.0f };
            float[] col3 = new float[4] { 1.0f, 1.0f, 1.0f, 1.0f };


            Gl.glEnable(Gl.GL_LIGHT0);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_POSITION, pos1);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, col1);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_SPECULAR, col1);


            Gl.glEnable(Gl.GL_LIGHT1);
            Gl.glLightfv(Gl.GL_LIGHT1, Gl.GL_POSITION, pos2);
            Gl.glLightfv(Gl.GL_LIGHT1, Gl.GL_DIFFUSE, col2);
            Gl.glLightfv(Gl.GL_LIGHT1, Gl.GL_SPECULAR, col2);


            Gl.glEnable(Gl.GL_LIGHT2);
            Gl.glLightfv(Gl.GL_LIGHT2, Gl.GL_POSITION, pos3);
            Gl.glLightfv(Gl.GL_LIGHT2, Gl.GL_DIFFUSE, col3);
            Gl.glLightfv(Gl.GL_LIGHT2, Gl.GL_SPECULAR, col3);

        }
        private void SetDefaultMaterial()
        {
            float[] mat_a = new float[4] { 0.1f, 0.1f, 0.1f, 1.0f };
            float[] mat_d = { 0.7f, 0.7f, 0.5f, 1.0f };
            float[] mat_s = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] shine = { 120.0f };

            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_AMBIENT, mat_a);
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_DIFFUSE, mat_d);
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_SPECULAR, mat_s);
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_SHININESS, shine);

        }
		private void InitMatrix_Ortho()
		{
			Size s = this.Size;
			double w = s.Width;
			double h = s.Height;

		//	this.MakeCurrent();
			
			Gl.glViewport(0, 0, s.Width, s.Height);
			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glLoadIdentity();

			if (w > h)
			{
				double ratio = (w / h) / 2.0;
				Gl.glOrtho(-ratio, ratio, -0.5, 0.5, -100, 100);
			}
			else
			{
				double ratio = (h / w) / 2.0;
				Gl.glOrtho(-0.5, 0.5, -ratio, ratio, -100, 100);
			}

			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glLoadIdentity();
		}
		private void InitMatrix()
		{
			double w = Size.Width;
			double h = Size.Height;
			Gl.glViewport(0, 0, Size.Width, Size.Height);
			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glLoadIdentity();
			
			Glu.gluPerspective(60.0, w / h, 0.01, 100);

			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glLoadIdentity();

			//Glu.gluLookAt(0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
			Glu.gluLookAt(0.3, 0.5, 2.0, 0.3, 0.0, 0.0, 0.0, 1.0, 0.0);
		}


		private void DrawWireframe(Mesh m, Color c)
		{
			if (m == null) return;

			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_LINE);
			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glColor3ub(c.R, c.G, c.B);

			Gl.glLineWidth(1.0f);
			Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
			fixed (double* vp = m.VertexPos)
			fixed (int* index = m.FaceIndex)
			{
				Gl.glVertexPointer(3, Gl.GL_DOUBLE, 0, new IntPtr(vp));
				Gl.glDrawElements(Gl.GL_TRIANGLES, m.FaceCount * 3, Gl.GL_UNSIGNED_INT, new IntPtr(index));
			}
			Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_LIGHTING);
		}

		public bool drawGroundGrids = true;
		private void DrawFlatShaded(Mesh m, Color c)
		{
            if (m == null) return;

			
            float[] mat = new float[4] { c.R / 255f, c.G / 255f, c.B / 255f, 1.0f };
			//Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_DIFFUSE, mat);
			Gl.glEnable(Gl.GL_CULL_FACE);
			//Gl.glDisable(Gl.GL_CULL_FACE);

			Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);
			//Gl.glEnable(Gl.GL_POINT_SMOOTH);
			//Gl.glEnable(Gl.GL_LINE_SMOOTH);
			//Gl.glHint(Gl.GL_POINT_SMOOTH_HINT, Gl.GL_NICEST);
			//Gl.glHint(Gl.GL_LINE_SMOOTH_HINT, Gl.GL_NICEST);

            fixed (double* vp = m.VertexPos)
            fixed (double* np = m.FaceNormal)
            fixed (int* fi = m.FaceIndex)
            {
                Gl.glBegin(Gl.GL_TRIANGLES);
                for (int i = 0, j = 0; i < m.FaceCount; i++, j += 3)
                {
                    Gl.glNormal3dv(new IntPtr(np + j));
                    Gl.glVertex3dv(new IntPtr(vp + fi[j] * 3));
                    Gl.glVertex3dv(new IntPtr(vp + fi[j + 1] * 3));
                    Gl.glVertex3dv(new IntPtr(vp + fi[j + 2] * 3));
                }
                Gl.glEnd();
            }
			//Gl.glDisable(Gl.GL_LINE_SMOOTH);
			//Gl.glDisable(Gl.GL_POINT_SMOOTH);
			
			Gl.glDisable(Gl.GL_NORMALIZE);
            Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glDisable(Gl.GL_CULL_FACE);
			//Gl.glEnable(Gl.GL_CULL_FACE);
		
		}
		private int[] faceClusterIndex = null;
		private void DrawFlatShaded_Color(Mesh m)
		{
			if (m == null || this.faceClusterIndex == null || this.faceClusterIndex.Length != m.FaceCount) return;

			Gl.glShadeModel(Gl.GL_FLAT);
			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);
			Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);

			//Color c = this.meshColor;
			
			fixed (double* vp = m.VertexPos)
			fixed (double* np = m.FaceNormal)
			fixed (int* fi = m.FaceIndex)
			{
				Gl.glBegin(Gl.GL_TRIANGLES);
				for (int i = 0, j = 0; i < m.FaceCount; i++, j += 3)
				{
					Color c = colorMall[this.faceClusterIndex[i]];
					Gl.glColor3ub(c.R, c.G, c.B);
					Gl.glNormal3dv(new IntPtr(np + j));
					Gl.glVertex3dv(new IntPtr(vp + fi[j] * 3));
					Gl.glVertex3dv(new IntPtr(vp + fi[j + 1] * 3));
					Gl.glVertex3dv(new IntPtr(vp + fi[j + 2] * 3));
				}
				Gl.glEnd();
			}
			Gl.glDisable(Gl.GL_LIGHTING);
		}
		private void DrawSmoothShaded(Mesh m, Color c)
		{
			if (m == null) return;

			Gl.glShadeModel(Gl.GL_SMOOTH);
			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);
			Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);

			//Color c = this.meshColor;
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
			Gl.glEnableClientState(Gl.GL_NORMAL_ARRAY);
			fixed (double* vp = m.VertexPos)
			fixed (double* np = m.VertexNormal)
			fixed (int* index = m.FaceIndex)
			{
				Gl.glVertexPointer(3, Gl.GL_DOUBLE, 0, new IntPtr(vp));
				Gl.glNormalPointer(Gl.GL_DOUBLE, 0, new IntPtr(np));
				Gl.glDrawElements(Gl.GL_TRIANGLES, m.FaceCount * 3, Gl.GL_UNSIGNED_INT, new IntPtr(index));
			}
			Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
			Gl.glDisableClientState(Gl.GL_NORMAL_ARRAY);
			Gl.glDisable(Gl.GL_LIGHTING);
		}
		
		
		private void DrawTransparent(Mesh m, Color c)
		{
			if (m == null) return;

            byte alphaChannel = 120;

            Gl.glPushClientAttrib(Gl.GL_CLIENT_VERTEX_ARRAY_BIT);
            Gl.glPushAttrib(Gl.GL_ALL_ATTRIB_BITS);

            Gl.glShadeModel(Gl.GL_FLAT);
            Gl.glEnable(Gl.GL_NORMALIZE);
			Gl.glDisable(Gl.GL_LIGHTING);
            Gl.glEnable(Gl.GL_BLEND);
            Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);
            Gl.glEnable(Gl.GL_CULL_FACE);
            Gl.glDisable(Gl.GL_DEPTH_TEST);
            Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);

            Gl.glColor4ub(c.R, c.G, c.B, alphaChannel);

            Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
            Gl.glEnableClientState(Gl.GL_NORMAL_ARRAY);
            fixed (double* vp = m.VertexPos)
            fixed (double* np = m.VertexNormal)
            fixed (int* index = m.FaceIndex)
            {
                Gl.glVertexPointer(3, Gl.GL_DOUBLE, 0, (IntPtr)vp);
                Gl.glNormalPointer(Gl.GL_DOUBLE, 0, (IntPtr)np);
                Gl.glDrawElements(Gl.GL_TRIANGLES, m.FaceCount * 3, Gl.GL_UNSIGNED_INT, (IntPtr)index);
            }
            Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
            Gl.glDisableClientState(Gl.GL_NORMAL_ARRAY);

            //////////////////////////////////////////////////////////////////////////             

			//Gl.glEnable(Gl.GL_LIGHTING);
			//Gl.glDepthFunc(Gl.GL_EQUAL);

			//float[] LightDiffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
			//float[] LightSpecular = { 1.0f, 1.0f, 1.0f, 1.0f };
			//Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, LightDiffuse);
			//Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_SPECULAR, LightSpecular);
			//Gl.glEnable(Gl.GL_LIGHT0);

			//float[] Ambient = { 0.7f, 0.7f, 0.7f, 0.7f };
			//float[] Diffuse = { 1.0f, 1.0f, 0.0f, 1.0f };
			//float[] SpecularRef = { 0.7f, 0.7f, 0.7f, 1.0f };

			//Gl.glColorMaterial(Gl.GL_FRONT, Gl.GL_DIFFUSE);
			//Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, Diffuse);
			//Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, Ambient);
			//Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, SpecularRef);
			//Gl.glMateriali(Gl.GL_FRONT, Gl.GL_SHININESS, 128);

			//Gl.glColor4ub(c.R, c.G, c.B, alphaChannel);

			//Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
			//Gl.glEnableClientState(Gl.GL_NORMAL_ARRAY);
			//fixed (double* vp = m.VertexPos)
			//fixed (double* np = m.VertexNormal)
			//fixed (int* index = m.FaceIndex)
			//{
			//    Gl.glVertexPointer(3, Gl.GL_DOUBLE, 0, (IntPtr)vp);
			//    Gl.glNormalPointer(Gl.GL_DOUBLE, 0, (IntPtr)np);
			//    Gl.glDrawElements(Gl.GL_TRIANGLES, m.FaceCount * 3, Gl.GL_UNSIGNED_INT, (IntPtr)index);
			//}
			//Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
			//Gl.glDisableClientState(Gl.GL_NORMAL_ARRAY);


            Gl.glDisable(Gl.GL_NORMALIZE);
            Gl.glDisable(Gl.GL_LIGHTING);
            Gl.glDisable(Gl.GL_BLEND);
            Gl.glDisable(Gl.GL_CULL_FACE);
            Gl.glEnable(Gl.GL_DEPTH_TEST);

            Gl.glPopAttrib();
            Gl.glPopClientAttrib();

		}
		
		private void DrawScreenCapRect()
		{
			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glPushMatrix();
			Gl.glLoadIdentity();
			Glu.gluOrtho2D(0, this.Width, 0, this.Height);
			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glPushMatrix();
			Gl.glLoadIdentity();

			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_LINE);
			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
			Gl.glColor3f(1.0f, 0.0f, 0.0f);
			Gl.glRectd(this.screenCapRect.X, this.screenCapRect.Y, this.screenCapRect.X + this.screenCapRect.Width,
				this.screenCapRect.Height + this.screenCapRect.Y);
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);

			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glPopMatrix();
			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glPopMatrix();
		}
        
		public void DrawCylinder(Vector3d u, Vector3d v, double radius, Color c)
		{
			double len1 = (u - v).Length();
			Vector3d dir = (v - u).Normalize();

			double r1 = radius, r2 = r1/2;
			double angle = Math.Acos(dir.z / dir.Length()) * 180 / Math.PI;
			Glu.GLUquadric quad = Glu.gluNewQuadric();

			Color ocolor = Color.SkyBlue;
			Gl.glColor3ub(ocolor.R, ocolor.G, ocolor.B);
			Gl.glPushMatrix();
			Gl.glTranslated(u.x, u.y, u.z);
			Glu.gluSphere(quad, r2, 40, 40);
			Gl.glPopMatrix();

			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glPushMatrix();
			Gl.glTranslated(u.x, u.y, u.z);
			Gl.glRotated(angle, -dir.y, dir.x, 0);
			Glu.gluCylinder(quad, r1, r1, len1, 40, 40);
			Gl.glPopMatrix();

			Gl.glPushMatrix();
			Gl.glTranslated(v.x, v.y, v.z);
			Gl.glRotated(angle, -dir.y, dir.x, 0);
			Glu.gluSphere(quad, r2, 40, 40);
			Gl.glPopMatrix();

			Glu.gluDeleteQuadric(quad);
		}
		public void DrawSphere(Vector3d u, double radius, Color c)
		{
			Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);

			Glu.GLUquadric quad = Glu.gluNewQuadric();

			//Color ocolor = c;
			//Gl.glColor3ub(ocolor.R, ocolor.G, ocolor.B);

			float[] mat = new float[4] { c.R / 255f, c.G / 255f, c.B / 255f, 1.0f };
			Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_DIFFUSE, mat);

			Gl.glPushMatrix();
			Gl.glTranslated(u.x, u.y, u.z);
			Glu.gluSphere(quad, radius, 40, 40);
			Gl.glPopMatrix();

			Glu.gluDeleteQuadric(quad);

			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glDisable(Gl.GL_NORMALIZE);
		}

	

        //////////////////////////////////////////////////////////////////////////
        // NEW_ADD
        public void SaveCamera(StreamWriter sw)
        {
            for (int i = 0; i < 4; ++i)
                for (int j = 0; j < 4; ++j)
                    sw.WriteLine(this.currTransformation[i, j]);
        }
        public void LoadCamera(StreamReader sw)
        {
            this.currTransformation = new Matrix4d();

            for (int k = 0; k < 16; ++k)
            {
                string s = sw.ReadLine();
                int i = k / 4;
                int j = k % 4;
                this.currTransformation[i, j] = double.Parse(s);
            }
        }

		// divid mesh into components, and store them
	
		public List<MeshComponent> metaComponents = null;
		public List<MeshComponent> meshComponents = null;

        public List<MeshComponent> subStructure1 = null, subStructure2 = null;

	
		public void Clear()
		{
			this.meshComponents = null;
			this.modelGraph = null;
			this.currTransformation = Matrix4d.IdentityMatrix();
			this.Refresh();
		}
		
		private void DrawLine(Vector3d u, Vector3d v, Color c)
		{
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glLineWidth(2.0f);
			Gl.glBegin(Gl.GL_LINES);
			Gl.glVertex3d(u.x, u.y, u.z);
			Gl.glVertex3d(v.x, v.y, v.z);
			Gl.glEnd();
			Gl.glLineWidth(1.0f);
		}
		private void DrawProxy3D_Flat(PolyProxy cuboid, Color c, float width)
		{
			if (cuboid == null) return;

		//	Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glDisable(Gl.GL_NORMALIZE);
			
			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);
			Gl.glColor3ub(c.R, c.G, c.B);
			for (int i = cuboid.polygons.Count - 1; i >= 0; --i)
			{
				Gl.glBegin(Gl.GL_POLYGON);
				Polygon poly = cuboid.polygons[i];
				foreach (Point3 pt in poly.points3)
				{
					Gl.glVertex3d(pt.pos.x, pt.pos.y, pt.pos.z);
				}
				Gl.glEnd();
			}
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
		//	Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);
			
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glLineWidth(width);
			Gl.glBegin(Gl.GL_LINES);
			foreach (Polygon poly in cuboid.polygons)
			{
				for (int i = 0; i < poly.points3.Count; ++i)
				{
					Point3 p = poly.points3[i];
					Point3 q = poly.points3[(i + 1) % poly.points3.Count];
					Gl.glVertex3d(p.pos.x, p.pos.y, p.pos.z);
					Gl.glVertex3d(q.pos.x, q.pos.y, q.pos.z);
				}
			}
			Gl.glEnd();
			Gl.glLineWidth(1.0f);
			Gl.glEnable(Gl.GL_LIGHTING);

		}
		private void DrawProxy3D(PolyProxy cuboid, Color c, float width)
		{
			if (cuboid == null) return;

			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glLineWidth(width);
			Gl.glBegin(Gl.GL_LINES);
			foreach (Polygon poly in cuboid.polygons)
			{
				for (int i = 0; i < poly.points3.Count; ++i)
				{
					Point3 p = poly.points3[i];
					Point3 q = poly.points3[(i + 1) % poly.points3.Count];
					Gl.glVertex3d(p.pos.x, p.pos.y, p.pos.z);
					Gl.glVertex3d(q.pos.x, q.pos.y, q.pos.z);
				}
			}
			Gl.glEnd();
			Gl.glLineWidth(1.0f);
			Gl.glEnable(Gl.GL_LIGHTING);
			
		}
		private void DrawPolygon_Transparent(Polygon plg, Color c, float width)
		{
			if (plg == null) return;

			Gl.glShadeModel(Gl.GL_FLAT);
			Gl.glDisable(Gl.GL_NORMALIZE);
			Gl.glDisable(Gl.GL_LIGHTING);

			Gl.glEnable(Gl.GL_BLEND);
			Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);

			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);

			Color temp = Color.FromArgb(40, c);
			Gl.glColor4ub(temp.R, temp.G, temp.B, temp.A);

			Gl.glBegin(Gl.GL_POLYGON);
			foreach (Point3 pt in plg.points3)
			{
				Gl.glVertex3d(pt.pos.x, pt.pos.y, pt.pos.z);
			}
			Gl.glEnd();


			// draw lines
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glHint(Gl.GL_LINE_SMOOTH, Gl.GL_NICEST);
			Gl.glColor3ub(92, 92, 92);
			Gl.glLineWidth(1.0f);
			Gl.glBegin(Gl.GL_LINES);
			for (int i = 0; i < plg.points3.Count; ++i)
			{
				Point3 p = plg.points3[i];
				Point3 q = plg.points3[(i + 1) % plg.points3.Count];
				Gl.glVertex3d(p.pos.x, p.pos.y, p.pos.z);
				Gl.glVertex3d(q.pos.x, q.pos.y, q.pos.z);
			}
			Gl.glEnd();
			Gl.glLineWidth(1.0f);
			Gl.glDisable(Gl.GL_LINE_SMOOTH);

			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);
			//	Gl.glDisable(Gl.GL_BLEND);

		}
		private void DrawProxy_Transparent(PolyProxy cuboid, Color c, float width)
		{
			if (cuboid == null) return;

			Gl.glShadeModel(Gl.GL_FLAT);
			Gl.glDisable(Gl.GL_NORMALIZE);
			Gl.glDisable(Gl.GL_LIGHTING);

			Gl.glEnable(Gl.GL_BLEND);
			Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);
			
			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);

			Color temp = Color.FromArgb(40, c);
			Gl.glColor4ub(temp.R, temp.G, temp.B, temp.A);
			for (int i = cuboid.polygons.Count - 1; i >= 0; --i)
			{
				Gl.glBegin(Gl.GL_POLYGON);
				Polygon poly = cuboid.polygons[i];
				foreach (Point3 pt in poly.points3)
				{
					Gl.glVertex3d(pt.pos.x, pt.pos.y, pt.pos.z);
				}
				Gl.glEnd();
			}


			// draw lines
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glHint(Gl.GL_LINE_SMOOTH, Gl.GL_NICEST);
			Gl.glColor3ub(92, 92, 92);
			Gl.glLineWidth(1.0f);
			Gl.glBegin(Gl.GL_LINES);
			foreach (Polygon poly in cuboid.polygons)
			{
				for (int i = 0; i < poly.points3.Count; ++i)
				{
					Point3 p = poly.points3[i];
					Point3 q = poly.points3[(i + 1) % poly.points3.Count];
					Gl.glVertex3d(p.pos.x, p.pos.y, p.pos.z);
					Gl.glVertex3d(q.pos.x, q.pos.y, q.pos.z);
				}
			}
			Gl.glEnd();
			Gl.glLineWidth(1.0f);
			Gl.glDisable(Gl.GL_LINE_SMOOTH);
			
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);
			Gl.glDisable(Gl.GL_BLEND);

			// test sample points
			//DrawPoints(cuboid.sampledPointsOnBox_visual, Color.DarkBlue);
			//DrawPoints(visContactPoints, Color.DarkBlue);
		}

		private void DrawProxy_Highlight(PolyProxy cuboid, int polyIndex, int tag)
		{
			if (cuboid == null) return;

			Color c = Color.Red;
			if (tag == 2)
				c = Color.Green;

			Gl.glShadeModel(Gl.GL_FLAT);
			Gl.glDisable(Gl.GL_NORMALIZE);
			Gl.glDisable(Gl.GL_LIGHTING);

			Gl.glEnable(Gl.GL_BLEND);
			Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);

			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);

			Color temp = Color.FromArgb(200, c);
			Gl.glColor4ub(temp.R, temp.G, temp.B, temp.A);
		
				Gl.glBegin(Gl.GL_POLYGON);
				Polygon poly = cuboid.polygons[polyIndex];
				foreach (Point3 pt in poly.points3)
				{
					Gl.glVertex3d(pt.pos.x, pt.pos.y, pt.pos.z);
				}
				Gl.glEnd();
		
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);
			Gl.glDisable(Gl.GL_BLEND);
		}

        Mesh mesh = null;
		public void MoveMesh2Ground()
		{
            if (this.mesh == null) return;

            Mesh m = this.mesh;

			double miny = double.MaxValue;
			for (int i = 0, j = 0; i < m.VertexCount; ++i, j += 3)
			{
				double y = m.VertexPos[j + 1];
				if (y < miny)
				{
					miny = y;
				}
			}
			miny += 0.5;

			for (int i = 0, j = 0; i < m.VertexCount; ++i, j += 3)
			{
				m.VertexPos[j + 1] -= (miny); // Y = -0.5 IS THE GROUND
			}

			if (this.meshComponents != null)
			foreach (MeshComponent mc in this.meshComponents)
			{
				for (int i = 0, j = 0; i < mc.vertexCount; ++i, j += 3)
				{
					mc.vertexPos[j + 1] -= miny;
				}
				mc.boundingBox.center.pos.y -= (miny);
			}

		}

		private void DrawRect(StreamWriter sw)
		{
			sw.WriteLine("100 100 moveto 100 600 lineto stroke ");
			sw.WriteLine("100 600 moveto 600 600 lineto stroke ");
			sw.WriteLine("600 600 moveto 600 100 lineto stroke ");
			sw.WriteLine("600 100 moveto 100 100 lineto stroke ");
		}

		static double gridWidth = 2;
		public int gridAxis = 1;
		private Vector3d grid_p0 = new Vector3d(-gridWidth, -0.5, gridWidth);
		private Vector3d grid_p1 = new Vector3d(gridWidth, -0.5, gridWidth);
		private Vector3d grid_p2 = new Vector3d(-gridWidth, -0.5, -gridWidth);
		private Vector3d grid_p3 = new Vector3d(gridWidth, -0.5, -gridWidth);
		private Vector3d bottomPoint = new Vector3d(0, -0.5, 0);
		//private Vector3d grid_p0 = new Vector3d(-gridWidth, 0, gridWidth);
		//private Vector3d grid_p1 = new Vector3d(gridWidth, 0, gridWidth);
		//private Vector3d grid_p2 = new Vector3d(-gridWidth, 0, -gridWidth);
		//private Vector3d grid_p3 = new Vector3d(gridWidth,0, -gridWidth);
		//private Vector3d bottomPoint = new Vector3d(0,0, 0);
		private void AdjustGridOrientation()
		{
            if (this.mesh == null) return;

			this.bottomPoint = this.mesh.MinCoord();
			Vector3d boundingBox = this.mesh.MaxCoord() - bottomPoint;
			int whichAxis = 1;
			SetGridOrientation(whichAxis);
		}
		private void SetGridOrientation(int whichAxis)
		{
			switch (whichAxis)
			{
				case 0: /// x
					{
						grid_p0 = new Vector3d(this.bottomPoint.x, -gridWidth, gridWidth);
						grid_p1 = new Vector3d(this.bottomPoint.x, gridWidth, gridWidth);
						grid_p2 = new Vector3d(this.bottomPoint.x, -gridWidth, -gridWidth);
					}
					break;
				case 1: /// y
					{
						grid_p0 = new Vector3d(-gridWidth, this.bottomPoint.y, gridWidth);
						grid_p1 = new Vector3d(gridWidth, this.bottomPoint.y, gridWidth);
						grid_p2 = new Vector3d(-gridWidth, this.bottomPoint.y, -gridWidth);
					}
					break;
				case 2: /// z
					{
						grid_p0 = new Vector3d(-gridWidth, gridWidth, this.bottomPoint.z);
						grid_p1 = new Vector3d(gridWidth, gridWidth, this.bottomPoint.z);
						grid_p2 = new Vector3d(-gridWidth, -gridWidth, this.bottomPoint.z);
					}
					break;
			}

			gridAxis = whichAxis;
		}
		public void SetModel(MeshRecord rec)
		{

			this.AdjustGridOrientation();
			this.Refresh();
		}
		public void ChangeCurrentModel()
		{
			this.AdjustGridOrientation();
			this.Refresh();
		}
		private void DrawGroundGrid()
		{
			int slices = 16;
			double dt = 2 * gridWidth / slices;
			double dz = dt;
			Gl.glDisable(Gl.GL_LIGHTING);
		//	Gl.glEnable(Gl.GL_BLEND);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glColor3ub(192, 192, 192);

			Gl.glLineWidth(1.0f);
			Gl.glBegin(Gl.GL_LINES);
			for (int i = 0; i <= slices; ++i)
			{
				switch (gridAxis)
				{
					case 0:
						{
							Gl.glVertex3d(grid_p0.x, grid_p0.y, grid_p0.z - dz * i);
							Gl.glVertex3d(grid_p1.x, grid_p1.y, grid_p1.z - dz * i);
						}
						break;
					case 1:
						{
							Gl.glVertex3d(grid_p0.x, grid_p0.y, grid_p0.z - dz * i);
							Gl.glVertex3d(grid_p1.x, grid_p1.y, grid_p1.z - dz * i);
						}
						break;
					case 2:
						{
							Gl.glVertex3d(grid_p0.x, grid_p0.y - dz * i, grid_p0.z);
							Gl.glVertex3d(grid_p1.x, grid_p1.y - dz * i, grid_p1.z);
						}
						break;
				}

			}
			Gl.glEnd();

			Gl.glBegin(Gl.GL_LINES);
			for (int i = 0; i <= slices; ++i)
			{
				switch (gridAxis)
				{
					case 0:
						{
							Gl.glVertex3d(grid_p0.x, grid_p0.y + dt * i, grid_p0.z);
							Gl.glVertex3d(grid_p2.x, grid_p2.y + dt * i, grid_p2.z);
						}
						break;
					case 1:
						{
							Gl.glVertex3d(grid_p0.x + dt * i, grid_p0.y, grid_p0.z);
							Gl.glVertex3d(grid_p2.x + dt * i, grid_p2.y, grid_p2.z);
						}
						break;
					case 2:
						{
							Gl.glVertex3d(grid_p0.x + dt * i, grid_p0.y, grid_p0.z);
							Gl.glVertex3d(grid_p2.x + dt * i, grid_p2.y, grid_p2.z);
						}
						break;
				}
			}
			Gl.glEnd();
			Gl.glEnable(Gl.GL_LIGHTING);
		}


		//********* draw *********//
		public void DrawModelGraph()
		{
			if (modelGraph.graphEdges == null || modelGraph.graphEdges.Count == 0) return;
			// draw 
			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			float radius = 6f;
			int circle_points = 100;
			float PI2 = 2.0f * 3.1416f;
			float angle = PI2 / circle_points;
			Gl.glPointSize(2.0f);
			foreach (MeshComponent mc in this.modelGraph.meshComponents)
			{
				Point3 center3 = mc.boundingBox.center;
				Color cc = colorMall[mc.colorIdx];
				Vector2d center = modelGraph.nodePos[mc.componentIndex];
				Gl.glColor3ub(cc.R, cc.G, cc.B);
				Gl.glBegin(Gl.GL_POLYGON);
				for (float ag = 0; ag < PI2; ag += angle)
				{
					Gl.glVertex2d(center.x + radius * Math.Cos(ag), center.y + radius * Math.Sin(ag));
				}
				Gl.glEnd();
			}

			Color linec = Color.DimGray;
			Gl.glColor3ub(linec.R, linec.G, linec.B);
			Gl.glLineWidth(2.0f);
			Gl.glBegin(Gl.GL_LINES);
			for (int i = 0; i < this.modelGraph.graphEdges.Count; ++i)
			{
				MeshComponent mc1 = this.modelGraph.graphEdges[i].conntectingNodes[0].meshComponent;
				MeshComponent mc2 = this.modelGraph.graphEdges[i].conntectingNodes[1].meshComponent;
				Vector2d center1 = modelGraph.nodePos[mc1.componentIndex];
				Vector2d center2 = modelGraph.nodePos[mc2.componentIndex];
				Gl.glVertex2d(center1.x, center1.y);
				Gl.glVertex2d(center2.x, center2.y);
			}
			Gl.glEnd();

			Gl.glEnable(Gl.GL_LIGHTING);
		}//end-draw model graph

		public void DrawModelGraph3D(int tag)
		{
			// tag = 0: meshView; tag = 1: graphView (small window)
			if (this.modelGraph == null || modelGraph.graphEdges == null || modelGraph.graphEdges.Count == 0) return;
			// draw 
			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			double radius = 0.015;
			if (tag == 1)
				radius = 0.02;
			// Draw nodes			
			Glu.GLUquadric quad = Glu.gluNewQuadric();
			foreach (ModelGraphNode mc in this.modelGraph.graphNodes)
			{
				Point3 center3 = mc.meshComponent.boundingBox.center;
				Color cc = colorMall[mc.nodeType];
				Gl.glColor3ub(cc.R, cc.G, cc.B);
                //if (mc.nodeIdx == this.mcIdx)
                //{
                //    Gl.glColor3ub(0, 0, 0);
                //}
				Gl.glPushMatrix();
				Gl.glTranslated(center3.pos.x, center3.pos.y, center3.pos.z);
				//Gl.glTranslated(mc.pos3D.x, mc.pos3D.y, mc.pos3D.z);
				Glu.gluSphere(quad, radius, 40, 40);
				Gl.glPopMatrix();
			}
			Glu.gluDeleteQuadric(quad);

			// Draw Edges
			Color linec = Color.Black;
			Gl.glColor3ub(linec.R, linec.G, linec.B);
			Gl.glLineWidth(4.0f);
			if (tag == 1)
				Gl.glLineWidth(2.0f);
			Gl.glBegin(Gl.GL_LINES);
			//***TEST*****// current dual edge
			ModelGraphEdge edge = null;
			//end***TEST*****// 

			foreach (ModelGraphEdge e in this.modelGraph.graphEdges)
			{
				ModelGraphNode mc1 = e.conntectingNodes[0];
				ModelGraphNode mc2 = e.conntectingNodes[1];
				//Point3 center1 = mc1.meshComponent.boundingBox.center;
				//Point3 center2 = mc2.meshComponent.boundingBox.center;
				//***TEST*****// 
				if ((edge != null && e == edge) || e.start != null)
				{
					Gl.glColor3ub(255, 0, 0);
				}
				else
					Gl.glColor3ub(linec.R, linec.G, linec.B);
				//end***TEST*****// 
				Gl.glVertex3d(mc1.pos3D.x, mc1.pos3D.y, mc1.pos3D.z);
				Gl.glVertex3d(mc2.pos3D.x, mc2.pos3D.y, mc2.pos3D.z);
			}
			Gl.glEnd();


			Gl.glEnable(Gl.GL_LIGHTING);
		}//end-draw model graph

		public void DrawMatchedModelGraph()
		{
			//if (this.sourceGraph == null || this.targetGraph == null) return;
			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			double radius = 0.015;
			// Draw nodes			
			List<ModelGraphNode> labeledNodes = new List<ModelGraphNode>();

			Glu.GLUquadric quad = Glu.gluNewQuadric();
			foreach (ModelGraphNode mc in this.modelGraph.graphNodes)
			{
				//Point3 center3 = mc.meshComponent.boundingBox.center;
				if (mc.nodeType == -1)
					mc.nodeType = 0;
				Color cc = colorMall[mc.nodeType];
				Gl.glColor3ub(cc.R, cc.G, cc.B);
				Gl.glPushMatrix();
				//Gl.glTranslated(center3.pos.x, center3.pos.y, center3.pos.z);
				Gl.glTranslated(mc.pos3D.x, mc.pos3D.y, mc.pos3D.z);
				//if(labeledNodes.Contains(mc))
				//    Glu.gluSphere(quad, 0.03, 40, 40);
				//else
					Glu.gluSphere(quad, radius, 40, 40);
				Gl.glPopMatrix();
			}
			Glu.gluDeleteQuadric(quad);
			// Draw Edges		

	
			Color linec = Color.Black;
			Gl.glColor3ub(linec.R, linec.G, linec.B);
			Gl.glLineWidth(1.0f);
			Gl.glBegin(Gl.GL_LINES);
			foreach (ModelGraphEdge e in this.modelGraph.graphEdges)
			{
				ModelGraphNode mc1 = e.conntectingNodes[0];
				ModelGraphNode mc2 = e.conntectingNodes[1];
				if (mc1 == null || mc2 == null)
					linec = Color.Black;
				if (e.isDummyEdge)
				{
					Color adde = Color.Gray;
					Gl.glColor3ub(adde.R, adde.G, adde.B);
				}
				else
					Gl.glColor3ub(linec.R, linec.G, linec.B);
				Gl.glVertex3d(mc1.pos3D.x, mc1.pos3D.y, mc1.pos3D.z);
				Gl.glVertex3d(mc2.pos3D.x, mc2.pos3D.y, mc2.pos3D.z);
			}
			Gl.glEnd();

			Gl.glEnable(Gl.GL_LIGHTING);
		}//end-draw model graph

		//*********end_draw *********//

    }//Class-SubgraphView

}//Namespace-ShapeGraph
