﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using Tao.OpenGl;
using Tao.Platform.Windows;
using MyGeometry;
using System.IO;

using System.Diagnostics;
using System.Runtime.InteropServices;



using MyCholmodSolver;

using HeuristicLab.Analysis;
using HeuristicLab.Data;

using MRFWrapper;

using System.Threading;
using System.ComponentModel;

using Accord.Statistics.Analysis;

using NumericalRecipes;


namespace ShapeGraph
{
	[TypeConverterAttribute(typeof(DeformerConverter))]
	public class ColorPalette
	{
		private Color[] colorMall = null;
		private MeshView view = null;
		public Color[] COLORMALL
		{
			get { return colorMall; }
			set {
				colorMall = value;
				this.view.Refresh();
			}
		}
		public ColorPalette(Color[] colormall, MeshView _view)
		{
			this.colorMall = colormall;
			this.view = _view;
		}
		public void SetColors(Color[] colormall)
		{
			this.colorMall = colormall;
		}
	}


	public unsafe class MeshView : SimpleOpenGlControl
	{

		[DllImport("opengl32", EntryPoint = "wglUseFontBitmaps", CallingConvention=CallingConvention.Winapi)]
		public static extern bool wglUseFontBitmaps(
		IntPtr hDC,
		[MarshalAs(UnmanagedType.U4)] UInt32 first,
		[MarshalAs(UnmanagedType.U4)] UInt32 count,
		[MarshalAs(UnmanagedType.U4)] UInt32 listBase
		);

		[DllImport("GDI32.DLL", EntryPoint = "SelectObject",
		CallingConvention = CallingConvention.Winapi)]
		public static extern IntPtr SelectObject(
		[In] IntPtr hDC,
		[In] IntPtr font
		);


		public interface IMeshDisplay
		{
			void Display();
			void SetData();
		}
		public enum EnumOperationMode { Viewing, FitCircle3D, SpecifyContacts, 
			ComponentGrouping, Picking, Sketching, AddRelationGraphEdge, AddEraseSlotPoints, SelectCuboidMeshComponent, 
			AssignTouchingConstraint, UpdateContact, SelectBoundingboxFace
		};
		public delegate void OnPickingHandler(object sender, int index);


		// private fields
		private MeshRecord meshRecord = null;
		private EnumOperationMode currentMode = EnumOperationMode.Viewing;

		// font
		//private bool initFont = false;
		public UInt32 fontBase = 0;
		private void BuildFont(PaintEventArgs pe)
		{
			IntPtr dc = pe.Graphics.GetHdc();
			IntPtr oldFontH = IntPtr.Zero;
			System.Drawing.Font font =
				new Font(
				"Arial",
				10F,
				System.Drawing.FontStyle.Bold,
				System.Drawing.GraphicsUnit.Point,
				((System.Byte)(0)));

			fontBase = (uint)Gl.glGenLists(128);

			IntPtr fontH = font.ToHfont();
			oldFontH = SelectObject(dc, fontH);

			bool ret = wglUseFontBitmaps(
				dc,
				0,
				128,
				fontBase);

			SelectObject(dc, oldFontH);						// Selects The Font We Want

			pe.Graphics.ReleaseHdc(dc);

			if (!ret) throw new Exception();
		}

		// display properties
		private float pointSize = 4.0f;
		private Color pointColor = Color.Red;
		private Color lineColor = Color.Black;
		public static int DEFAULTCOLORINDEX = 14;


		public int selectedFace = -1;
		public int gindex = 0;
		public int spaIndex = 0;
		public int paIndex = 0;
		public int votingIndex = 0;
		public int currVotingIndex = 0;
		public Rectangle screenCapRect = new Rectangle(0,0,800,600);
		public bool ShowSPA = false;
		public bool ShowPA = false;
		public bool ShowCuboidComponent = false;
		public bool ShowClusterColor = false;
		public bool ShowComponent = false;
		public bool showWireFrame = false;
		public bool ShowSolidMesh = false;
		public bool showOriginalComponents = false;
		public bool ShowComponentProxy = true;
		public bool fixselection = false;
		public bool proxyFitted = false;
		public ColorPalette colorpalette = null;
		public static Color[] colorMall = null;
		public static float[][] materialMall = null;
		public void InitialRandomColors(int K)
		{
            Random rand = new Random();
            colorMall = new Color[K];
            for (int i = 0; i < K; ++i)
            {
                colorMall[i] = Color.FromArgb(
                    rand.Next(0, 255),
                    rand.Next(0, 255),
                    rand.Next(0, 255)
                );
            }

            // 8-data, color scheme --add to 10--2013-12-27
            if (true)
			{
                int off_set = 0;
				int c1 = (int)(255*0.2), c2 =  (int)(255*0.5), c3 = (int)(255*0.8);
				colorMall[0 + off_set] = Color.FromArgb(117, 107, 177);
				colorMall[1 + off_set] = Color.FromArgb(153,255,204);
				colorMall[2 + off_set] = Color.FromArgb(255, 255, 153);//(c1, c2, c3);
				colorMall[3 + off_set] = Color.FromArgb(255,153,153);//(152, 78, 163);
				colorMall[4 + off_set] = Color.FromArgb(102,178,255);
				colorMall[5 + off_set] = Color.FromArgb(255,128,0);//(255, 255, 51);
				colorMall[6 + off_set] = Color.FromArgb(c1, c3, c2);//(c3, c1, c2);
				colorMall[7 + off_set] = Color.FromArgb(255, 51,51);
				colorMall[8 + off_set] = Color.FromArgb(c3, c1, c2);//(c3, c2, c1);
				colorMall[9 + off_set] = Color.FromArgb(152, 78, 163);//(122, 221, 173);
				colorMall[10 + off_set] = Color.FromArgb(c3, c2/2, c1);
				//colorMall[0 + off_set] = Color.FromArgb(228, 26, 28);
				//colorMall[1 + off_set] = Color.FromArgb(c2, c3, c1);//(55, 126, 184);
				//colorMall[2 + off_set] = Color.FromArgb(c1, c3, c2);//(98, 193, 59);
				//colorMall[3 + off_set] = Color.FromArgb(c1, c2, c3);//(152, 78, 163);
				//colorMall[4 + off_set] = Color.FromArgb(c2,c1,c3);//(255, 127, 0);
				//colorMall[5 + off_set] = Color.FromArgb(c2, c1, c3);//(255, 255, 51);
				//colorMall[6 + off_set] = Color.FromArgb(7, 147, 140);
				//colorMall[7 + off_set] = Color.FromArgb(c3,c1,c2);//(52, 68, 211);
				//colorMall[8 + off_set] = Color.FromArgb(235, 62, 83);
				//colorMall[9 + off_set] = Color.FromArgb(c3,c2,c1);//(122, 221, 173);
				//colorMall[10 + off_set] = Color.FromArgb(181, 181, 232);
				colorMall[11 + off_set] = Color.FromArgb(44, 178, 249);
				colorMall[12 + off_set] = Color.FromArgb(181, 181, 232);
				colorMall[13 + off_set] = Color.FromArgb(255, 127, 0);
            }

            colorMall[DEFAULTCOLORINDEX] = Color.Gray;

            if (this.colorpalette == null) this.colorpalette = new ColorPalette(colorMall, this);
            this.colorpalette.SetColors(colorMall);

		}
		public void InitialRandomMaterials(int K)
		{
			Random rand = new Random();
			materialMall = new float[K][];
			for (int i = 0; i < K; ++i)
			{
				int j = i%K;
				materialMall[i] = new float[4] {
					colorMall[j].R / 255.0f,
					colorMall[j].G / 255.0f,
					colorMall[j].B / 255.0f,
					1.0f
				};
			}
		}

		// for viewing
		private Trackball ball = new Trackball(100, 100);
		private Trackball editball = new Trackball(100, 100);
		private double scaleRatio;
		private Vector2d currMousePosition = new Vector2d();
		private Vector2d mouseDownPosition = new Vector2d();
		private bool isMouseDown = false;
		public bool showModel = true;
		public bool showFinalArrangement = false;
		public bool showMeanArrangement = false;
		private double depthTolerance = -0.0001;
		public Matrix4d currTransformation = Matrix4d.IdentityMatrix();
		


		// for sketching

		private List<Vector3d> samplepoints = new List<Vector3d>();


		// public properties
		public List<MeshRecord> meshes = new List<MeshRecord>();
		public MeshRecord MeshRecord
		{
			get { return meshRecord; }
			set 
			{
				if (this.meshRecord != null)
					this.meshRecord.CurrentTransformation = this.currTransformation;
				this.meshRecord = value;
                if (this.meshRecord != null) 
                    this.currTransformation = this.meshRecord.CurrentTransformation;
				this.Refresh(); 
			}
		}
		public EnumOperationMode CurrentMode
		{
			get { return currentMode; }
			set { currentMode = value; }
		}



		public MeshView()
		{
			InitializeComponent();

			this.AutoMakeCurrent = true;
			this.InitializeContexts();
			this.InitOpenGL();

			this.InitialRandomColors(53);
			this.InitialRandomMaterials(53);
		}
		private void InitializeComponent()
		{
			this.SuspendLayout();
			this.ResumeLayout(false);
		}
		protected override void OnSizeChanged(EventArgs e)
		{
			base.OnSizeChanged(e);

			Size s = this.Size;
			if (s.Width == 0 || s.Height == 0) return;

			this.scaleRatio = (s.Width > s.Height) ? s.Height : s.Width;
			
			this.ball.SetBounds(s.Width * 1.0, s.Height * 1.0);
			this.editball.SetBounds(s.Width * 1.0, s.Height * 1.0);

			this.Refresh();
		}
		protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs e)
		{
			base.OnMouseDown(e);

			Program.CurrentActiveView = this;
		//	Program.SetTabObject(this.MeshRecord);

			//if (this.meshRecord != null)
			//{
			//    Program.SetPropertyGridObject(this.meshRecord);
			//}


			this.currMousePosition = new Vector2d(e.X, this.Height - e.Y);
			this.mouseDownPosition = currMousePosition;
			this.isMouseDown = true;

			switch (this.currentMode)
			{
				case EnumOperationMode.Viewing:
					switch (e.Button)
					{
						case MouseButtons.Left: ball.Click(currMousePosition, Trackball.MotionType.Rotation); break;
						case MouseButtons.Middle: ball.Click(currMousePosition / this.scaleRatio, Trackball.MotionType.Pan); break;
						case MouseButtons.Right: ball.Click(currMousePosition, Trackball.MotionType.Scale); break;
					}
					break;
				case EnumOperationMode.Picking:
					
					break;

				case EnumOperationMode.ComponentGrouping:
					if (e.Button == System.Windows.Forms.MouseButtons.Right)
					{
						//this.ContextMenuStrip = Program.GetFormMain().GetContextMenuStrip();
						//this.ContextMenuStrip.Show();
					}
					break;
				case EnumOperationMode.UpdateContact:
					
					break;
			}
		}
		protected override void OnMouseMove(System.Windows.Forms.MouseEventArgs e)
		{
			base.OnMouseMove(e);

			this.currMousePosition = new Vector2d(e.X, this.Height - e.Y);

			switch (this.currentMode)
			{
				case EnumOperationMode.Viewing:
					if (isMouseDown)
					{
						switch (e.Button)
						{
							case MouseButtons.Left: ball.Drag(currMousePosition); break;
							case MouseButtons.Middle: ball.Drag(currMousePosition / this.scaleRatio); break;
							case MouseButtons.Right: ball.Drag(currMousePosition); break;
						}
						this.Refresh();
					}
					break;
				case EnumOperationMode.SpecifyContacts:
					if (this.isMouseDown)
					{
						this.Refresh();
					}
					break;
				case EnumOperationMode.ComponentGrouping:
				case EnumOperationMode.FitCircle3D:
					this.Refresh();
					break;
				case EnumOperationMode.AddEraseSlotPoints:
					this.Refresh();
					break;
				case EnumOperationMode.SelectCuboidMeshComponent:
					this.Refresh();
					break;
				case EnumOperationMode.UpdateContact:
					//this.SelectBoxVertexByRect();
					this.Refresh();
					break;
				//case EnumOperationMode.SelectBoundingboxFace:
				//    this.SelectBoxFaces(1);
				//    this.Refresh();
				//    break;
			}
		}
		protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs e)
		{
			base.OnMouseUp(e);

			this.currMousePosition = new Vector2d(e.X, this.Height - e.Y);
			this.isMouseDown = false;

			switch (this.currentMode)
			{
				case EnumOperationMode.Viewing:
					if (currMousePosition == mouseDownPosition) break;
					Matrix4d m = ball.GetMatrix();
					this.currTransformation = m * this.currTransformation;
					this.ball.End();
			//		this.UpdateAllQuadsHomoMatrix();

					this.Refresh();
					break;

				case EnumOperationMode.Picking:
					this.Refresh();
					break;


				case EnumOperationMode.ComponentGrouping:
					if (e.Button == System.Windows.Forms.MouseButtons.Left)
						//this.SelectComponentsByPoint(this.currMousePosition);
						this.SelectComponentsByRect();
					this.Refresh();
					break;

				case EnumOperationMode.AssignTouchingConstraint:
					
					break;

				case EnumOperationMode.UpdateContact:
					this.SelectBoxVertexByRect();
					this.Refresh();
					break;

				//case EnumOperationMode.SelectBoundingboxFace:
				//    this.SelectBoxFaces(2);
				//    this.Refresh();
				//    break;
			}
		}
		protected override void OnMouseDoubleClick(MouseEventArgs e)
		{			

			base.OnMouseDoubleClick(e);

		}

		public bool drawmodel = false;
		protected override void OnPaint(PaintEventArgs e)
		{
			//if (!initFont)
			//{
			//    BuildFont(e);
			//    initFont = true;
			//}


			if (this.DesignMode == true)
			{
				base.OnPaint(e); 
				return;
			}

			this.MakeCurrent();

			Gl.glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
			Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);

			
			// draw texture image if any
			//this.Render2dScene();

			if (isGraphView)
			{
				//this.DrawModelGraph();
				Render3dGraph();
				if(this.drawmodel)
					this.Render3dScene();
				this.SwapBuffers();
				return;
			}
			
			// draw 3d scene
			this.Render3dScene();

			this.SwapBuffers();
		}

		private void Render3dGraph()
		{

			this.InitMatrix();

			// -------------------------------------------------------------------------
			// draw 3d sense
			Matrix4d m = ball.GetMatrix() * currTransformation;

			Gl.glMatrixMode(Gl.GL_MODELVIEW);

			Gl.glPushMatrix();
			Gl.glMultMatrixd(m.Transpose().ToArray());

			Gl.glEnable(Gl.GL_POLYGON_OFFSET_FILL);

			if (this.drawModelGraph)
			{
				if ((this == Program.GetFormMain().GetSourceGraphView() &&
					this.modelGraph != null) ||
					(this == Program.GetFormMain().GetTargetGraphView() &&
					this.modelGraph != null))
				{
					this.DrawMatchedModelGraph();
					//this.DrawMatchedModelGraph2D();
				}
				else
					this.DrawModelGraph3D(1);
			}

			Gl.glPopMatrix();

			Gl.glPopMatrix();
		}
		
		private void Render2dScene()
		{
			Gl.glPushMatrix();

			Gl.glViewport(0, 0, this.Size.Width, this.Size.Height);

			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glLoadIdentity();

			Glu.gluOrtho2D(0, this.Size.Width, this.Size.Height, 0);

			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glLoadIdentity();

			Gl.glPushMatrix();

			Gl.glColor3f(1.0f, 1.0f, 1.0f);

			if (this.drawModelGraph)
				DrawModelGraph();

			Gl.glPopMatrix();

			Gl.glPopMatrix();

		}
		
		private void Render3dScene()
		{

			this.InitMatrix();

			// -------------------------------------------------------------------------
			// draw 3d sense
			Matrix4d m = ball.GetMatrix() * currTransformation;


			Gl.glMatrixMode(Gl.GL_MODELVIEW);

			Gl.glPushMatrix();
			Gl.glMultMatrixd(m.Transpose().ToArray());

			Gl.glEnable(Gl.GL_POLYGON_OFFSET_FILL);
			
			// draw here --
			// draw mesh
			if (this.meshRecord != null && this.meshRecord.Mesh != null)
			{
				//this.DrawFlatShaded_Color(this.meshRecord.Mesh);

				if (this.meshComponents != null)
				{
					foreach (MeshComponent mc in this.meshComponents)
					{
						if (!this.ShowSolidMesh)
							this.DrawTransparent(mc.mesh, colorMall[mc.colorIdx]);
						else
							this.DrawFlatShaded(mc.mesh, colorMall[mc.colorIdx]);
					}
				}

			}
			else if (this.modelGraph != null)
			{
				if (this.modelGraph.graphNodes != null)
				{
					foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
					{
						if(this.isGraphView)
							this.DrawTransparent(mn.meshComponent.mesh, colorMall[mn.nodeType]);
						else
							this.DrawFlatShaded(mn.meshComponent.mesh, colorMall[mn.nodeType]);
					}
				}
			}

			// show component proxies
			if (this.ShowComponentProxy && this.meshComponents != null)
			{
				foreach (MeshComponent c in this.meshComponents)
				{
					this.DrawProxy_Transparent(c.boundingBox, colorMall[c.colorIdx], 1.0f);
				}
			}
			if (drawModelGraph)
			{
				if( (this == Program.GetFormMain().GetSourceGraphView() &&
					this.modelGraph != null) ||
					(this == Program.GetFormMain().GetTargetGraphView() &&
					this.modelGraph != null))
				{
					this.DrawMatchedModelGraph();
					//this.DrawMatchedModelGraph2D();
				}
				else
					this.DrawModelGraph3D(0);
			}

			// highlight selection
			this.HighlightSelection();

			// show vertex
			if (this.showSelectedBoxVertex)
				DrawPoints(this.selectedboxVertices, Color.Black);

			// draw ground grid
			if (this.drawGroundGrids)
				this.DrawGroundGrid();

			if (this.samplepoints != null)
			{
				this.DrawPoints(this.samplepoints, Color.Red);
			}

			// draw ball track
			if (this.trackball != null)
				this.DrawFlatShaded(this.trackball, Color.Red);

			Gl.glPopMatrix();
			
			Gl.glPopMatrix();
		}

		private void HighlightSelection()
		{
			// show selected components
			foreach (MeshComponent mc in this.selectedMeshComponents)
			{
				//		this.DrawTransparent(mc.mesh, Color.Lime);
				this.DrawFlatShaded(mc.mesh, Color.OrangeRed);
			}

			// show selection rect
			if ((this.currentMode == EnumOperationMode.ComponentGrouping ||
				 this.currentMode == EnumOperationMode.SpecifyContacts ||
				 this.currentMode == EnumOperationMode.FitCircle3D ||
				 this.currentMode == EnumOperationMode.UpdateContact)
					&& this.isMouseDown)
				this.DrawSelectionRect();

			//// show selected proxy face
			//if (this.selectedPolyFace != null)
			//    this.DrawPolygon_Transparent(this.selectedPolyFace, Color.Yellow, 1);

			//if (this.prevSelectedPolyFace != null)
			//    this.DrawPolygon_Transparent(this.prevSelectedPolyFace, Color.Yellow, 1);

			//// show selected body node
			//if (this.selectedBodyNode != null)
			//    this.selectedBodyNode.draw(Color.Yellow);
		}

		private bool useGLOrthoProjection = false;
		private OpenGLProjector glProjector = null;
		public void ObtainGLProjector()
		{
			if (!this.useGLOrthoProjection)
			{
				Gl.glPushMatrix();
				Matrix4d mat = ball.GetMatrix() * currTransformation;
				Gl.glMultMatrixd(mat.Transpose().ToArray());
				this.glProjector = new OpenGLProjector();
				Gl.glPopMatrix();
			}
			else
			{
				this.glProjector = new OpenGLProjector();
			}
		}
		public void ScreenCaptureDefault()
		{
			this.ScreenCapture(this.screenCapRect, @"d:\\cap1.jpg");
		}
		public void ScreenCapture(Rectangle rect, string file)
		{
			int w = rect.Width, h = rect.Height;

			w -= w % 4; h -= h % 4;

			this.Refresh();
			this.Refresh();

			// generate suggestion images
			byte[] bytes = new byte[w * h * 3];


			Gl.glReadPixels(rect.X, rect.Y, w, h, Gl.GL_BGR, Gl.GL_UNSIGNED_BYTE, bytes);


			//Image<Bgr, Byte> image = new Image<Bgr, Byte>(w, h); //specify the width and height here
			//image.Bytes = bytes; //your byte array
			//image = image.Flip(Emgu.CV.CvEnum.FLIP.VERTICAL);

			//image.Save(file);

			Program.OutputText("saved screen cap file to " + file, true);
		}


		private bool PointInTriangle(Vector2d p, Vector2d a, Vector2d b, Vector2d c)
		{
			if (SameSide(p,a,b,c) && SameSide(p,b,a,c) && SameSide(p,c, a,b))
				return true;
			return false;
		}
		private bool SameSide(Vector2d p1, Vector2d p2, Vector2d a, Vector2d b)
		{
			Vector3d A = new Vector3d(a);
			Vector3d B = new Vector3d(b);
			Vector3d P = new Vector3d(p1);
			Vector3d Q = new Vector3d(p2);
			Vector3d E = B-A;
			Vector3d cp1 = E.Cross(P-A);
			Vector3d cp2 = E.Cross(Q-A);
			if (cp1.Dot(cp2) >= 0) 
				return true;
			return false;
		}


		private void InitOpenGL()
		{
            // OpenGl.GL state
            Gl.glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
            Gl.glDisable(Gl.GL_DITHER);
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            Gl.glEnable(Gl.GL_POINT_SMOOTH);
            Gl.glEnable(Gl.GL_LINE_SMOOTH);


            // Material
            SetDefaultMaterial();


            // Lighting
            Gl.glLoadIdentity();
            SetDefaultLight();


            // Fog
            float[] fogColor = new float[] { 0.3f, 0.3f, 0.4f, 1.0f };

            Gl.glFogi(Gl.GL_FOG_MODE, Gl.GL_LINEAR);
            Gl.glFogfv(Gl.GL_FOG_COLOR, fogColor);
            Gl.glFogf(Gl.GL_FOG_DENSITY, 0.35f);
            Gl.glHint(Gl.GL_FOG_HINT, Gl.GL_DONT_CARE);
            Gl.glFogf(Gl.GL_FOG_START, 5.0f);
            Gl.glFogf(Gl.GL_FOG_END, 25.0f);


            Gl.glClearColor(0, 0, 0, 0);
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();
            Gl.glOrtho(0.0, 1.0, 0.0, 1.0, -1.0, 1.0);
		}
        private void SetDefaultLight()
        {
            float[] pos1 = new float[4] { 0.1f, 0.1f, -0.02f, 0.0f };
            float[] pos2 = new float[4] { -0.1f, 0.1f, -0.02f, 0.0f };
            float[] pos3 = new float[4] { 0.0f, 0.0f, 0.1f, 0.0f };
            float[] col1 = new float[4] { 0.7f, 0.7f, 0.8f, 1.0f };
            float[] col2 = new float[4] { 0.8f, 0.7f, 0.7f, 1.0f };
            float[] col3 = new float[4] { 1.0f, 1.0f, 1.0f, 1.0f };


            Gl.glEnable(Gl.GL_LIGHT0);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_POSITION, pos1);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, col1);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_SPECULAR, col1);


            Gl.glEnable(Gl.GL_LIGHT1);
            Gl.glLightfv(Gl.GL_LIGHT1, Gl.GL_POSITION, pos2);
            Gl.glLightfv(Gl.GL_LIGHT1, Gl.GL_DIFFUSE, col2);
            Gl.glLightfv(Gl.GL_LIGHT1, Gl.GL_SPECULAR, col2);


            Gl.glEnable(Gl.GL_LIGHT2);
            Gl.glLightfv(Gl.GL_LIGHT2, Gl.GL_POSITION, pos3);
            Gl.glLightfv(Gl.GL_LIGHT2, Gl.GL_DIFFUSE, col3);
            Gl.glLightfv(Gl.GL_LIGHT2, Gl.GL_SPECULAR, col3);

        }
        private void SetDefaultMaterial()
        {
            float[] mat_a = new float[4] { 0.1f, 0.1f, 0.1f, 1.0f };
            float[] mat_d = { 0.7f, 0.7f, 0.5f, 1.0f };
            float[] mat_s = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] shine = { 120.0f };

            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_AMBIENT, mat_a);
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_DIFFUSE, mat_d);
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_SPECULAR, mat_s);
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_SHININESS, shine);

        }
		private void InitMatrix_Ortho()
		{
			Size s = this.Size;
			double w = s.Width;
			double h = s.Height;

		//	this.MakeCurrent();
			
			Gl.glViewport(0, 0, s.Width, s.Height);
			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glLoadIdentity();

			if (w > h)
			{
				double ratio = (w / h) / 2.0;
				Gl.glOrtho(-ratio, ratio, -0.5, 0.5, -100, 100);
			}
			else
			{
				double ratio = (h / w) / 2.0;
				Gl.glOrtho(-0.5, 0.5, -ratio, ratio, -100, 100);
			}

			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glLoadIdentity();
		}
		private void InitMatrix()
		{
			double w = Size.Width;
			double h = Size.Height;
			Gl.glViewport(0, 0, Size.Width, Size.Height);
			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glLoadIdentity();
			
			Glu.gluPerspective(60.0, w / h, 0.01, 100);

			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glLoadIdentity();

			//Glu.gluLookAt(0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
			Glu.gluLookAt(0.3, 0.5, 2.0, 0.3, 0.0, 0.0, 0.0, 1.0, 0.0);
		}


		private void DrawPoints(List<Vector3d> points, Color c)
		{
			Gl.glPointSize(5);
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glBegin(Gl.GL_POINTS);
			foreach (Vector3d pt in points)
			{
				Gl.glVertex3d(pt.x, pt.y, pt.z);
			}
			Gl.glEnd();
			Gl.glPointSize(1.0f);
		}

		private void DrawPoints(double[] vpos, Color c)
		{
			Gl.glPointSize(5);
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glBegin(Gl.GL_POINTS);
			for (int i = 0; i < vpos.Length; i+=3)
			{
				Gl.glVertex3d(vpos[i], vpos[i+1], vpos[i+2]);
			}
			Gl.glEnd();
			Gl.glPointSize(1.0f);
		}

		private void DrawPoints(double[] vpos, double[] color)
		{
			this.MakeCurrent();
			Gl.glPointSize(pointSize+5);
			Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
			Gl.glEnableClientState(Gl.GL_COLOR_ARRAY);
			fixed (double* vp = vpos)
			fixed (double* cp = color)
			{
				Gl.glColorPointer(3, Gl.GL_DOUBLE, 0, new IntPtr(cp));
				Gl.glVertexPointer(3, Gl.GL_DOUBLE, 0, new IntPtr(vp));
				Gl.glDrawArrays(Gl.GL_POINTS, 0, vpos.Length/3);
			}
			Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
			Gl.glDisableClientState(Gl.GL_COLOR_ARRAY);
		}
		private void DrawPoints()
		{
			Mesh m = this.meshRecord.Mesh;

			this.MakeCurrent();
			Gl.glPointSize(pointSize);
			Color c = pointColor;
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
			fixed (double* vp = m.VertexPos)
			{
				
				Gl.glColorPointer(3, Gl.GL_DOUBLE, 0, new IntPtr(vp));
				Gl.glDrawArrays(Gl.GL_POINTS, 0, m.VertexCount);
			}
			Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
		}
		private void DrawWireframe(Mesh m, Color c)
		{
			if (m == null) return;

			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_LINE);
			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glColor3ub(c.R, c.G, c.B);

			Gl.glLineWidth(1.0f);
			Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
			fixed (double* vp = m.VertexPos)
			fixed (int* index = m.FaceIndex)
			{
				Gl.glVertexPointer(3, Gl.GL_DOUBLE, 0, new IntPtr(vp));
				Gl.glDrawElements(Gl.GL_TRIANGLES, m.FaceCount * 3, Gl.GL_UNSIGNED_INT, new IntPtr(index));
			}
			Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_LIGHTING);
		}

		public bool drawGroundGrids = true;
		private void DrawFlatShaded(Mesh m, Color c)
		{
            if (m == null) return;

			
            float[] mat = new float[4] { c.R / 255f, c.G / 255f, c.B / 255f, 1.0f };
			//Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_DIFFUSE, mat);
			Gl.glEnable(Gl.GL_CULL_FACE);
			//Gl.glDisable(Gl.GL_CULL_FACE);

			Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);
			//Gl.glEnable(Gl.GL_POINT_SMOOTH);
			//Gl.glEnable(Gl.GL_LINE_SMOOTH);
			//Gl.glHint(Gl.GL_POINT_SMOOTH_HINT, Gl.GL_NICEST);
			//Gl.glHint(Gl.GL_LINE_SMOOTH_HINT, Gl.GL_NICEST);

			//Gl.glEnable(Gl.GL_MULTISAMPLE);
			//Gl.glEnable(Gl.GL_BLEND);
			//Gl.glBlendFunc(Gl.GL_DST_ALPHA, Gl.GL_ONE_MINUS_DST_ALPHA);
			//Gl.glHint(Gl.GL_POLYGON_SMOOTH_HINT, Gl.GL_NICEST);
			//Gl.glEnable(Gl.GL_POLYGON_SMOOTH);
            fixed (double* vp = m.VertexPos)
            fixed (double* np = m.FaceNormal)
            fixed (int* fi = m.FaceIndex)
            {
                Gl.glBegin(Gl.GL_TRIANGLES);
                for (int i = 0, j = 0; i < m.FaceCount; i++, j += 3)
                {
                    Gl.glNormal3dv(new IntPtr(np + j));
                    Gl.glVertex3dv(new IntPtr(vp + fi[j] * 3));
                    Gl.glVertex3dv(new IntPtr(vp + fi[j + 1] * 3));
                    Gl.glVertex3dv(new IntPtr(vp + fi[j + 2] * 3));
                }
                Gl.glEnd();
            }
			//Gl.glDisable(Gl.GL_LINE_SMOOTH);
			//Gl.glDisable(Gl.GL_POINT_SMOOTH);
			
			Gl.glDisable(Gl.GL_NORMALIZE);
            Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glDisable(Gl.GL_CULL_FACE);
			//Gl.glEnable(Gl.GL_CULL_FACE);
		
		}
		private int[] faceClusterIndex = null;
		private void DrawFlatShaded_Color(Mesh m)
		{
			if (m == null || this.faceClusterIndex == null || this.faceClusterIndex.Length != m.FaceCount) return;

			Gl.glShadeModel(Gl.GL_FLAT);
			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);
			Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);

			//Color c = this.meshColor;
			
			fixed (double* vp = m.VertexPos)
			fixed (double* np = m.FaceNormal)
			fixed (int* fi = m.FaceIndex)
			{
				Gl.glBegin(Gl.GL_TRIANGLES);
				for (int i = 0, j = 0; i < m.FaceCount; i++, j += 3)
				{
					Color c = colorMall[this.faceClusterIndex[i]];
					Gl.glColor3ub(c.R, c.G, c.B);
					Gl.glNormal3dv(new IntPtr(np + j));
					Gl.glVertex3dv(new IntPtr(vp + fi[j] * 3));
					Gl.glVertex3dv(new IntPtr(vp + fi[j + 1] * 3));
					Gl.glVertex3dv(new IntPtr(vp + fi[j + 2] * 3));
				}
				Gl.glEnd();
			}
			Gl.glDisable(Gl.GL_LIGHTING);
		}
		private void DrawSmoothShaded(Mesh m, Color c)
		{
			if (m == null) return;

			Gl.glShadeModel(Gl.GL_SMOOTH);
			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);
			Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);

			//Color c = this.meshColor;
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
			Gl.glEnableClientState(Gl.GL_NORMAL_ARRAY);
			fixed (double* vp = m.VertexPos)
			fixed (double* np = m.VertexNormal)
			fixed (int* index = m.FaceIndex)
			{
				Gl.glVertexPointer(3, Gl.GL_DOUBLE, 0, new IntPtr(vp));
				Gl.glNormalPointer(Gl.GL_DOUBLE, 0, new IntPtr(np));
				Gl.glDrawElements(Gl.GL_TRIANGLES, m.FaceCount * 3, Gl.GL_UNSIGNED_INT, new IntPtr(index));
			}
			Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
			Gl.glDisableClientState(Gl.GL_NORMAL_ARRAY);
			Gl.glDisable(Gl.GL_LIGHTING);
		}
		
		
		private void DrawTransparent(Mesh m, Color c)
		{
			if (m == null) return;

            byte alphaChannel = 100;

            Gl.glPushClientAttrib(Gl.GL_CLIENT_VERTEX_ARRAY_BIT);
            Gl.glPushAttrib(Gl.GL_ALL_ATTRIB_BITS);

            Gl.glShadeModel(Gl.GL_FLAT);
            Gl.glEnable(Gl.GL_NORMALIZE);
			Gl.glDisable(Gl.GL_LIGHTING);
            Gl.glEnable(Gl.GL_BLEND);
            Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);
            Gl.glEnable(Gl.GL_CULL_FACE);
            Gl.glDisable(Gl.GL_DEPTH_TEST);
            Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);

			Gl.glEnable(Gl.GL_MULTISAMPLE);

            Gl.glColor4ub(c.R, c.G, c.B, alphaChannel);

            Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
            Gl.glEnableClientState(Gl.GL_NORMAL_ARRAY);
            fixed (double* vp = m.VertexPos)
            fixed (double* np = m.VertexNormal)
            fixed (int* index = m.FaceIndex)
            {
                Gl.glVertexPointer(3, Gl.GL_DOUBLE, 0, (IntPtr)vp);
                Gl.glNormalPointer(Gl.GL_DOUBLE, 0, (IntPtr)np);
                Gl.glDrawElements(Gl.GL_TRIANGLES, m.FaceCount * 3, Gl.GL_UNSIGNED_INT, (IntPtr)index);
            }
            Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
            Gl.glDisableClientState(Gl.GL_NORMAL_ARRAY);

            //////////////////////////////////////////////////////////////////////////             

			//Gl.glEnable(Gl.GL_LIGHTING);
			//Gl.glDepthFunc(Gl.GL_EQUAL);

			//float[] LightDiffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
			//float[] LightSpecular = { 1.0f, 1.0f, 1.0f, 1.0f };
			//Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, LightDiffuse);
			//Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_SPECULAR, LightSpecular);
			//Gl.glEnable(Gl.GL_LIGHT0);

			//float[] Ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
			//float[] Diffuse = { 0.0f, 0.0f, 0.0f, 1.0f };
			//float[] SpecularRef = { 1.0f, 1.0f, 1.0f, 1.0f };

			//Gl.glColorMaterial(Gl.GL_FRONT, Gl.GL_DIFFUSE);
			//Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, Diffuse);
			//Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, Ambient);
			//Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, SpecularRef);
			//Gl.glMateriali(Gl.GL_FRONT, Gl.GL_SHININESS, 128);

			//Gl.glColor4ub(c.R, c.G, c.B, alphaChannel);

			//Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
			//Gl.glEnableClientState(Gl.GL_NORMAL_ARRAY);
			//fixed (double* vp = m.VertexPos)
			//fixed (double* np = m.VertexNormal)
			//fixed (int* index = m.FaceIndex)
			//{
			//    Gl.glVertexPointer(3, Gl.GL_DOUBLE, 0, (IntPtr)vp);
			//    Gl.glNormalPointer(Gl.GL_DOUBLE, 0, (IntPtr)np);
			//    Gl.glDrawElements(Gl.GL_TRIANGLES, m.FaceCount * 3, Gl.GL_UNSIGNED_INT, (IntPtr)index);
			//}
			//Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
			//Gl.glDisableClientState(Gl.GL_NORMAL_ARRAY);


			//Gl.glDisable(Gl.GL_NORMALIZE);
			//Gl.glDisable(Gl.GL_LIGHTING);
			//Gl.glDisable(Gl.GL_BLEND);
			//Gl.glDisable(Gl.GL_CULL_FACE);
			//Gl.glEnable(Gl.GL_DEPTH_TEST);

            Gl.glPopAttrib();
            Gl.glPopClientAttrib();

		}
		private void DrawSelectedVertice_ByPoint()
		{
			Mesh m = this.meshRecord.Mesh;

			Gl.glColor3f(1.0f, 0.4f, 0.4f);
			Gl.glPointSize(this.pointSize + 4);
			fixed (double* vp = m.VertexPos)
			{
				Gl.glEnableClientState(Gl.GL_VERTEX_ARRAY);
				Gl.glVertexPointer(3, Gl.GL_DOUBLE, 0, new IntPtr(vp));
				Gl.glBegin(Gl.GL_POINTS);
				for (int i = 0; i < m.VertexCount; i++)
				{
					if (m.Flag[i] == 0) continue;


					switch (m.Flag[i] % 12)
					{
						case 0: Gl.glColor3f(1.0f, 0.0f, 1.0f); break;
						case 1: Gl.glColor3f(1.0f, 0.0f, 0.0f); break;
						case 2: Gl.glColor3f(0.0f, 1.0f, 0.0f); break;
						case 3: Gl.glColor3f(0.0f, 0.0f, 1.0f); break;
						case 4: Gl.glColor3f(1.0f, 1.0f, 0.0f); break;
						case 5: Gl.glColor3f(0.0f, 1.0f, 1.0f); break;
						case 6: Gl.glColor3f(0.5f, 0.0f, 0.0f); break;
						case 7: Gl.glColor3f(0.0f, 0.5f, 0.0f); break;
						case 8: Gl.glColor3f(0.0f, 0.0f, 0.5f); break;
						case 9: Gl.glColor3f(0.5f, 0.5f, 0.0f); break;
						case 10: Gl.glColor3f(0.0f, 0.5f, 0.5f); break;
						case 11: Gl.glColor3f(0.5f, 0.0f, 0.5f); break;
					}

					//Gl.glColor3f(0.0f, 1.0f, 0.0f);
					//Gl.glArrayElement(i);
					Gl.glVertex3dv(new IntPtr(vp + i * 3));
				}
				Gl.glEnd();
				Gl.glDisableClientState(Gl.GL_VERTEX_ARRAY);
			}
		}
		private void DrawScreenCapRect()
		{
			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glPushMatrix();
			Gl.glLoadIdentity();
			Glu.gluOrtho2D(0, this.Width, 0, this.Height);
			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glPushMatrix();
			Gl.glLoadIdentity();

			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_LINE);
			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
			Gl.glColor3f(1.0f, 0.0f, 0.0f);
			Gl.glRectd(this.screenCapRect.X, this.screenCapRect.Y, this.screenCapRect.X + this.screenCapRect.Width,
				this.screenCapRect.Height + this.screenCapRect.Y);
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);

			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glPopMatrix();
			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glPopMatrix();
		}
		private void DrawSelectionRect()
		{
			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glPushMatrix();
			Gl.glLoadIdentity();
			Glu.gluOrtho2D(0, this.Width, 0, this.Height);
			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glPushMatrix();
			Gl.glLoadIdentity();

			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);
			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
			Gl.glColor3f(1.0f, 0.0f, 0.0f);
			Gl.glRectd(this.mouseDownPosition.x, this.mouseDownPosition.y, 
				this.currMousePosition.x, this.currMousePosition.y);
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);

			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glPopMatrix();
			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glPopMatrix();
		}


		//List<int> selectedMeshcomponentIdx = new List<int>(),
		//    selectedFaceIdx = new List<int>();
		//int highlight_componentIdx = -1, highlight_faceIdx = -1;
		//private void SelectBoxFaces(int tag)
		//{
		//    // tag = 1: mouse move
		//    // tag = 2: mouse down (select)
		//    Vector2d cursor = currMousePosition;
		//    OpenGLProjector projector = new OpenGLProjector();
		//    double minDepth = double.MaxValue;
		//    int mcIdx = -1, faceIdx = -1;
		//    for(int i = 0; i <this.modelGraph.meshComponents.Count; ++i)
		//    {
		//        MeshComponent mc = this.modelGraph.meshComponents[i];
		//        List<Vector2d> mapPos = new List<Vector2d>();
		//        foreach (Point3 pt in mc.boundingBox.points3)
		//        {
		//            Vector3d v = projector.Project(pt.pos);
		//            mapPos.Add(new Vector2d(v.x, v.y));
		//        }
		//        //
		//        for(int j = 0; j < mc.boundingBox.polygons.Count; ++j)
		//        {
		//            Polygon pl = mc.boundingBox.polygons[j];
		//            int minx = int.MaxValue, miny = int.MaxValue,
		//                maxx = int.MinValue, maxy = int.MinValue;
		//            double depthvalue = 0;
		//            foreach (Point3 pt in pl.points3)
		//            {
		//                Vector3d v = projector.Project(pt.pos);
		//                minx = minx < (int)v.x ? minx : (int)v.x;
		//                miny = miny < (int)v.y ? miny : (int)v.y;
		//                maxx = maxx > (int)v.x ? maxx : (int)v.x;
		//                maxy = maxy > (int)v.y ? maxy : (int)v.y;
		//                depthvalue += v.z;
		//            }
		//            depthvalue /= pl.points3.Count;
		//            Rectangle rect = new Rectangle(minx, miny, maxx - minx, maxy - miny);
		//            if (rect.Contains((int)currMousePosition.x, (int)currMousePosition.y)
		//                && depthvalue < minDepth)
		//            {
		//                mcIdx = i;
		//                faceIdx = j;
		//                minDepth = depthvalue;
		//            }
		//        }//faces
		//    }//components
		//    if (mcIdx == -1) return;
		//    highlight_componentIdx = mcIdx;
		//    highlight_faceIdx = faceIdx;
		//    if (tag == 2)
		//    {
		//        if (selectedMeshcomponentIdx.Count() >= 2)
		//        {
		//            selectedMeshcomponentIdx.RemoveAt(0);
		//            selectedFaceIdx.RemoveAt(0);
		//        }
		//        selectedMeshcomponentIdx.Add(mcIdx);
		//        selectedFaceIdx.Add(faceIdx);
		//    }
		//}//SelectBoxFace
		
		private void SelectBoxVertexByRect()
		{
			Vector2d minV = Vector2d.Min(mouseDownPosition, currMousePosition);
			Vector2d size = Vector2d.Max(mouseDownPosition, currMousePosition) - minV;
			Rectangle rect = new Rectangle((int)minV.x, (int)minV.y, (int)size.x, (int)size.y);
			Rectangle viewport = new Rectangle(0, 0, this.Width, this.Height);
			this.ObtainGLProjector();
			OpenGLProjector projector = new OpenGLProjector();
			bool laser = true;
			double eps = this.depthTolerance;
			//if ((Control.ModifierKeys & Keys.Shift) == Keys.Shift)
			foreach (MeshComponent component in this.modelGraph.meshComponents)
			{
				foreach (Point3 pt in component.boundingBox.points3)
				{
					Vector3d v = projector.Project(pt.pos);
					if (viewport.Contains((int)v.x, (int)v.y))
					{
						bool flag = rect.Contains((int)v.x, (int)v.y);
						flag &= (laser || projector.GetDepthValue((int)v.x, (int)v.y) - v.z >= eps);
						if (flag)
						{
							this.selectedboxVertices.Add(pt.pos);
						}
					}
				}
			}
				
			
		}
		private void SelectVertexByRect()
		{
			Vector2d minV = Vector2d.Min(mouseDownPosition, currMousePosition);
			Vector2d size = Vector2d.Max(mouseDownPosition, currMousePosition) - minV;
			Rectangle rect = new Rectangle((int)minV.x, (int)minV.y, (int)size.x, (int)size.y);
			Rectangle viewport = new Rectangle(0, 0, this.Width, this.Height);
			OpenGLProjector projector = new OpenGLProjector();
			Mesh m = this.meshRecord.Mesh;
			bool laser = true;
			double eps = this.depthTolerance;

			if ((Control.ModifierKeys & Keys.Shift) == Keys.Shift)
				for (int i = 0, j = 0; i < m.VertexCount; i++, j += 3)
				{
					Vector3d v = projector.Project(m.VertexPos, j);
					if (viewport.Contains((int)v.x, (int)v.y))
					{
						bool flag = rect.Contains((int)v.x, (int)v.y);
						flag &= (laser || projector.GetDepthValue((int)v.x, (int)v.y) - v.z >= eps);
						if (flag) m.Flag[i] = (byte)1;
					}
				}

			else if ((Control.ModifierKeys & Keys.Control) == Keys.Control)
				for (int i = 0, j = 0; i < m.VertexCount; i++, j += 3)
				{
					Vector3d v = projector.Project(m.VertexPos, j);
					if (viewport.Contains((int)v.x, (int)v.y))
					{
						bool flag = rect.Contains((int)v.x, (int)v.y);
						flag &= (laser || projector.GetDepthValue((int)v.x, (int)v.y) - v.z >= eps);
						if (flag) m.Flag[i] = (byte)0;
					}
				}

			else
				for (int i = 0, j = 0; i < m.VertexCount; i++, j += 3)
				{
					Vector3d v = projector.Project(m.VertexPos, j);
					if (viewport.Contains((int)v.x, (int)v.y))
					{
						bool flag = rect.Contains((int)v.x, (int)v.y);
						flag &= (laser || projector.GetDepthValue((int)v.x, (int)v.y) - v.z >= eps);
						m.Flag[i] = (byte)((flag) ? 1 : 0);
					}
				}
		}
		private void DrawUserPoint(Vector3d p, Color c, float size)
		{
			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glPointSize(size);
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glBegin(Gl.GL_POINTS);
			Gl.glVertex3d(p.x, p.y, p.z);
			Gl.glEnd();
			Gl.glPointSize(1.0f);
			Gl.glEnable(Gl.GL_LIGHTING);
		}
		private void DrawUserPoints(List<Vector3d> pts, Color c, float size)
		{
			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glPointSize(size);
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glBegin(Gl.GL_POINTS);
			foreach (Vector3d p in pts)
				Gl.glVertex3d(p.x, p.y, p.z);
			Gl.glEnd();
			Gl.glPointSize(1.0f);
			Gl.glEnable(Gl.GL_LIGHTING);
		}

		public void DrawCylinder(Vector3d u, Vector3d v, double radius, Color c)
		{
			double len1 = (u - v).Length();
			Vector3d dir = (v - u).Normalize();

			double r1 = radius, r2 = r1/2;
			double angle = Math.Acos(dir.z / dir.Length()) * 180 / Math.PI;
			Glu.GLUquadric quad = Glu.gluNewQuadric();

			Color ocolor = Color.SkyBlue;
			Gl.glColor3ub(ocolor.R, ocolor.G, ocolor.B);
			Gl.glPushMatrix();
			Gl.glTranslated(u.x, u.y, u.z);
			Glu.gluSphere(quad, r2, 40, 40);
			Gl.glPopMatrix();

			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glPushMatrix();
			Gl.glTranslated(u.x, u.y, u.z);
			Gl.glRotated(angle, -dir.y, dir.x, 0);
			Glu.gluCylinder(quad, r1, r1, len1, 40, 40);
			Gl.glPopMatrix();

			Gl.glPushMatrix();
			Gl.glTranslated(v.x, v.y, v.z);
			Gl.glRotated(angle, -dir.y, dir.x, 0);
			Glu.gluSphere(quad, r2, 40, 40);
			Gl.glPopMatrix();

			Glu.gluDeleteQuadric(quad);
		}
		public void DrawSphere(Vector3d u, double radius, Color c)
		{
			Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);

			Glu.GLUquadric quad = Glu.gluNewQuadric();

			//Color ocolor = c;
			//Gl.glColor3ub(ocolor.R, ocolor.G, ocolor.B);

			float[] mat = new float[4] { c.R / 255f, c.G / 255f, c.B / 255f, 1.0f };
			Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_DIFFUSE, mat);

			Gl.glPushMatrix();
			Gl.glTranslated(u.x, u.y, u.z);
			Glu.gluSphere(quad, radius, 40, 40);
			Gl.glPopMatrix();

			Glu.gluDeleteQuadric(quad);

			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glDisable(Gl.GL_NORMALIZE);
		}

		private void SelectVertexByPoint(Vector2d u)
		{
			Mesh m = this.meshRecord.Mesh;
			double eps = this.depthTolerance;

			this.ObtainGLProjector();
			OpenGLProjector projector = this.glProjector;
			
			
			Rectangle viewport = new Rectangle(0, 0, this.Width, this.Height);

			double minDis = double.MaxValue;
			int minIndex = -1;

			for (int i = 0, j = 0; i < m.VertexCount; i++, j += 3)
			{
				m.Flag[i] = 0;

				Vector3d v = projector.Project(m.VertexPos, j);
				if (viewport.Contains((int)v.x, (int)v.y) &&
					projector.GetDepthValue((int)v.x, (int)v.y) - v.z >= eps)
				{
					double dx = (u.x - v.x);
					double dy = (u.y - v.y);
					double dis = dx * dx + dy * dy;
					if (dis < minDis)
					{
						minDis = dis;
						minIndex = i;
					}
				}
			}

			if (minIndex != -1)
				m.Flag[minIndex] = 1;
		}
		private void SelectFaceByPoint()
		{
			Mesh m = this.meshRecord.Mesh;
			double eps = this.depthTolerance;

			OpenGLProjector projector = new OpenGLProjector();
			Rectangle viewport = new Rectangle(0, 0, this.Width, this.Height);

			double minDis = double.MaxValue;
			int minIndex = -1;
			Vector2d u = currMousePosition;

			for (int i = 0, j = 0; i < m.FaceCount; i++, j += 3)
			{
				Vector3d v = projector.Project(m.DualVertexPos, j);
				if (viewport.Contains((int)v.x, (int)v.y) &&
					projector.GetDepthValue((int)v.x, (int)v.y) - v.z >= eps)
				{
					double dx = (u.x - v.x);
					double dy = (u.y - v.y);
					double dis = dx * dx + dy * dy;
					if (dis < minDis)
					{
						minDis = dis;
						minIndex = i;
					}
				}
			}

			this.selectedFace = minIndex;

			Program.OutputText("Selected face: " + minIndex.ToString(), true);
		}
		private int PickSelectedVertexByPoint(Vector2d u)
		{
			Mesh m = this.meshRecord.Mesh;
			double eps = this.depthTolerance;

			this.ObtainGLProjector();
			OpenGLProjector projector = this.glProjector;

			Rectangle viewport = new Rectangle(0, 0, this.Width, this.Height);

			double minDis = double.MaxValue;
			int minIndex = -1;


			for (int i = 0, j = 0; i < m.VertexCount; i++, j += 3)
			{
				m.Flag[i] = 0; // reset flag

				Vector3d v = projector.Project(m.VertexPos, j);
				if (//m.Flag[i] != 0 &&
					viewport.Contains((int)v.x, (int)v.y) &&
					projector.GetDepthValue((int)v.x, (int)v.y) - v.z >= eps)
				{
					double dx = (u.x - v.x);
					double dy = (u.y - v.y);
					double dis = dx * dx + dy * dy;
					if (dis < minDis)
					{
						minDis = dis;
						minIndex = i;
					}
				}
			}

			if (minIndex != -1)
				m.Flag[minIndex] = 1;

			return minIndex;
		}

        //////////////////////////////////////////////////////////////////////////
        // NEW_ADD
        public void SaveCamera(StreamWriter sw)
        {
            for (int i = 0; i < 4; ++i)
                for (int j = 0; j < 4; ++j)
                    sw.WriteLine(this.currTransformation[i, j]);
        }
        public void LoadCamera(StreamReader sr)
        {
            this.currTransformation = new Matrix4d();

            for (int k = 0; k < 16; ++k)
            {
                string s = sr.ReadLine();
                int i = k / 4;
                int j = k % 4;
                this.currTransformation[i, j] = double.Parse(s);
            }
        }

		// divid mesh into components, and store them
		public List<List<MeshComponent>> originalMeshComponents = null;
		public List<MeshComponent> metaComponents = null;
		public List<MeshComponent> meshComponents = null;
		public List<MeshComponent> selectedMeshComponents = new List<MeshComponent>();
	
		// for ctrl-z use
		public List<MeshComponent> lastSelectedMeshComponents = new List<MeshComponent>();

		public List<MeshComponent> DividMeshIntoCompnents(Mesh m)
		{
			List<MeshComponent> allcomponents = new List<MeshComponent>();

			Mesh mesh = m;

			int n = mesh.VertexCount;
			bool[] visited = new bool[n];
			double[] pos = mesh.VertexPos;
			for (int i = 0; i < n; ++i)
			{
				if (!visited[i])
				{
					visited[i] = true;
					List<int> vertices = new List<int>();
					Queue<int> Q = new Queue<int>();
					Q.Enqueue(i);
					while (Q.Count > 0)
					{
						int s = Q.Dequeue();
						vertices.Add(s);
						foreach (int v in mesh.AdjVV[s])
						{
							if (!visited[v])
							{
								visited[v] = true;
								Q.Enqueue(v);
							}
						}
					}

					Set<int> faces = new Set<int>();
					if (vertices.Count < 2)
						continue;
					foreach (int v in vertices)
					{
						foreach (int f in mesh.AdjVF[v])
						{
							faces.Add(f);
						}
					}

					List<double> vpos = new List<double>();
					foreach (int v in vertices)
					{
						int b = v*3;
						vpos.Add(pos[b]);
						vpos.Add(pos[b+1]);
						vpos.Add(pos[b+2]);
					}

					MeshComponent component = new MeshComponent(vertices.ToArray(), vpos.ToArray(), faces.ToArray());
					component.CreateMesh(m);
					component.FindBoundingBox();
					allcomponents.Add(component);
				}
			}
			// label index of components
			this.AssignMeshComponnetIndex(allcomponents);
			this.AssignMeshComponnetColor(allcomponents);

			foreach (MeshComponent mc in allcomponents)
			{
				mc.InitComponentGroupIndices();
			}

			return allcomponents;

		}
		public void MergeCoherentPatches()
		{
			if (this.meshRecord.Mesh == null || this.meshComponents == null) return;
			Mesh m = this.meshRecord.Mesh;

			// build kdtree for the mesh vertices, for fast query
			double[,] vpos = new double[m.VertexCount, 3];
			for (int i = 0,j=0; i < m.VertexCount; ++i,j+=3)
			{
				vpos[i,0] = m.VertexPos[j];
				vpos[i,1] = m.VertexPos[j+1];
				vpos[i,2] = m.VertexPos[j+2];
			}
			int[] inputtags = new int[m.VertexCount];
			for (int i = 0; i < m.VertexCount; ++i)
				inputtags[i] = i;

			int nx = 3;
			int ny = 0;
			int normtype = 2;
			alglib.kdtree kdt;
			double[,] r = new double[0, 0];

			int[] tags = new int[0];
			
			//
			Program.OutputText("building kd tree ... ", true);
			alglib.kdtreebuildtagged(vpos,inputtags,nx,ny,normtype,out kdt);


			Program.OutputText("looking for coherent patches ... ", true);
			while (true)
			{
				// build a map
				int[] labels = new int[m.VertexCount];
				int startlabel = 0;
				foreach (MeshComponent mc in this.meshComponents)
				{
					foreach (int v in mc.vertexIndex)
					{
						labels[v] = startlabel;
					}
					startlabel++;
				}

				// knn search
				MeshComponent src = null, tar = null;
				int knn = 5;
				foreach (MeshComponent mc in this.meshComponents)
				{
					bool find = false;
					foreach (int v in mc.vertexIndex)
					{
						double[] _query_pos = new double[3] {
							m.VertexPos[v*3],
							m.VertexPos[v*3+1],
							m.VertexPos[v*3+2],
						};
						int res = alglib.kdtreequeryknn(kdt, _query_pos, knn, true);
						
						alglib.kdtreequeryresultsx(kdt, ref r);
						alglib.kdtreequeryresultstags(kdt, ref tags);

						for (int j = 0; j < knn; ++j)
						{
							MeshComponent mc2 = this.meshComponents[labels[tags[j]]];
							if (mc2 == mc) continue;

							double d1 = r[j, 0] - _query_pos[0];
							double d2 = r[j, 1] - _query_pos[1];
							double d3 = r[j, 2] - _query_pos[2];
							double dd = Math.Sqrt(d1 * d1 + d2 * d2 + d3 * d3);
							if (dd < 1e-6)
							{
								src = mc;
								tar = mc2;
								find = true;
								break;
							}
						}
						if (find) break;
					}
					if (find)
					{
						break;
					}
				}

				// merge components
				if (src != null && tar != null)
				{
					List<MeshComponent> mcs = new List<MeshComponent>();
					mcs.Add(src);
					mcs.Add(tar);
					this.GroupComponents(mcs, this.meshRecord.Mesh);
				}
				else
					break;
			}
			Program.OutputText("done ... # components = " + this.meshComponents.Count, true);
		}

		private void DrawMeshComponentsBoxes()
		{
			
			if (this.meshComponents != null)
			{
				foreach (MeshComponent component in this.meshComponents)
				{
					this.DrawProxy3D(component.boundingBox, Color.CornflowerBlue, 1);
				}
			}
			foreach (MeshComponent component in this.selectedMeshComponents)
			{
				this.DrawProxy3D(component.boundingBox, Color.Lime, 1);
			}
		}
	
		private void RenderSolidSphere(Vector3d p, double radius, Color c)
		{
		//	Gl.glEnable(Gl.GL_LIGHTING);
			
			Glu.GLUquadric quad = Glu.gluNewQuadric();
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glPushMatrix();
			Gl.glTranslated(p.x, p.y, p.z);
			Glu.gluSphere(quad, radius, 30, 30);
			Gl.glPopMatrix();
			Glu.gluDeleteQuadric(quad);

		//	Gl.glDisable(Gl.GL_LIGHTING);
		}

		public MeshComponent GroupComponents(List<MeshComponent> components, Mesh mesh)
		{
			if (components.Count < 1) return null;

			this.lastSelectedMeshComponents = new List<MeshComponent>();
            MeshComponent d = this.MergeComponents(components, mesh);

            // update current set of mesh components 
			foreach (MeshComponent comp in components)
			{
				this.lastSelectedMeshComponents.Add(comp);
				this.meshComponents.Remove(comp);
			}
			this.meshComponents.Add(d);
			this.selectedMeshComponents.Clear();
			this.selectedMeshComponents.Add(d);
			this.AssignMeshComponnetIndex(this.meshComponents);

            return d;
		}

		public void WithdrawlGrouping()
		{
			// ctrl-z   withdrawl the grouping 
			if (this.lastSelectedMeshComponents.Count == 0) return;
			// remove the one just added
			this.meshComponents.RemoveAt(this.meshComponents.Count - 1);
			foreach (MeshComponent mc in this.lastSelectedMeshComponents)
				this.meshComponents.Add(mc);
			this.AssignMeshComponnetIndex(this.meshComponents);

			foreach (MeshComponent mc in this.meshComponents)
			{
				mc.InitComponentGroupIndices();
			}
			this.lastSelectedMeshComponents.Clear();
		}

		public void DeGroupComponents(List<MeshComponent> components)
		{
			Mesh mesh = this.meshRecord.Mesh;
			int n = mesh.VertexCount;
			bool[] visited = new bool[n];
			for (int i = 0; i < n; ++i)
				visited[i] = true;

			// update current set of mesh components 
			foreach (MeshComponent comp in components)
			{
				for (int i = 0; i < comp.vertexIndex.Length; ++i)
					visited[comp.vertexIndex[i]] = false;
			}
						
			double[] pos = this.meshRecord.Mesh.VertexPos;
			for (int i = 0; i < n; ++i)
			{
				if (!visited[i])
				{
					visited[i] = true;
					List<int> vertices = new List<int>();
					Queue<int> Q = new Queue<int>();
					Q.Enqueue(i);
					while (Q.Count > 0)
					{
						int s = Q.Dequeue();
						vertices.Add(s);
						foreach (int v in mesh.AdjVV[s])
						{
							if (!visited[v])
							{
								visited[v] = true;
								Q.Enqueue(v);
							}
						}
					}

					Set<int> faces = new Set<int>();
					foreach (int v in vertices)
					{
						foreach (int f in mesh.AdjVF[v])
						{
							faces.Add(f);
						}
					}

					List<double> vpos = new List<double>();
					foreach (int v in vertices)
					{
						int b = v * 3;
						vpos.Add(pos[b]);
						vpos.Add(pos[b + 1]);
						vpos.Add(pos[b + 2]);
					}

					MeshComponent component = new MeshComponent(vertices.ToArray(), vpos.ToArray(), faces.ToArray());
					component.CreateMesh(mesh);
					component.FindBoundingBox();
					this.meshComponents.Add(component);
				}
			}
			// label index of components
			this.AssignMeshComponnetIndex(this.meshComponents);

			foreach (MeshComponent mc in this.meshComponents)
			{
				mc.InitComponentGroupIndices();
			}

			this.selectedMeshComponents.Clear();
		}//end-de-grouping

		public void Clear()
		{
			this.meshComponents = null;
			this.meshRecord = null;
			this.modelGraph = null;
			this.currTransformation = Matrix4d.IdentityMatrix();
			this.Refresh();
		}

		public MeshComponent MergeComponents(List<MeshComponent> components, Mesh mesh)
		{
			if (components.Count <= 1) return null;

			List<double> pos = new List<double>();
			List<int> findex = new List<int>();
			List<int> vindex = new List<int>();
			List<int> componentsGroupedInfo = new List<int>();
			foreach (MeshComponent comp in components)
			{
				pos.AddRange(comp.vertexPos);
				findex.AddRange(comp.faceIndex);
				vindex.AddRange(comp.vertexIndex);
				componentsGroupedInfo.AddRange(comp.componentsGroupIndices);
			}
			MeshComponent d = new MeshComponent(vindex, pos, findex);
			d.FindBoundingBox();
			d.colorIdx = components[0].colorIdx;
			d.componentsGroupIndices = componentsGroupedInfo;

			// create mesh
			if (this.meshRecord.Mesh != null)
				d.CreateMesh(mesh);
				//d.CreateMesh(this.meshRecord.Mesh);

			return d;
		
		}
		public void SaveComponentGroupInfo()
		{
			if (this.meshComponents == null) return;

			string file = @".\data\componentGroupInfo.txt";
			StreamWriter sw = new StreamWriter(file,true);

			sw.WriteLine("# " + @".\data\components\" + 
				this.meshRecord.objName + " " + this.meshComponents.Count);
			foreach (MeshComponent mc in this.meshComponents)
			{
				foreach (int v in mc.componentsGroupIndices)
				{
					sw.Write(v.ToString() + " ");
				}
				sw.WriteLine();
			}

			sw.Close();

			Program.OutputText("saved file to " + file, true);
		}

		public void DecomposeBySeg()
		{
			if (this.faceClusterIndex == null || this.meshRecord.Mesh == null) return;

			Mesh m = this.meshRecord.Mesh;
			int N = 0;
			foreach (int v in this.faceClusterIndex)
			{
				if (N < v)
				{
					N = v;
				}
			}
			List<int>[] facesgroups = new List<int>[N+1];
			for (int i = 0; i < N + 1; ++i)
			{
				facesgroups[i] = new List<int>();
			}
			int f = 0;
			foreach (int i in this.faceClusterIndex)
			{
				facesgroups[i].Add(f++);
			}

			double[] vpos = m.VertexPos;
			int[] findice = m.FaceIndex;
			
			List<MeshComponent> allcomponents = new List<MeshComponent>();

			foreach (List<int> fgrp in facesgroups)
			{
				List<int> faces = new List<int>();
				Set<int> vrts = new Set<int>();
				List<double> vps = new List<double>();
				foreach (int ff in fgrp)
				{
					faces.Add(ff);
					int b = ff*3;
					int v1 = findice[b];
					int v2 = findice[b+1];
					int v3 = findice[b+2];
					vrts.Add(v1);
					vrts.Add(v2);
					vrts.Add(v3);
				}
				foreach (int v in vrts)
				{
					vps.Add(vpos[v * 3]);
					vps.Add(vpos[v * 3+1]);
					vps.Add(vpos[v * 3+2]);
				}

				MeshComponent cmp = new MeshComponent(vrts.ToList(), vps, faces);
				cmp.CreateMesh(m);
				cmp.FindBoundingBox();
				if (this.modelType == 0)
					cmp.SamplePointsOnBbox();
				allcomponents.Add(cmp);
			}

			this.AssignMeshComponnetIndex(allcomponents);
			this.AssignMeshComponnetColor(allcomponents);

			foreach (MeshComponent mc in allcomponents)
			{
				mc.InitComponentGroupIndices();
			}

			this.meshComponents = allcomponents;

		}
		public void AssignMeshComponnetIndex(List<MeshComponent> components)
		{
			int index = 0;
			foreach (MeshComponent comp in components)
			{
				comp.componentIndex = index++;
			}
		}
		public void AssignMeshComponnetColor(List<MeshComponent> components)
		{
			int index = 0;
			foreach (MeshComponent comp in components)
			{
				comp.colorIdx = index++ % colorMall.Length;
			}
		}
		
		private void DrawLine(Vector3d u, Vector3d v, Color c)
		{
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glLineWidth(2.0f);
			Gl.glBegin(Gl.GL_LINES);
			Gl.glVertex3d(u.x, u.y, u.z);
			Gl.glVertex3d(v.x, v.y, v.z);
			Gl.glEnd();
			Gl.glLineWidth(1.0f);
		}
		private void DrawProxy3D_Flat(PolyProxy cuboid, Color c, float width)
		{
			if (cuboid == null) return;

		//	Gl.glEnable(Gl.GL_LIGHTING);
			Gl.glDisable(Gl.GL_NORMALIZE);
			
			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);
			Gl.glColor3ub(c.R, c.G, c.B);
			for (int i = cuboid.polygons.Count - 1; i >= 0; --i)
			{
				Gl.glBegin(Gl.GL_POLYGON);
				Polygon poly = cuboid.polygons[i];
				foreach (Point3 pt in poly.points3)
				{
					Gl.glVertex3d(pt.pos.x, pt.pos.y, pt.pos.z);
				}
				Gl.glEnd();
			}
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
		//	Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_NORMALIZE);
			
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glLineWidth(width);
			Gl.glBegin(Gl.GL_LINES);
			foreach (Polygon poly in cuboid.polygons)
			{
				for (int i = 0; i < poly.points3.Count; ++i)
				{
					Point3 p = poly.points3[i];
					Point3 q = poly.points3[(i + 1) % poly.points3.Count];
					Gl.glVertex3d(p.pos.x, p.pos.y, p.pos.z);
					Gl.glVertex3d(q.pos.x, q.pos.y, q.pos.z);
				}
			}
			Gl.glEnd();
			Gl.glLineWidth(1.0f);
			Gl.glEnable(Gl.GL_LIGHTING);

		}
		private void DrawProxy3D(PolyProxy cuboid, Color c, float width)
		{
			if (cuboid == null) return;

			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glColor3ub(c.R, c.G, c.B);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glLineWidth(width);
			Gl.glBegin(Gl.GL_LINES);
			foreach (Polygon poly in cuboid.polygons)
			{
				for (int i = 0; i < poly.points3.Count; ++i)
				{
					Point3 p = poly.points3[i];
					Point3 q = poly.points3[(i + 1) % poly.points3.Count];
					Gl.glVertex3d(p.pos.x, p.pos.y, p.pos.z);
					Gl.glVertex3d(q.pos.x, q.pos.y, q.pos.z);
				}
			}
			Gl.glEnd();
			Gl.glLineWidth(1.0f);
			Gl.glEnable(Gl.GL_LIGHTING);
			
		}
		private void DrawPolygon_Transparent(Polygon plg, Color c, float width)
		{
			if (plg == null) return;

			Gl.glShadeModel(Gl.GL_FLAT);
			Gl.glDisable(Gl.GL_NORMALIZE);
			Gl.glDisable(Gl.GL_LIGHTING);

			Gl.glEnable(Gl.GL_BLEND);
			Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);

			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);

			Color temp = Color.FromArgb(40, c);
			Gl.glColor4ub(temp.R, temp.G, temp.B, temp.A);

			Gl.glBegin(Gl.GL_POLYGON);
			foreach (Point3 pt in plg.points3)
			{
				Gl.glVertex3d(pt.pos.x, pt.pos.y, pt.pos.z);
			}
			Gl.glEnd();


			// draw lines
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glHint(Gl.GL_LINE_SMOOTH, Gl.GL_NICEST);
			Gl.glColor3ub(92, 92, 92);
			Gl.glLineWidth(1.0f);
			Gl.glBegin(Gl.GL_LINES);
			for (int i = 0; i < plg.points3.Count; ++i)
			{
				Point3 p = plg.points3[i];
				Point3 q = plg.points3[(i + 1) % plg.points3.Count];
				Gl.glVertex3d(p.pos.x, p.pos.y, p.pos.z);
				Gl.glVertex3d(q.pos.x, q.pos.y, q.pos.z);
			}
			Gl.glEnd();
			Gl.glLineWidth(1.0f);
			Gl.glDisable(Gl.GL_LINE_SMOOTH);

			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);
			//	Gl.glDisable(Gl.GL_BLEND);

		}
		private void DrawProxy_Transparent(PolyProxy cuboid, Color c, float width)
		{
			if (cuboid == null) return;

			Gl.glShadeModel(Gl.GL_FLAT);
			Gl.glDisable(Gl.GL_NORMALIZE);
			Gl.glDisable(Gl.GL_LIGHTING);

			Gl.glEnable(Gl.GL_BLEND);
			Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);
			
			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);

			Color temp = Color.FromArgb(40, c);
			Gl.glColor4ub(temp.R, temp.G, temp.B, temp.A);
			for (int i = cuboid.polygons.Count - 1; i >= 0; --i)
			{
				Gl.glBegin(Gl.GL_POLYGON);
				Polygon poly = cuboid.polygons[i];
				foreach (Point3 pt in poly.points3)
				{
					Gl.glVertex3d(pt.pos.x, pt.pos.y, pt.pos.z);
				}
				Gl.glEnd();
			}


			// draw lines
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glHint(Gl.GL_LINE_SMOOTH, Gl.GL_NICEST);
			Gl.glColor3ub(92, 92, 92);
			Gl.glLineWidth(1.0f);
			Gl.glBegin(Gl.GL_LINES);
			foreach (Polygon poly in cuboid.polygons)
			{
				for (int i = 0; i < poly.points3.Count; ++i)
				{
					Point3 p = poly.points3[i];
					Point3 q = poly.points3[(i + 1) % poly.points3.Count];
					Gl.glVertex3d(p.pos.x, p.pos.y, p.pos.z);
					Gl.glVertex3d(q.pos.x, q.pos.y, q.pos.z);
				}
			}
			Gl.glEnd();
			Gl.glLineWidth(1.0f);
			Gl.glDisable(Gl.GL_LINE_SMOOTH);
			
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);
			Gl.glDisable(Gl.GL_BLEND);

			// test sample points
			//DrawPoints(cuboid.sampledPointsOnBox_visual, Color.DarkBlue);
			//DrawPoints(visContactPoints, Color.DarkBlue);
		}

		private void DrawProxy_Highlight(PolyProxy cuboid, int polyIndex, int tag)
		{
			if (cuboid == null) return;

			Color c = Color.Red;
			if (tag == 2)
				c = Color.Green;

			Gl.glShadeModel(Gl.GL_FLAT);
			Gl.glDisable(Gl.GL_NORMALIZE);
			Gl.glDisable(Gl.GL_LIGHTING);

			Gl.glEnable(Gl.GL_BLEND);
			Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);

			Gl.glDisable(Gl.GL_CULL_FACE);
			Gl.glDisable(Gl.GL_DEPTH_TEST);
			Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);

			Color temp = Color.FromArgb(200, c);
			Gl.glColor4ub(temp.R, temp.G, temp.B, temp.A);
		
				Gl.glBegin(Gl.GL_POLYGON);
				Polygon poly = cuboid.polygons[polyIndex];
				foreach (Point3 pt in poly.points3)
				{
					Gl.glVertex3d(pt.pos.x, pt.pos.y, pt.pos.z);
				}
				Gl.glEnd();
		
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glEnable(Gl.GL_DEPTH_TEST);
			Gl.glDisable(Gl.GL_BLEND);
		}

		private void SelectComponentsByRect()
		{
			if (this.meshComponents == null)
			{
				return;
			}

			Vector2d minV = Vector2d.Min(mouseDownPosition, currMousePosition);
			Vector2d size = Vector2d.Max(mouseDownPosition, currMousePosition) - minV;
			Rectangle rect = new Rectangle((int)minV.x, (int)minV.y, (int)size.x, (int)size.y);
			Rectangle viewport = new Rectangle(0, 0, this.Width, this.Height);
			this.ObtainGLProjector();
			OpenGLProjector projector = this.glProjector;

			List<MeshComponent> components = this.meshComponents;
			bool laser = false;
			double eps = this.depthTolerance;

			if ((Control.ModifierKeys & Keys.Shift) == Keys.Shift)
			{
				foreach (MeshComponent component in components)
				{
					if (this.selectedMeshComponents.Contains(component)) continue;
					foreach (Point3 pt in component.boundingBox.points3)
					{
						Vector3d v = projector.Project(pt.pos);
						if (viewport.Contains((int)v.x, (int)v.y))
						{
							bool flag = rect.Contains((int)v.x, (int)v.y);
							flag &= (laser || projector.GetDepthValue((int)v.x, (int)v.y) - v.z >= eps);
							if (flag)
							{
								this.selectedMeshComponents.Add(component);
								break;
							}
						}
					}
				}
			}
			else if ((Control.ModifierKeys & Keys.Control) == Keys.Control)
			{
				foreach (MeshComponent component in components)
				{
					if (!this.selectedMeshComponents.Contains(component)) continue;
					foreach (Point3 pt in component.boundingBox.points3)
					{
						Vector3d v = projector.Project(pt.pos);
						if (viewport.Contains((int)v.x, (int)v.y))
						{
							bool flag = rect.Contains((int)v.x, (int)v.y);
							flag &= (laser || projector.GetDepthValue((int)v.x, (int)v.y) - v.z >= eps);
							if (flag)
							{
								this.selectedMeshComponents.Remove(component);
								break;
							}
						}
					}
				}
			}
			else
			{
				this.selectedMeshComponents.Clear();
				foreach (MeshComponent component in components)
				{
					//Vector3d v = projector.Project(component.boundingBox.center.pos);
					foreach (Point3 pt in component.boundingBox.points3)
					{
					    Vector3d v = projector.Project(pt.pos);
						if (viewport.Contains((int)v.x, (int)v.y))
						{
							bool flag = rect.Contains((int)v.x, (int)v.y);
							flag &= (laser || projector.GetDepthValue((int)v.x, (int)v.y) - v.z >= eps);
							if (flag)
							{
								this.selectedMeshComponents.Add(component);
								break;
							}
						}
					}
				}
			}
		}
		
		private MeshComponent SelectComponentsByPoint(Vector2d mousepos)
		{
			if (this.meshComponents == null) return null;


			Rectangle viewport = new Rectangle(0, 0, this.Width, this.Height);
			this.ObtainGLProjector();
			OpenGLProjector projector = this.glProjector;

			List<MeshComponent> components = this.meshComponents;
			double eps = this.depthTolerance;

			double mindis = double.MaxValue; MeshComponent selected = null;
			foreach (MeshComponent component in components)
			{
				foreach (Point3 pt in component.boundingBox.points3)
				{
					Vector2d v = projector.Project(pt.pos).XY();
					double dis = (v - mousepos).Length();
					if (dis < mindis && dis < 500)
					{
						selected = component;
						mindis = dis;
					}
				}
			}

			if (mindis > 200) selected = null;

			return selected;
		}

		public void MoveMesh2Ground()
		{
			if (this.MeshRecord == null) return;

			Mesh m = this.MeshRecord.Mesh;

			double miny = double.MaxValue;
			for (int i = 0, j = 0; i < m.VertexCount; ++i, j += 3)
			{
				double y = m.VertexPos[j + 1];
				if (y < miny)
				{
					miny = y;
				}
			}
			miny += 0.5;

			for (int i = 0, j = 0; i < m.VertexCount; ++i, j += 3)
			{
				m.VertexPos[j + 1] -= (miny); // Y = -0.5 IS THE GROUND
			}

			if (this.meshComponents != null)
			foreach (MeshComponent mc in this.meshComponents)
			{
				for (int i = 0, j = 0; i < mc.vertexCount; ++i, j += 3)
				{
					mc.vertexPos[j + 1] -= miny;
				}
				mc.boundingBox.center.pos.y -= (miny);
			}

		}

		private void DrawRect(StreamWriter sw)
		{
			sw.WriteLine("100 100 moveto 100 600 lineto stroke ");
			sw.WriteLine("100 600 moveto 600 600 lineto stroke ");
			sw.WriteLine("600 600 moveto 600 100 lineto stroke ");
			sw.WriteLine("600 100 moveto 100 100 lineto stroke ");
		}

		// static members

		public static bool PointProjectionInPolygon(Vector3d p, Polygon plg)
		{
			Vector3d c = plg.center.pos; Vector3d n1 = plg.normal;
			Vector3d e1 = plg.points3[1].pos - plg.points3[0].pos;
			Vector3d e2 = plg.points3[2].pos - plg.points3[1].pos;
			double l1 = e1.Length() / 2;
			double l2 = e2.Length() / 2;
			Vector3d pc = p - c;
			return Math.Abs(pc.Dot(e1.Normalize())) < l1 &&
				Math.Abs(pc.Dot(e2.Normalize())) < l2;
		}
		public static bool PointInTriangle(Vector3d p, Vector3d a, Vector3d b, Vector3d c)
		{
			Vector3d pa = p - a;
			Vector3d pb = p - b;
			Vector3d pc = p - c;
			Vector3d v1 = pa.Cross(pb);
			Vector3d v2 = pb.Cross(pc);
			Vector3d v3 = pc.Cross(pa);
			double sign1 = v1.Dot(v2);
			double sign2 = v2.Dot(v3);
			double sign3 = v3.Dot(v1);
			return (sign1 <= 0 && sign2 <= 0 && sign3 <= 0) || 
				(sign1 >= 0 && sign2 >= 0 && sign3 >= 0);
		}

		static double gridWidth = 2;
		public int gridAxis = 1;
		private Vector3d grid_p0 = new Vector3d(-gridWidth, 0, gridWidth);
		private Vector3d grid_p1 = new Vector3d(gridWidth, 0, gridWidth);
		private Vector3d grid_p2 = new Vector3d(-gridWidth, 0, -gridWidth);
		private Vector3d grid_p3 = new Vector3d(gridWidth, 0, -gridWidth);
		private Vector3d bottomPoint = new Vector3d(0, 0, 0);
		//private Vector3d grid_p0 = new Vector3d(-gridWidth, 0, gridWidth);
		//private Vector3d grid_p1 = new Vector3d(gridWidth, 0, gridWidth);
		//private Vector3d grid_p2 = new Vector3d(-gridWidth, 0, -gridWidth);
		//private Vector3d grid_p3 = new Vector3d(gridWidth,0, -gridWidth);
		//private Vector3d bottomPoint = new Vector3d(0,0, 0);
		private void AdjustGridOrientation()
		{
			if (this.meshRecord == null) return;

			this.bottomPoint = this.meshRecord.Mesh.MinCoord();
			Vector3d boundingBox = this.meshRecord.Mesh.MaxCoord() - bottomPoint;
			int whichAxis = 1;
			SetGridOrientation(whichAxis);
		}
		private void SetGridOrientation(int whichAxis)
		{
			switch (whichAxis)
			{
				case 0: /// x
					{
						grid_p0 = new Vector3d(this.bottomPoint.x, -gridWidth, gridWidth);
						grid_p1 = new Vector3d(this.bottomPoint.x, gridWidth, gridWidth);
						grid_p2 = new Vector3d(this.bottomPoint.x, -gridWidth, -gridWidth);
					}
					break;
				case 1: /// y
					{
						grid_p0 = new Vector3d(-gridWidth, this.bottomPoint.y, gridWidth);
						grid_p1 = new Vector3d(gridWidth, this.bottomPoint.y, gridWidth);
						grid_p2 = new Vector3d(-gridWidth, this.bottomPoint.y, -gridWidth);
					}
					break;
				case 2: /// z
					{
						grid_p0 = new Vector3d(-gridWidth, gridWidth, this.bottomPoint.z);
						grid_p1 = new Vector3d(gridWidth, gridWidth, this.bottomPoint.z);
						grid_p2 = new Vector3d(-gridWidth, -gridWidth, this.bottomPoint.z);
					}
					break;
			}

			gridAxis = whichAxis;
		}
		public void SetModel(MeshRecord rec)
		{
			meshRecord = rec;
			this.meshes.Add(rec);

			this.AdjustGridOrientation();
			this.Refresh();
		}
		public void ChangeCurrentModel(MeshRecord rec)
		{
			meshRecord = rec;
			this.AdjustGridOrientation();
			this.Refresh();
		}
		private void DrawGroundGrid()
		{
			int slices = 16;
			double dt = 2 * gridWidth / slices;
			double dz = dt;
			Gl.glDisable(Gl.GL_LIGHTING);
		//	Gl.glEnable(Gl.GL_BLEND);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			Gl.glColor3ub(192, 192, 192);

			Gl.glLineWidth(1.0f);
			Gl.glBegin(Gl.GL_LINES);
			for (int i = 0; i <= slices; ++i)
			{
				switch (gridAxis)
				{
					case 0:
						{
							Gl.glVertex3d(grid_p0.x, grid_p0.y, grid_p0.z - dz * i);
							Gl.glVertex3d(grid_p1.x, grid_p1.y, grid_p1.z - dz * i);
						}
						break;
					case 1:
						{
							Gl.glVertex3d(grid_p0.x, grid_p0.y, grid_p0.z - dz * i);
							Gl.glVertex3d(grid_p1.x, grid_p1.y, grid_p1.z - dz * i);
						}
						break;
					case 2:
						{
							Gl.glVertex3d(grid_p0.x, grid_p0.y - dz * i, grid_p0.z);
							Gl.glVertex3d(grid_p1.x, grid_p1.y - dz * i, grid_p1.z);
						}
						break;
				}

			}
			Gl.glEnd();

			Gl.glBegin(Gl.GL_LINES);
			for (int i = 0; i <= slices; ++i)
			{
				switch (gridAxis)
				{
					case 0:
						{
							Gl.glVertex3d(grid_p0.x, grid_p0.y + dt * i, grid_p0.z);
							Gl.glVertex3d(grid_p2.x, grid_p2.y + dt * i, grid_p2.z);
						}
						break;
					case 1:
						{
							Gl.glVertex3d(grid_p0.x + dt * i, grid_p0.y, grid_p0.z);
							Gl.glVertex3d(grid_p2.x + dt * i, grid_p2.y, grid_p2.z);
						}
						break;
					case 2:
						{
							Gl.glVertex3d(grid_p0.x + dt * i, grid_p0.y, grid_p0.z);
							Gl.glVertex3d(grid_p2.x + dt * i, grid_p2.y, grid_p2.z);
						}
						break;
				}
			}
			Gl.glEnd();
			Gl.glEnable(Gl.GL_LIGHTING);
		}


		////////////////////////////// Graph Operation --- Han (2013-12-24)//////////////////////////////
		//************************* Variables *************************//
		public ModelGraph modelGraph = null;
		public bool drawModelGraph = true;
		public bool isGraphView = false;
		public bool showNodeTypeGroup = false; // label component
		public GraphGroup graphGroup = new GraphGroup();

		public ModelGraph originalModelGraph = null;
		public bool showSelectedBoxVertex = false;
		public List<Vector3d> selectedboxVertices = new List<Vector3d>();
		public List<ModelGraphNode> selectedNodes = new List<ModelGraphNode>();
		// load selected type of models, e.g., chairs
		public List<ModelGraph> modelGraphs = new List<ModelGraph>();
		public int modelType = -1;
		//***For visualization***//
		public List<Vector3d> visContactPoints = new List<Vector3d>(); 
		//***TEST***//
		public int mcIdx = -1;
		public List<DualEdge> orderDEs = null;
		public List<List<ModelGraphEdge>> cutPairs = null;
		public List<ModelGraphEdge> cutSet = null;
		//************************* Functions *************************//

		//public void FindBestTransform(List<Vector3d> srcpts, List<Vector3d> tarpts, out Matrix4d S, out Matrix4d T, out Matrix4d Q)
		//{
		//    Q = S = T = Matrix4d.IdentityMatrix();

		//    // translation and scaling only, no rotation
		//    int n = srcpts.Count;
	
		//    MatrixNd mat = new MatrixNd(n * 3, 6);
		//    MatrixNd b = new MatrixNd(n * 3, 1);
		//    for (int i = 0, j = 0; i < n; ++i, j += 3)
		//    {
		//        Vector3d s = srcpts[i], t = tarpts[i];
		//        mat[j, 0] = s.x; mat[j, 3] = 1; b[j, 0] = t.x;
		//        mat[j + 1, 1] = s.y; mat[j + 1, 4] = 1; b[j + 1, 0] = t.y;
		//        mat[j + 2, 2] = s.z; mat[j + 2, 5] = 1; b[j + 2, 0] = t.z;
		//    }
		//    MatrixNd AT = mat.Transpose();
		//    MatrixNd ATA = AT.Mul(mat);	// 6x6 matrix
		//    MatrixNd ATb = AT.Mul(b);
		//    double[,] A = new double[6, 6];
		//    for (int i = 0; i < 6; ++i)
		//    {
		//        for (int j = 0; j < 6; ++j)
		//        {
		//            A[i, j] = ATA[i, j];
		//        }
		//    }
		//    // solve
		//    int info;
		//    alglib.matinvreport rep;
		//    alglib.rmatrixinverse(ref A, out info, out rep);
		//    if (info > 0) // matrix is not singular
		//    {
		//        for (int i = 0; i < 6; ++i)
		//        {
		//            for (int j = 0; j < 6; ++j)
		//            {
		//                ATA[i, j] = A[i, j];
		//            }
		//        }
		//        MatrixNd res = ATA.Mul(ATb);
		//        Q = new Matrix4d();
		//        Q[0, 0] = res[0, 0];
		//        Q[1, 1] = res[1, 0];
		//        Q[2, 2] = res[2, 0];
		//        Q[0, 3] = res[3, 0];
		//        Q[1, 3] = res[4, 0];
		//        Q[2, 3] = res[5, 0];
		//        Q[3, 3] = 1.0;
		//    }
		//    else
		//    {
		//        Vector3d c1 = new Vector3d(), c2 = new Vector3d(),
		//            v1 = new Vector3d(), v2 = new Vector3d();
			
		//        double ss = 1;
		//        S = Matrix4d.ScalingMatrix(1, 1, ss);
		//        Vector3d z_axis = new Vector3d(), x_axis = new Vector3d();
		//        int ind = 0;
		//        while (ind < n)
		//        {
		//            c1 = (srcpts[ind] + srcpts[n - 1]) / 2;
		//            c2 = (tarpts[0] + tarpts[n - 1]) / 2;
		//            v1 = srcpts[n - 1] - srcpts[0];
		//            v2 = tarpts[n - 1] - tarpts[0];
		//            if (v1.Dot(v2) < 0) v1 = new Vector3d() - v1;
		//            ss = v2.Length() / v1.Length();
		//            S = Matrix4d.ScalingMatrix(1, 1, ss);
		//            z_axis = v1.Normalize();
		//            x_axis = new Vector3d(v1.y, -v1.x, 0).Normalize();
		//            if (v1.x != 0 && v1.y != 0)
		//                break;
		//        }
		//        if (v1.x == 0 && v1.y == 0)
		//        {
		//            x_axis = new Vector3d(1, 0, 0);
		//        }
		//        Vector3d y_axis = z_axis.Cross(x_axis).Normalize();
		//        Matrix3d R3 = new Matrix3d(x_axis, y_axis, z_axis);
		//        Matrix4d R4 = new Matrix4d(R3);
		//        R4[3, 3] = 1.0;
		//        S = R4 * S * R4.Inverse();

		//        // the rotation matrix only if angle is altered a certain degree
		//        Matrix4d R = Matrix4d.IdentityMatrix();
		//        double cos = v1.Normalize().Dot(v2.Normalize());
		//        if (cos < Math.Cos(1.0 / 18 * Math.PI))
		//        {
		//            Vector3d axis = v1.Cross(v2).Normalize();
		//            double theta = Math.Acos(cos);
		//            R = Matrix4d.RotationMatrix(axis, theta);
		//        }
		//        T = Matrix4d.TranslationMatrix(c2 - c1);
		//        Q = Matrix4d.TranslationMatrix(c2) * R * S * Matrix4d.TranslationMatrix(new Vector3d() - c1);
		//    }
		//}


		public void FindBestTransform(List<Vector3d> srcpts, List<Vector3d> tarpts, out Matrix4d S, out Matrix4d T, out Matrix4d Q)
		{
			Q = S = T = Matrix4d.IdentityMatrix();

			// translation and scaling only, no rotation
			int n = srcpts.Count;
			if (n < 1)
			{
				Q = Matrix4d.IdentityMatrix();
				return;
			}
			else if (n == 1)
			{
				Q = Matrix4d.TranslationMatrix(tarpts[0] - srcpts[0]);
			}
			else if (n == 2)
			{
				Vector3d c1 = (srcpts[0] + srcpts[1]) / 2;
				Vector3d c2 = (tarpts[0] + tarpts[1]) / 2;
				Vector3d v1 = srcpts[1] - srcpts[0];
				Vector3d v2 = tarpts[1] - tarpts[0];
				if (v1.Dot(v2) < 0) v1 = new Vector3d() - v1;

				// scale along axis v1
				double ss = v2.Length() / v1.Length();
				S = Matrix4d.ScalingMatrix(1, 1, ss);
				Vector3d z_axis = v1.Normalize();
				Vector3d x_axis = new Vector3d(v1.y, -v1.x, 0).Normalize();
				if (v1.x == 0 && v1.y == 0)
				{
					x_axis = new Vector3d(1, 0, 0);
				}
				Vector3d y_axis = z_axis.Cross(x_axis).Normalize();
				Matrix3d R3 = new Matrix3d(x_axis, y_axis, z_axis);
				Matrix4d R4 = new Matrix4d(R3);
				R4[3, 3] = 1.0;
				S = R4 * S * R4.Inverse();

				// the rotation matrix only if angle is altered a certain degree
				Matrix4d R = Matrix4d.IdentityMatrix();
				double cos = v1.Normalize().Dot(v2.Normalize());
				if (cos < Math.Cos(1.0 / 18 * Math.PI))
				{
					Vector3d axis = v1.Cross(v2).Normalize();
					double theta = Math.Acos(cos);
					R = Matrix4d.RotationMatrix(axis, theta);
				}
				T = Matrix4d.TranslationMatrix(c2 - c1);
				Q = Matrix4d.TranslationMatrix(c2) * R * S * Matrix4d.TranslationMatrix(new Vector3d() - c1);
			}
			else //if (n == 3)
			{
				this.FindBestTransform2(srcpts, tarpts, out S, out T, out Q);
			}
			//else // l-s solver
			//{
			//    MatrixNd mat = new MatrixNd(n * 3, 6);
			//    MatrixNd b = new MatrixNd(n * 3, 1);
			//    for (int i = 0, j = 0; i < n; ++i, j += 3)
			//    {
			//        Vector3d s = srcpts[i], t = tarpts[i];
			//        mat[j, 0] = s.x; mat[j, 3] = 1; b[j, 0] = t.x;
			//        mat[j + 1, 1] = s.y; mat[j + 1, 4] = 1; b[j + 1, 0] = t.y;
			//        mat[j + 2, 2] = s.z; mat[j + 2, 5] = 1; b[j + 2, 0] = t.z;
			//    }
			//    MatrixNd AT = mat.Transpose();
			//    MatrixNd ATA = AT.Mul(mat);	// 6x6 matrix
			//    MatrixNd ATb = AT.Mul(b);
			//    double[,] A = new double[6, 6];
			//    for (int i = 0; i < 6; ++i)
			//    {
			//        for (int j = 0; j < 6; ++j)
			//        {
			//            A[i, j] = ATA[i, j];
			//        }
			//    }
			//    // solve
			//    int info;
			//    alglib.matinvreport rep;
			//    alglib.rmatrixinverse(ref A, out info, out rep);
			//    if (info > 0) // matrix is not singular
			//    {
			//        for (int i = 0; i < 6; ++i)
			//        {
			//            for (int j = 0; j < 6; ++j)
			//            {
			//                ATA[i, j] = A[i, j];
			//            }
			//        }
			//        MatrixNd res = ATA.Mul(ATb);
			//        Q = new Matrix4d();
			//        Q[0, 0] = res[0, 0];
			//        Q[1, 1] = res[1, 0];
			//        Q[2, 2] = res[2, 0];
			//        Q[0, 3] = res[3, 0];
			//        Q[1, 3] = res[4, 0];
			//        Q[2, 3] = res[5, 0];
			//        Q[3, 3] = 1.0;
			//    }
			//    else
			//    {
			//        //		Program.OutputText("matrix singular, using heuristics",true);
			//        this.FindBestTransform2(srcpts, tarpts, out S, out T, out Q);
			//    }
			//}
		}
		public void FindBestTransform2(List<Vector3d> srcpts, List<Vector3d> tarpts, out Matrix4d S, out Matrix4d T, out Matrix4d Q)
		{
			int n = srcpts.Count;

			Vector3d t1 = new Vector3d();
			Vector3d t2 = new Vector3d();

            Vector3d maxt1 = new Vector3d(double.MinValue, double.MinValue, double.MinValue),
                maxt2 = new Vector3d(double.MinValue, double.MinValue, double.MinValue),
                mint1 = new Vector3d(double.MaxValue, double.MaxValue, double.MaxValue),
                mint2 = new Vector3d(double.MaxValue, double.MaxValue, double.MaxValue);
            foreach (Vector3d tt in srcpts)
            {
                t1 += tt;
                maxt1 = Vector3d.Max(maxt1, tt);
                mint1 = Vector3d.Min(maxt1, tt);
            }
            foreach (Vector3d tt in tarpts)
            {
                t2 += tt;
                maxt2 = Vector3d.Max(maxt2, tt);
                mint2 = Vector3d.Min(maxt2, tt);
            }
			t1 /= srcpts.Count;
			t2 /= tarpts.Count;

			Vector3d trans = t2 - t1;
			T = Matrix4d.TranslationMatrix(trans);

			// find the scales
			int k = srcpts.Count;
			double sx = 0, sy = 0, sz = 0;
			for (int i = 0; i < k; ++i)
			{
				Vector3d p1 = srcpts[i] - t1;
				Vector3d p2 = tarpts[i] - t2;
				sx += p2.x / p1.x;
				sy += p2.y / p1.y;
				sz += p2.z / p1.z;
			}
			sx /= k;
			sy /= k;
			sz /= k;

			if (double.IsNaN(sx) || double.IsInfinity(sx))
			{
				sx = 1.0;
			}
			if (double.IsNaN(sy) || double.IsInfinity(sy))
			{
				sy = 1.0;
			}
			if (double.IsNaN(sz) || double.IsInfinity(sz))
			{
				sz = 1.0;
			}

            Vector3d scale1 = maxt1 - mint1, scale2 = maxt2 - mint2;
            sx = scale2.x / scale1.x;
            sy = scale2.y / scale1.y;
            sz = scale2.z / scale1.z;

			Vector3d scale = new Vector3d(sx, sy, sz);
            
  

			if (double.IsNaN(scale.x) || double.IsNaN(trans.x)) throw new Exception();

			S = Matrix4d.IdentityMatrix();
			Vector3d v1 = srcpts[1] - srcpts[0];
			Vector3d v2 = tarpts[1] - tarpts[0];
			Matrix4d R = Matrix4d.IdentityMatrix();
			double cos = v1.Normalize().Dot(v2.Normalize());

			if (cos < Math.Cos(1.0 / 18 * Math.PI))
			{
				Vector3d axis = v1.Cross(v2).Normalize();
				double theta = Math.Acos(cos);
				R = Matrix4d.RotationMatrix(axis, theta);
			}
			T = Matrix4d.TranslationMatrix(t2 - t1);
			Q = Matrix4d.TranslationMatrix(t2) * R * S * Matrix4d.TranslationMatrix(new Vector3d() - t1);
		}
		// ****** Save & Load ****** //
		public void GetComponentMeshes()
		{
			if (this.meshComponents == null)
				return;
			else
			{
				foreach (MeshComponent mc in this.meshComponents)
				{
					if (mc.mesh == null)
						mc.CreateMesh(this.meshRecord.Mesh);
				}
			}
		}

		public void SaveCurrentMesh()
		{
			List<Mesh> meshes = new List<Mesh>();
			foreach (MeshComponent c in this.modelGraph.meshComponents)
			{
				if (c.mesh != null)
					meshes.Add(c.mesh);
			}
			Mesh m = Mesh.CreateMesh(meshes);
			//m.ComputeVertexNormal();
			//m.ComputeFaceNormal();
			this.meshRecord = new MeshRecord(m);
		}

		public void SaveSegDetail(string filename)
		{
			if (this.meshRecord == null || this.meshComponents == null) return;

			string file = this.meshRecord.Filename;
			int fileloc = file.LastIndexOf("\\");
			string segfile = file.Substring(0, fileloc);
			segfile += @"\graphfiles\";
			int nameLen = file.Length - 5 - fileloc;
			if (fileloc >= 0 && nameLen > 0 && fileloc + nameLen < file.Length)
				segfile += file.Substring(fileloc + 1, nameLen);

			if (filename != null)
			{
				segfile = filename + @"iter_01";
			}

			segfile += @".seg";
			StreamWriter sw = new StreamWriter(segfile);

			//sw.WriteLine(this.meshRecord.Mesh.FaceCount);
            int facecount = 0;
            foreach (MeshComponent mc in this.meshComponents)
            {
                facecount += mc.mesh.FaceCount;
            }
            sw.WriteLine(facecount);

			int cindex = 0;
            facecount = 0;
			foreach (MeshComponent mc in this.meshComponents)
			{
				//foreach (int findex in mc.mesh.FaceCount)
				for (int i = 0; i < mc.mesh.FaceCount; ++i )
				{
					int fidx = facecount + i;
					sw.WriteLine(fidx + " " + cindex);
				}
				cindex++;
				facecount += mc.mesh.FaceCount;
			}
			sw.Close();

			Program.OutputText("saved file to" + segfile, true);
		}

		public bool LoadSegDetail()
		{
			if (this.meshRecord == null) return false;

			string file = this.meshRecord.Filename;
			int fileloc = file.LastIndexOf("\\");
			string segfile = file.Substring(0, fileloc);
			//segfile += @"\";
			segfile += @"\graphfiles\";
			int nameLen = file.Length - 5 - fileloc;
			string modelName = file.Substring(fileloc + 1, nameLen);
			segfile += modelName + @".seg";

			char[] delimiters = { ' ', '\t' };
			string s = "";

			if (!File.Exists(segfile)) return false;

			int[] clusterIndex = new int[this.meshRecord.Mesh.FaceCount];

			StreamReader sr = new StreamReader(segfile);
			s = sr.ReadLine();
			string[] tokens = s.Split(delimiters);
			int N = int.Parse(tokens[0]);

			if (this.meshRecord.Mesh.FaceCount != N)
			{
				Program.OutputText("Wrong face number!", true);
				return false;
			}
			for (int i = 0; i < N; ++i)
			{
				s = sr.ReadLine();
				tokens = s.Split(delimiters);

				int findex = int.Parse(tokens[0]);
				int cindex = int.Parse(tokens[1]);

				clusterIndex[findex] = cindex;
			}

			sr.Close();

			this.faceClusterIndex = clusterIndex;

			Program.OutputText("loaded seg file from \\" + modelName, true);

			return true;
		}

		public bool LoadComponentDetail(string dir, int tag)
		{
			// tag = 1: extract .obj files directly from the folder
			// tag = 2: read from mesh/color/graph
			// load input files
			// 1. seperate component .obj files
			// 2. material (color for each type of node)
			// 3. labels (node type) for each component
			// 4. graph (adjacency between node)

			string objDir = dir;
			if(tag == 2)
				objDir += @"\mesh\";
			string[] objfiles = Directory.GetFiles(objDir, "*.obj");
			string extension = "obj";

			// 1. seperate component .obj files
			this.meshComponents = new List<MeshComponent>();
			char[] delimiters = { '.' };
			List<Mesh> meshes = new List<Mesh>();
			int currentVertexCount = 0, currentFaceCount = 0;
			for (int fileIdx = 0; fileIdx < objfiles.Length; fileIdx++)
			{
				string fullPath = objfiles[fileIdx];
				string[] tokens = fullPath.Split(delimiters);
				string filename = Path.GetFileName(fullPath);
				int findex = fullPath.LastIndexOf('\\');
				string objname = fullPath.Substring(findex + 1, fullPath.Length - findex - 1);
				// load mesh model
				StreamReader sr = new StreamReader(fullPath);
				Mesh m = new Mesh(sr, extension, false);
				sr.Close();							
				meshes.Add(m);
				currentVertexCount += m.VertexCount;
				currentFaceCount += m.FaceCount;
				MeshComponent mc = new MeshComponent(m);
				//if (currentVertexCount > 0)
				//{
				//    for (int i = 0; i < mc.faceIndex.Length; ++i)
				//        mc.faceIndex[i] += currentVertexCount;
				//}
				mc.name = objname.Substring(0, objname.Length-4);
				this.meshComponents.Add(mc);
			}//end-for
			Program.OutputText("Load Mesh File From " + objDir, true);
			Mesh mesh = Mesh.CreateMesh(meshes);
			this.MeshRecord = new MeshRecord(mesh);
			this.GetComponentMeshes();
			this.AssignMeshComponnetIndex(this.meshComponents);
			this.AssignMeshComponnetColor(this.meshComponents);
			if (tag == 1)
				return true;

			int index = dir.LastIndexOf('\\');
			this.meshRecord.objName = dir.Substring(index + 1, dir.Length - index - 1);
			this.meshRecord.Filename = dir + @".obj";

			// 2. load material
			string materialFileName = dir + @"\material.txt";
			//if (!File.Exists(materialFileName)) return false;
			LoadColorFile(materialFileName);

			// 3. labels (node type) for each component
			String labelFileName = dir + @"\labels.txt";
			if (!File.Exists(labelFileName)) return false;
			LoadLabelFile(labelFileName);

			// 4. graph (adjacency between node)
			String graphFileName = dir + @"\graph_structure.txt";
			if (!File.Exists(graphFileName)) return false;
			LoadGraphFile(graphFileName);

			return true;
		}//End-load-component

		List<String> materialNames = null;
		public void LoadColorFile(string filename)
		{
            if (!File.Exists(filename)) return;
			// read material file
			StreamReader sr = new StreamReader(filename);
			char[] delimiters = { '\t', ',', '[', ']', ' ' };
			List<Color> colors = new List<Color>();
			materialNames = new List<string>();
			while (true)
			{
				string s = sr.ReadLine();
				if (s == null || s == "") break;
				if (s[0] == '#' || s[0] == '%') continue;
				s.Replace(" ", "");
				string[] tokens = s.Split(delimiters);
				if (tokens.Length == 1) continue;
				materialNames.Add(tokens[0]);
				double R = double.Parse(tokens[2]);
				double G = double.Parse(tokens[3]);
				double B = double.Parse(tokens[4]);
				int iR = (int)(R * 255.0),
					iG = (int)(G * 255.0),
					iB = (int)(B * 255.0);
				colors.Add(Color.FromArgb(iR, iG, iB));
			}
			colorMall = new Color[colors.Count];
			for (int i = 0; i < colors.Count; ++i)
			{
				colorMall[i] = colors[i];
			}
		}//End-read-material

		public void LoadLabelFile(string filename)
		{
			// read node label file
			this.modelGraph = new ModelGraph(this.meshComponents);
			this.modelGraph.mesh = new Mesh(this.meshRecord.Mesh);

			StreamReader sr = new StreamReader(filename);
			char[] delimiters = { '\t', ' ' };
			int ncomponents = this.meshComponents.Count;
			List<string> nodeNames = new List<string>();
			foreach (MeshComponent mc in this.meshComponents)
				nodeNames.Add(mc.name);
			for(int ic = 0; ic < ncomponents; ++ic)
			{
				string s = sr.ReadLine();
				if (s == null || s == "") break;
				if (s[0] == '#' || s[0] == '%') continue;
				//s = s.Trim();
				string[] tokens = s.Split(delimiters);
				if (tokens.Length == 1) continue;
				string matname = tokens[1];
				matname = matname.Replace(" ", "");
				// find corresponding material
                int matIdx = -1;
                if (materialNames != null && materialNames.Count > 0)
                {
                    matIdx = materialNames.IndexOf(matname);
                    if (matIdx == -1)
                    {
                        Program.OutputText("No such material exist!", true);
                        break;
                    }
                }
                else
                    matIdx = int.Parse(matname);
				string nodeName = tokens[0].Trim();
				int nodeIdx = nodeNames.IndexOf(nodeName);
				this.modelGraph.graphNodes[nodeIdx].name = nodeName;// this.meshComponents[ic].name;
				this.meshComponents[nodeIdx].colorIdx = matIdx;
				this.meshComponents[nodeIdx].componentIndex = ic;
				this.modelGraph.graphNodes[nodeIdx].nodeType = matIdx;
				this.modelGraph.graphNodes[nodeIdx].nodeIdx = ic;
			}
			sr.Close();
		}//End-load-label-file

		public void LoadMaterialFile(string filename)
		{
			if (!File.Exists(filename)) return;
			// read node label file
			this.modelGraph = new ModelGraph(this.meshComponents);
			this.modelGraph.mesh = new Mesh(this.meshRecord.Mesh);

			StreamReader sr = new StreamReader(filename);
			char[] delimiters = { ' ' };
			int ncomponents = this.meshComponents.Count;
			int index = 0;
			while(true)
			{
				string s = sr.ReadLine();
				if (s == null || s == "") break;
				if (s[0] == '#' || s[0] == '%') continue;
				//s = s.Trim();
				string[] tokens = s.Split(delimiters);
				if (tokens.Length == 1) continue;
				int matIdx = (int)Double.Parse(tokens[1]);
				if (matIdx == -1)
				{
					Program.OutputText("Wrong record!", true);
					break;
				}
				this.modelGraph.graphNodes[index].name = tokens[0];
				this.meshComponents[index].colorIdx = matIdx;
				this.meshComponents[index].componentIndex = index;
				this.modelGraph.graphNodes[index].nodeType = matIdx;
				this.modelGraph.graphNodes[index].nodeIdx = index;
				++index;
			}
			index = filename.LastIndexOf('\\');
			string modelName = filename.Substring(index, filename.Length-index);
			Program.OutputText("Load Material File From " + modelName, true);
		}//End-load-material-file

		List<string> nodeNames = null;
		public void LoadGraphFile(string filename)
		{
			if (!File.Exists(filename)) return;
			// collect all file names
			int ncomponent = this.meshComponents.Count;
			List<string> componentFileNames = new List<string>();
			for (int ic = 0; ic < ncomponent; ++ic)
			{
				componentFileNames.Add(this.modelGraph.graphNodes[ic].name);
				this.modelGraph.graphNodes[ic].adjNodes = new List<ModelGraphNode>();
			}
			nodeNames = new List<string>();
			nodeNames.AddRange(componentFileNames);
			StreamReader sr = new StreamReader(filename);
			char[] delimiters = { '\t', ' ' };
			int eidx = 0;
			this.modelGraph.graphEdges = new List<ModelGraphEdge>();

			while (true)
			{
				string s = sr.ReadLine();
				if (s == null || s == "") break;
				if (s[0] == '#' || s[0] == '%') continue;
				string[] tokens = s.Split(delimiters);
				if (tokens.Length == 1) continue;
				string obj1 = tokens[0], obj2 = tokens[1];
				int objIdx1 = componentFileNames.IndexOf(obj1);
				int objIdx2 = componentFileNames.IndexOf(obj2);
				if(objIdx1 == -1 || objIdx2 == -1)
				{
					Program.OutputText("Error: obj filename!", true);
					break;
				}
				ModelGraphNode mn1 = this.modelGraph.graphNodes[objIdx1];
				ModelGraphNode mn2 = this.modelGraph.graphNodes[objIdx2];
				ModelGraphEdge edge = new ModelGraphEdge(new Vector3d(), -1, eidx++);
				edge.AddConnectingNode(mn1);
				edge.AddConnectingNode(mn2);
				mn1.adjNodes.Add(mn2);
				mn2.adjNodes.Add(mn1);
				this.modelGraph.graphEdges.Add(edge);
			}

			// Translate
			this.modelGraph.mesh.ScaleToUnitBox();
			this.modelGraph.mesh.MoveToCenter();
			Vector3d minCoord = this.modelGraph.mesh.MinCoord();
			Matrix4d trans = Matrix4d.TranslationMatrix(-1.0 * minCoord);
			this.meshRecord.Mesh.Transform(trans);
			this.AdjustGridOrientation();
			this.modelGraph.mesh.Transform(trans);

			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				mn.TransForm(trans);
				mn.meshComponent.mesh.Transform(trans);
				mn.meshComponent.mesh.UpdateVertexPos();
				mn.meshComponent.boundingBox.TransFromOld(trans);
				mn.meshComponent.boundingBox.Update();
			}


			Face cycle = null;
			this.modelGraph.faceCycles = new List<Face>();

			while (true)
			{
				string s = sr.ReadLine();
				if (s == null || s == "" || s[0] == '#')
				{
					if (cycle != null)
					{
						this.modelGraph.faceCycles.Add(cycle);
						foreach (ModelGraphEdge me in this.modelGraph.graphEdges)
						{
							if (cycle.vertices.Contains(me.conntectingNodes[0]) &&
								cycle.vertices.Contains(me.conntectingNodes[1]))
								cycle.edges.Add(me);
						}
					}
					cycle = new Face();
					if (s != null && s[0] == '#') continue;
					else break;
				}
				string[] tokens = s.Split(delimiters);
				int nidx = nodeNames.IndexOf(tokens[0]);
				ModelGraphNode mn = this.modelGraph.graphNodes[nidx];
				cycle.vertices.Add(mn);
			}



			sr.Close();
			int index = filename.LastIndexOf('\\');
			string modelName = filename.Substring(index, filename.Length - index);
			Program.OutputText("Load Graph File From " + modelName, true);
		}//End-load-graph-file

		public void SaveModelGraphInfo(string filename)
		{
			if (this.meshRecord == null || this.meshComponents == null) return;
			if (this.modelGraph == null || this.modelGraph.meshComponents.Count == 0) return;
			string file = this.meshRecord.Filename;
            if (filename != null)
                file = filename;
			int fileloc = file.LastIndexOf("\\");
			string graphfile = file.Substring(0, fileloc);
            if (filename == null)
                graphfile += @"\graphfiles\";
            else graphfile += @"\";
			int nameLen = file.Length - 5 - fileloc;
			if(fileloc >= 0 && nameLen > 0 && fileloc+nameLen < file.Length)
				graphfile += file.Substring(fileloc + 1, nameLen);

			if (filename != null)
			{
				graphfile = filename;
				graphfile = filename + @"iter_01";
			}


			string matfile = graphfile.Clone() as string;
			graphfile += @".graph";
			matfile += @".material";

			StreamWriter sw = new StreamWriter(matfile);
			sw.WriteLine("#node number --- node index/color index/node type index");
			sw.WriteLine(this.modelGraph.graphNodes.Count);
			foreach(ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				sw.WriteLine(mn.name + " " + mn.nodeType);
			}
			sw.Close();
			Program.OutputText("saved node material file to" + matfile, true);
			sw = new StreamWriter(graphfile);
			sw.WriteLine("#edge number --- node 1/node 2");
			sw.WriteLine(this.modelGraph.graphEdges.Count);
			foreach(ModelGraphEdge me in this.modelGraph.graphEdges)
			{
				if (me.conntectingNodes.Count != 2)
				{
					MessageBox.Show("Wrong number of contacts components!");
					return;
				}
				ModelGraphNode mn1 = me.conntectingNodes[0];
				ModelGraphNode mn2 = me.conntectingNodes[1];
				sw.Write(mn1.name + " " + mn2.name + " ");
				sw.WriteLine();
			}
			sw.Close();
			Program.OutputText("saved graph file to" + graphfile, true);
		}

		public ModelGraph LoadModelGraphInfo(int gidx)
		{
			if (this.meshRecord == null) return null;

			string file = this.meshRecord.Filename;
			int fileloc = file.LastIndexOf("\\");
			if (fileloc < 0) return null;

			string graphfile = file.Substring(0, fileloc);
			//graphfile += @"\";
			graphfile += @"\graphfiles\";
			int nameLen = file.Length - 5 - fileloc;
			string objName = file.Substring(fileloc + 1, nameLen);
			graphfile += file.Substring(fileloc + 1, nameLen);
			string matfile = graphfile.Clone() as string;
			string contactfile = graphfile.Clone() as string;
			string cyclefile = graphfile.Clone() as string;
			string camfile = graphfile.Clone() as string;

			graphfile += @".graph";
			matfile += @".material";
			contactfile += @".con";
			cyclefile += @".cycle";
			camfile += @".cam";
			
			this.LoadMaterialFile(matfile);
			this.LoadGraphFile(graphfile);
			this.LoadContactFile(contactfile);
			this.LoadCycleFile(cyclefile);
			this.modelGraph.objfilename = objName;
            this.modelGraph.meshfile = file.Substring(0, fileloc + 1);
			if (File.Exists(camfile))
			{
				StreamReader sr = new StreamReader(camfile);
				this.LoadCamera(sr);
				sr.Close();
			}
			return this.modelGraph;
		}

		public void SaveCycleInfo(string filename)
		{
			if (this.meshRecord == null || this.meshComponents == null) return;
			if (this.modelGraph == null || this.modelGraph.meshComponents.Count == 0) return;
			string file = this.meshRecord.Filename;
			if (filename != null)
				file = filename;
			int fileloc = file.LastIndexOf("\\");

			string cyclefile = file.Substring(0, fileloc);
			if (filename == null)
				cyclefile += @"\graphfiles\";
			else
				cyclefile += @"\";
			
			if (filename == null)
			{
				int nameLen = file.Length - 5 - fileloc;
				if (fileloc >= 0 && nameLen > 0 && fileloc + nameLen < file.Length)
					cyclefile += file.Substring(fileloc + 1, nameLen);
			}
			else
				cyclefile = filename + @"iter_08"; 
			cyclefile += @".cycle";

			StreamWriter sw = new StreamWriter(cyclefile);
			int idx = 1;
			if (this.modelGraph.faceCycles != null)
			{
				foreach (Face face in this.modelGraph.faceCycles)
				{
					sw.WriteLine("#Cycle " + idx.ToString());
					foreach (ModelGraphNode mn in face.vertices)
						sw.WriteLine(mn.name);
					++idx;
				}
			}
			sw.Close();
			Program.OutputText("saved cycle file to" + cyclefile, true);
		}

		public void LoadCycleFile(string filename)
		{
			// collect all file names
			if (!File.Exists(filename)) return;
			StreamReader sr = new StreamReader(filename);
			char[] delimiters = { '\t', ' ' };

			Face cycle = null;
			this.modelGraph.faceCycles = new List<Face>();

			while (true)
			{
				string s = sr.ReadLine();
				if (s == null || s == "" || s[0] == '#')
				{
					if (cycle != null)
					{
						this.modelGraph.faceCycles.Add(cycle);
						foreach (ModelGraphEdge me in this.modelGraph.graphEdges)
						{
							if (cycle.vertices.Contains(me.conntectingNodes[0]) &&
								cycle.vertices.Contains(me.conntectingNodes[1]))
								cycle.edges.Add(me);
						}
					}
					cycle = new Face();
					if (s != null && s[0] == '#') continue;
					else break;
				}
				string[] tokens = s.Split(delimiters);
				int nidx = nodeNames.IndexOf(tokens[0]);
				ModelGraphNode mn = this.modelGraph.graphNodes[nidx];
				cycle.vertices.Add(mn);
			}
			sr.Close();

			int index = filename.LastIndexOf('\\');
			string modelName = filename.Substring(index, filename.Length - index);
			Program.OutputText("Load Contact File From " + modelName, true);
		}//End-load-contact-file

		public void SaveContactInfo(string filename)
		{
			if (this.meshRecord == null || this.meshComponents == null) return;
			if (this.modelGraph == null || this.modelGraph.meshComponents.Count == 0) return;
			string file = this.meshRecord.Filename;
            if (filename != null)
                file = filename;
			int fileloc = file.LastIndexOf("\\");
            
			string contactfile = file.Substring(0, fileloc);
            if (filename == null)
                contactfile += @"\graphfiles\";
            else
                contactfile += @"\";
			int nameLen = file.Length - 5 - fileloc;
			if (fileloc >= 0 && nameLen > 0 && fileloc + nameLen < file.Length)
				contactfile += file.Substring(fileloc + 1, nameLen);
			contactfile += @".con";

			StreamWriter sw = new StreamWriter(contactfile);
			sw.WriteLine("#contact --- face index of two connecting nodes");
			sw.WriteLine("#+-1: X, +-2: Y, +-3: Z");			
			sw.WriteLine(this.modelGraph.graphEdges.Count);
			foreach (ModelGraphEdge me in this.modelGraph.graphEdges)
			{
				if (me.conntectingNodes.Count != 2)
				{
					MessageBox.Show("Wrong number of contacts components!");
					return;
				}
				//sw.Write(MapPostivieToNeg(me.faceIndex[0]) + " " +
				//    MapPostivieToNeg(me.faceIndex[1]));
				//sw.Write(MapSignToOrder(me.faceIndex[0]) + " " +
				//    MapSignToOrder(me.faceIndex[1]));
				sw.Write(me.faceIndex[0] + " " + me.faceIndex[1]);
				sw.WriteLine();
			}
			sw.Close();
			Program.OutputText("saved contact file to" + contactfile, true);
		}

        private int MapPostivieToNeg(int dir)
        {
            int index = -1;
            switch (dir)
            {
                case 1:
                    index = 1;
                    break;
                case 3:
                    index = -1;
                    break;
                case 6:
                    index = 2;
                    break;
                case 5:
                    index = -2;
                    break;
                case 4:
                    index = 3;
                    break;
                case 2:
                    index = -3;
                    break;
            }
            return index;
        }

		public void LoadContactFile(string filename)
		{
			// collect all file names
			if (!File.Exists(filename)) return;
			StreamReader sr = new StreamReader(filename);
			char[] delimiters = { '\t', ' ' };
			int eIdx = 0;
			while(true)
			{
				string s = sr.ReadLine();
				if (s == null || s == "") break;
				if (s[0] == '#' || s[0] == '%') continue;
				string[] tokens = s.Split(delimiters);
				if (tokens.Count() != 2) continue;
				int idx1 = (int)Double.Parse(tokens[0]);
				int idx2 = (int)Double.Parse(tokens[1]);
				ModelGraphEdge me = this.modelGraph.graphEdges[eIdx++];
				me.faceIndex[0] = idx1;
				me.faceIndex[1] = idx2;
			}
			sr.Close();
			int index = filename.LastIndexOf('\\');
			string modelName = filename.Substring(index, filename.Length - index);
			Program.OutputText("Load Contact File From " + modelName, true);
		}//End-load-contact-file

		public void ExtractSupport(ref ModelGraph mg)
		{
			foreach (ModelGraphEdge me in mg.graphEdges)
			{
				if (Math.Abs(me.faceIndex[0]) == 2)
				{
					me.support = true;
				}
			}
		}//ExtractSupport

		public ModelGraph LoadModelGraphInfo_old(int gidx)
		{
			if (this.meshRecord == null) return null;

			string file = this.meshRecord.Filename;
			int fileloc = file.LastIndexOf("\\");
			string graphfile = file.Substring(0, fileloc);
			graphfile += @"\graphfiles\";
			int nameLen = file.Length - 5 - fileloc;
			string objName = file.Substring(fileloc + 1, nameLen);
			graphfile += file.Substring(fileloc + 1, nameLen);
			string matfile = graphfile.Clone() as string;
			graphfile += @".graph";
			matfile += @".material";

			char[] delimiters = { ' ', '\t' };
			string s = "";

			if (!File.Exists(graphfile)) return null;
			if (this.meshComponents.Count == 0) return null;

			ModelGraph mg;
			mg = new ModelGraph(this.meshComponents);
			mg.mesh = new Mesh(this.meshRecord.Mesh);
			//**Mesh reuse**//
			//mg.mesh = this.meshRecord.Mesh;
			mg.meshfile = file.Substring(0, file.LastIndexOf('\\')+1);
			mg.objfilename = file.Substring(fileloc + 1, nameLen);

			StreamReader sr = new StreamReader(graphfile);
			sr.ReadLine();
			sr.ReadLine();
			s = sr.ReadLine();
			string[] tokens = s.Split(delimiters);
			int nnode = int.Parse(tokens[0]);
			string nodename = @"Seg_";
			for (int i = 0; i < nnode; ++i)
			{
				s = sr.ReadLine();
				tokens = s.Split(delimiters);

				int nodeIdx = int.Parse(tokens[0]);
				int colorIdx = int.Parse(tokens[1]);
				int nodeTypeIdx = int.Parse(tokens[2]);

				mg.meshComponents[i].componentIndex = nodeIdx;
				mg.meshComponents[i].colorIdx = colorIdx;
				mg.graphNodes[i].nodeType = nodeTypeIdx;
				mg.graphNodes[i].name = nodename + i.ToString();
				mg.nodesByType[nodeTypeIdx].Add(mg.graphNodes[i]);
			}
			s = sr.ReadLine();
			tokens = s.Split(delimiters);
			int nedge = int.Parse(tokens[0]);
			mg.graphEdges = new List<ModelGraphEdge>();
			for (int i = 0; i < nedge; ++i)
			{
				s = sr.ReadLine();
				tokens = s.Split(delimiters);

				int mc1_ind = int.Parse(tokens[0]);
				int mc2_ind = int.Parse(tokens[1]);

				Vector3d pos = new Vector3d();
				for (int j = 0; j < 3; j++)
					pos[j] = double.Parse(tokens[2 + j]);

				ModelGraphNode mn1 = mg.graphNodes[mc1_ind];
				ModelGraphNode mn2 = mg.graphNodes[mc2_ind];
				ModelGraphEdge edge = new ModelGraphEdge(pos, gidx, i);
				edge.AddConnectingNode(mn1);
				edge.AddConnectingNode(mn2);
				mg.graphEdges.Add(edge);
			}
			sr.Close();
			Program.OutputText("loaded graph file", true);
			foreach (ModelGraphNode mn in mg.graphNodes)
				mn.OrderEdges(0);
			// Translate
			Vector3d minCoord = mg.mesh.MinCoord();
			Matrix4d trans = Matrix4d.TranslationMatrix(-1.0 * minCoord);
			this.meshRecord.Mesh.Transform(trans);
			this.AdjustGridOrientation();
			mg.mesh.Transform(trans);

			foreach (ModelGraphNode mn in mg.graphNodes)
			{
				mn.TransForm(trans);
				mn.meshComponent.mesh.Transform(trans);
				mn.meshComponent.mesh.UpdateVertexPos();
				mn.meshComponent.boundingBox.TransFromOld(trans);
				mn.meshComponent.boundingBox.Update();
				if (mn.edges == null) continue;
				foreach (ModelGraphEdge e in mn.edges)
				{
					if (!e.updated)
					{
						e.TransformFromCurrent(trans);
						e.UpdatePos();
						e.updated = true;
					}
				}
			}
			foreach (ModelGraphNode mn in mg.graphNodes)
			{
				if (mn.edges == null) continue;
				foreach (ModelGraphEdge e in mn.edges)
				{
					e.updated = false;
				}
			}
			//ExtractSupport(mg);
			//UpdateVisPositions(mg, gidx);
			this.visContactPoints = new List<Vector3d>();
			foreach (ModelGraphEdge e in mg.graphEdges)
				this.visContactPoints.Add(e.pos);
			return mg;
		}

		public void WriteMTLFile(StreamWriter mtlsw, ModelGraphNode mn)
		{
			// write color
			int cindex = mn.nodeType;
			mtlsw.WriteLine("newmtl colorGroup" + cindex.ToString());
			Color gcolor = colorMall[cindex];
			double[] dcolor = new double[3];
			dcolor[0] = (double)gcolor.R / 255.0;
			dcolor[1] = (double)gcolor.G / 255.0;
			dcolor[2] = (double)gcolor.B / 255.0;
			mtlsw.WriteLine("Ka " + dcolor[0].ToString() + " " +
				dcolor[1].ToString() + " " + dcolor[2].ToString());
			mtlsw.WriteLine("Kd " + dcolor[0].ToString() + " " +
				dcolor[1].ToString() + " " + dcolor[2].ToString());
			mtlsw.WriteLine("Ks " + dcolor[0].ToString() + " " +
				dcolor[1].ToString() + " " + dcolor[2].ToString());
		}

		public void WriteMTLFile(StreamWriter mtlsw)
		{
			// write color
			List<int> colorIndices = new List<int>();
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				int cindex = mn.nodeType;
				if (colorIndices.Contains(cindex)) continue;
				colorIndices.Add(cindex);
				mtlsw.WriteLine("newmtl colorGroup" + cindex.ToString());
				Color gcolor = colorMall[cindex];
				double[] dcolor = new double[3];
				dcolor[0] = (double)gcolor.R / 255.0;
				dcolor[1] = (double)gcolor.G / 255.0;
				dcolor[2] = (double)gcolor.B / 255.0;
				mtlsw.WriteLine("Ka " + dcolor[0].ToString() + " " +
					dcolor[1].ToString() + " " + dcolor[2].ToString());
				mtlsw.WriteLine("Kd " + dcolor[0].ToString() + " " +
					dcolor[1].ToString() + " " + dcolor[2].ToString());
				mtlsw.WriteLine("Ks " + dcolor[0].ToString() + " " +
					dcolor[1].ToString() + " " + dcolor[2].ToString());
			}
		}
		public void WriteMeshFile(StreamWriter sw)
		{
			int gidx = 0;			
			// write mesh
			int vertexcount = 0;

			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				Mesh mesh = mn.meshComponent.mesh;
				sw.WriteLine("g " + gidx.ToString());
				for (int i = 0, j = 0; i < mesh.VertexCount; i++, j += 3)
				{
					sw.Write("v ");
					sw.Write(mesh.VertexPos[j].ToString() + " ");
					sw.Write(mesh.VertexPos[j + 1].ToString() + " ");
					sw.Write(mesh.VertexPos[j + 2].ToString());
					sw.WriteLine();
				}
				sw.WriteLine("usemtl " + "colorGroup" + mn.nodeType.ToString());
				for (int i = 0, j = 0; i < mesh.FaceCount; i++, j += 3)
				{
					sw.Write("f ");
					sw.Write((mesh.FaceIndex[j] + vertexcount + 1).ToString() + " ");
					sw.Write((mesh.FaceIndex[j + 1] + vertexcount + 1).ToString() + " ");
					sw.Write((mesh.FaceIndex[j + 2] + vertexcount + 1).ToString());
					sw.WriteLine();
				}
				++gidx;
				vertexcount += mesh.VertexCount;
			}
		}//End-Write-Meshcomponent-mtl

		public void ExportSubgraphs(string pathname)
		{

			string foldername = pathname;
			if (this.sourceGraph != null)
				foldername += this.sourceGraph.objfilename + @"_";
			if (this.targetGraph != null)
				foldername += this.targetGraph.objfilename + @"\";
			if (!Directory.Exists(foldername))
				Directory.CreateDirectory(foldername);

			// export all subgraphs
			for (int sidx = 0; sidx < this.sourceGraph.equSubgraphs.Count; ++sidx)
			{
				string filename = foldername + @"subgraph_" + sidx.ToString();
				filename += @".subgraph";
				Subgraph a_sg = this.sourceGraph.equSubgraphs[sidx];
				Subgraph b_sg = this.targetGraph.equSubgraphs[sidx];
				//save subgraph
				SaveSubgraph(filename, a_sg, b_sg);			

				// varied geometry
				Subgraph new_asg = null, new_bsg = null;
				VaryGeometryForSubgraph(this.sourceGraph, this.targetGraph, a_sg, b_sg,
					out new_asg, out new_bsg);
				if (new_asg == null || new_bsg == null 
					|| new_asg.nodeSets.Count != a_sg.nodeSets.Count
					|| new_bsg.nodeSets.Count != b_sg.nodeSets.Count
					|| new_asg.edgeNodes.Count != a_sg.edgeNodes.Count
					|| new_bsg.edgeNodes.Count != b_sg.edgeNodes.Count)
				{
					Program.OutputText("Wrong geometry?", true);
					break;
				}
				
				//SaveSubgraph(filename, new_asg, new_bsg);
				for (int nsg = 0; nsg < 2; ++nsg)
				{
					Subgraph oriSG = a_sg, newSG = new_asg;
					filename = foldername + @"subgraph_" + sidx.ToString();
					if (nsg == 1)
					{
						oriSG = b_sg;
						newSG = new_bsg;
						filename += @"_" + this.sourceGraph.objfilename;
					}
					else
						filename += @"_" + this.targetGraph.objfilename;
					filename += @"_updateGeometry.txt";
					StreamWriter sw = new StreamWriter(filename);
					for (int i = 0; i < oriSG.nodeSets.Count; ++i)
					{
 						sw.WriteLine(oriSG.nodeSets[i].name + " " +
							newSG.nodeSets[i].name);
					}
					sw.Close();
				}
			}
			Program.OutputText("saved file to " + foldername, true);

			for (int idx = 0; idx < this.sourceGraph.equSubgraphs.Count; ++idx)
			{
				for (int i = 0; i < this.sourceGraph.equSubgraphs[idx].edgeNodes.Count; ++i)
				{
					Program.OutputText(this.sourceGraph.equSubgraphs[idx].edgeNodes[i].name, true);
					Program.OutputText(this.sourceGraph.equSubgraphs[idx].connNodes[i].name, true);
				}
			}
			// export used material
			//string matname = foldername + "material.txt";
			//int maxlabel = 0;
			//foreach (ModelGraphNode mn in this.sourceGraph.graphNodes)
			//    maxlabel = mn.nodeType > maxlabel ? mn.nodeType : maxlabel;
			//foreach (ModelGraphNode mn in this.targetGraph.graphNodes)
			//    maxlabel = mn.nodeType > maxlabel ? mn.nodeType : maxlabel;
			//StreamWriter sw_mat = new StreamWriter(matname);
			//for (int i = 0; i <= maxlabel; ++i)
			//{
			//    sw_mat.WriteLine(i.ToString() + " " + "[" +
			//        colorMall[i].R.ToString() + "," +
			//        colorMall[i].G.ToString() + "," +
			//        colorMall[i].B.ToString() + "]");
			//}
			//sw_mat.Close();
		}

		public int selectedSGIndex = -1;
		public void ExportSelfSubgraphs(string pathname)
		{

			string foldername = pathname;
			if (!Directory.Exists(foldername))
				Directory.CreateDirectory(foldername);

			if (this.modelGraph != null)
				foldername += this.modelGraph.objfilename + @"_";
			

			if (selectedSGIndex != -1)
			{
				string filename = foldername + @"subgraph_" + selectedSGIndex.ToString();
				filename += @".subgraph";
				Subgraph a_sg = this.modelGraph.equSubgraphs[selectedSGIndex * 2];
				Subgraph b_sg = this.modelGraph.equSubgraphs[selectedSGIndex * 2 + 1];
				//save subgraph
				SaveSubgraph(filename, a_sg, b_sg);
			}
			else
			{
				// export all subgraphs
				for (int sidx = 0; sidx < this.modelGraph.equSubgraphs.Count / 2; ++sidx)
				{
					string filename = foldername + @"subgraph_" + sidx.ToString();
					filename += @".subgraph";
					Subgraph a_sg = this.modelGraph.equSubgraphs[sidx * 2];
					Subgraph b_sg = this.modelGraph.equSubgraphs[sidx * 2 + 1];
					//save subgraph
					SaveSubgraph(filename, a_sg, b_sg);
				}
			}
			Program.OutputText("saved file to " + foldername, true);
			this.selectedSGIndex = -1;
			
			//for (int idx = 0; idx < this.modelGraph.equSubgraphs.Count/2; ++idx)
			//{
			//    for (int i = 0; i < this.modelGraph.equSubgraphs[idx * 2].edgeNodes.Count; ++i)
			//    {
			//        Program.OutputText(this.modelGraph.equSubgraphs[idx*2].edgeNodes[i].name, true);
			//        Program.OutputText(this.modelGraph.equSubgraphs[idx*2+1].connNodes[i].name, true);
			//    }
			//}
			// export used material
			//string matname = foldername + "material.txt";
			//int maxlabel = 0;
			//foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			//    maxlabel = mn.nodeType > maxlabel ? mn.nodeType : maxlabel;
			//foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			//    maxlabel = mn.nodeType > maxlabel ? mn.nodeType : maxlabel;
			//StreamWriter sw_mat = new StreamWriter(matname);
			//for (int i = 0; i <= maxlabel; ++i)
			//{
			//    sw_mat.WriteLine(i.ToString() + " " + "[" +
			//        colorMall[i].R.ToString() + "," +
			//        colorMall[i].G.ToString() + "," +
			//        colorMall[i].B.ToString() + "]");
			//}
			//sw_mat.Close();
		}

        public void ExportSelfmatchingSubgraphs(string pathName)
        {
            // export all subgraphs
            for (int sidx = 0; sidx < this.modelGraph.equSubgraphs.Count; sidx+=2)
            {
                int idx = sidx / 2;
                string filename = pathName + @"subgraph_" + idx.ToString();
                filename += @".subgraph";
                Subgraph a_sg = this.modelGraph.equSubgraphs[sidx];
                Subgraph b_sg = this.modelGraph.equSubgraphs[sidx+1];
                //save subgraph
                SaveSubgraph(filename, a_sg, b_sg);
            }
        }
		public void SaveSubgraph(string filename, Subgraph a_sg, Subgraph b_sg)
		{
			StreamWriter sw = new StreamWriter(filename);
			List<ModelGraphNode> mns_1 = a_sg.nodeSets;
			sw.WriteLine("#" + this.modelGraph.objfilename);
			sw.WriteLine("#Nodes in Subgraph 1");
			foreach (ModelGraphNode mn in mns_1)
			{
				sw.WriteLine(mn.name);
			}
			// edges
			sw.WriteLine("#Edges in Subgraph 1");
			int nedges = a_sg.edgeNodes.Count;
			if (a_sg.edgeNodes.Count !=
				a_sg.connNodes.Count)
			{
				Program.OutputText("Not Matched Node Correspondence!", true);
			}
			for (int i = 0; i < nedges; ++i)
			{
				sw.Write(a_sg.edgeNodes[i].name);
				sw.Write(" ");
				sw.WriteLine(a_sg.connNodes[i].name);
			}
			List<ModelGraphNode> mns_2 = b_sg.nodeSets;
			sw.WriteLine("#" + this.modelGraph.objfilename);
			sw.WriteLine("#Nodes in Subgraph 2");
			foreach (ModelGraphNode mn in mns_2)
			{
				sw.WriteLine(mn.name);
			}
			sw.WriteLine("#Edges in Subgraph 2");
			nedges = a_sg.edgeNodes.Count;
			if (b_sg.edgeNodes.Count !=
				b_sg.connNodes.Count)
			{
				Program.OutputText("Not Matched Node Correspondence!", true);
			}
			for (int i = 0; i < nedges; ++i)
			{
				sw.Write(b_sg.edgeNodes[i].name);
				sw.Write(" ");
				sw.WriteLine(b_sg.connNodes[i].name);
			}
			sw.Close();
		}


		public void LoadSubgraphs(string[] sgNames, int tag)
		{
            if (tag == 1)
            {
                this.sourceGraph.equSubgraphs = new List<Subgraph>();
                this.targetGraph.equSubgraphs = new List<Subgraph>();
                this.sourceGraph.equCutpairs = new List<List<ModelGraphEdge>>();
                this.targetGraph.equCutpairs = new List<List<ModelGraphEdge>>();
            }
            else
            {
                this.modelGraph.equSubgraphs = new List<Subgraph>();
                this.modelGraph.equCutpairs = new List<List<ModelGraphEdge>>();
                this.sourceGraph = this.modelGraph;
                this.targetGraph = this.modelGraph;
            }
			int nsubgraphs = sgNames.Length;
			// re-order names
            //int index = sgNames[0].LastIndexOf('\\');
            //string filename = sgNames[0].Substring(0, index + 10);
            //for (int ns = 0; ns < nsubgraphs; ++ns)
            //    sgNames[ns] = filename + ns.ToString()+@".subgraph";
			char[] delimiters = { ' ' };
			for (int ns = 0; ns < nsubgraphs; ++ns)
			{
				StreamReader sr = new StreamReader(sgNames[ns]);
				Subgraph sg1 = new Subgraph();
				sg1.nodeSets = new List<ModelGraphNode>();
				sg1.outgoingEdges = new List<ModelGraphEdge>();
				sg1.edgeNodes = new List<ModelGraphNode>();
				sg1.connNodes = new List<ModelGraphNode>();
				// nodes for G1

				string s = sr.ReadLine();
				while (s[0] == '#')
					s = sr.ReadLine();
				// for the old version without prefix 
				int length = this.sourceGraph.objfilename.Length;
				
				while (true)
				{
                    if (tag == 1)
                    {
                        if (s.Length < length
                        || s.Substring(0, length) != this.sourceGraph.objfilename)
                            s = this.sourceGraph.objfilename + @"_" + s;
                    }
					ModelGraphNode mn = null;
					foreach (ModelGraphNode mgn in this.sourceGraph.graphNodes)
					{
						if (mgn.name == s)
						{
							mn = mgn;
							break;
						}
					}
					sg1.nodeSets.Add(mn);
					s = sr.ReadLine();
					if (s == null || s == "" || s[0] == '#') break;
				}
				// edges for G1
				while (true)
				{
					s = sr.ReadLine();
					if (s == null || s == "" || s[0] == '#') break;
					string[] tokens = s.Split(delimiters);
					ModelGraphNode edgenode = null, connnode = null;
                    if (tag == 1)
                    {
                        for (int i = 0; i < 2; ++i)
                        {
                            if (tokens[i].Length < length ||
                                tokens[i].Substring(0, length) != this.sourceGraph.objfilename)
                                tokens[i] = this.sourceGraph.objfilename + @"_" + tokens[i];
                        }
                    }
					foreach (ModelGraphNode mgn in this.sourceGraph.graphNodes)
					{
						if (mgn.name == tokens[0])
							edgenode = mgn;
						else if (mgn.name == tokens[1])
							connnode = mgn;
					}
					if (edgenode == null || connnode == null)
					{
						Program.OutputText("No such nodes found!", true);
					}
					sg1.edgeNodes.Add(edgenode);
					sg1.connNodes.Add(connnode);
					ModelGraphEdge me = null;
					foreach (ModelGraphEdge mge in this.sourceGraph.graphEdges)
					{
						if ((mge.conntectingNodes[0].name == tokens[0] &&
							mge.conntectingNodes[1].name == tokens[1]) ||
							(mge.conntectingNodes[1].name == tokens[0] &&
							mge.conntectingNodes[0].name == tokens[1]))
						{
							me = mge;
							break;
						}
					}
					if (me != null) sg1.outgoingEdges.Add(me);
				}
				sg1.ingoingEdges = this.sourceGraph.GetIngoingEdges(sg1.nodeSets);
				this.sourceGraph.equCutpairs.Add(sg1.outgoingEdges);
				this.sourceGraph.equSubgraphs.Add(sg1);

				while (s[0] == '#')
					s = sr.ReadLine();
				// nodes for G2
				Subgraph sg2 = new Subgraph();
				sg2.nodeSets = new List<ModelGraphNode>();
				sg2.outgoingEdges = new List<ModelGraphEdge>();
				sg2.edgeNodes = new List<ModelGraphNode>();
				sg2.connNodes = new List<ModelGraphNode>();
				length = this.targetGraph.objfilename.Length;
				while (true)
				{
					ModelGraphNode mn = null;
                    if (tag == 1)
                    {
                        if (s.Length < length ||
                            s.Substring(0, length) != this.targetGraph.objfilename)
                            s = this.targetGraph.objfilename + @"_" + s;
                    }
					foreach (ModelGraphNode mgn in this.targetGraph.graphNodes)
					{
						if (mgn.name == s)
						{
							mn = mgn;
							break;
						}
					}
					sg2.nodeSets.Add(mn);
					s = sr.ReadLine();
					if (s == null || s == "" || s[0] == '#') break;
				}
				
				// edges for G2
				while (true)
				{
					s = sr.ReadLine();
					if (s == null || s == "" || s[0] == '#') break;
					string[] tokens = s.Split(delimiters);
					ModelGraphNode edgenode = null, connnode = null;
                    if (tag == 1)
                    {
                        for (int i = 0; i < 2; ++i)
                        {
                            if (tokens[i].Length < length ||
                                tokens[i].Substring(0, length) != this.targetGraph.objfilename)
                                tokens[i] = this.targetGraph.objfilename + @"_" + tokens[i];
                        }
                    }
					foreach (ModelGraphNode mgn in this.targetGraph.graphNodes)
					{
						if (mgn.name == tokens[0])
							edgenode = mgn;
						else if (mgn.name == tokens[1])
							connnode = mgn;
					}
					if (edgenode == null || connnode == null)
					{
						Program.OutputText("No such nodes found!", true);
					}
					sg2.edgeNodes.Add(edgenode);
					sg2.connNodes.Add(connnode);
					ModelGraphEdge me = null;
					foreach (ModelGraphEdge mge in this.targetGraph.graphEdges)
					{
						if ((mge.conntectingNodes[0].name == tokens[0] &&
							mge.conntectingNodes[1].name == tokens[1]) ||
							(mge.conntectingNodes[1].name == tokens[0] &&
							mge.conntectingNodes[0].name == tokens[1]))
						{
							me = mge;
							break;
						}
					}
					if (me != null) sg2.outgoingEdges.Add(me);

				}
				sg2.ingoingEdges = this.targetGraph.GetIngoingEdges(sg2.nodeSets);
				this.targetGraph.equCutpairs.Add(sg2.outgoingEdges);
				this.targetGraph.equSubgraphs.Add(sg2);
				sr.Close();
			}
				
		}//End-load-subgraph

		public void ExportGraphPDFs(string filename)
		{
			double minx = double.MaxValue, miny = double.MaxValue,
			maxx = double.MinValue, maxy = double.MinValue;
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				double x1 = mn.meshComponent.boundingBox.points3[0].pos.x,
					y1 = mn.meshComponent.boundingBox.points3[0].pos.y,
					x2 = mn.meshComponent.boundingBox.points3[6].pos.x,
					y2 = mn.meshComponent.boundingBox.points3[6].pos.y;
				minx = minx < x1 ? minx : x1;
				maxx = maxx > x2 ? maxx : x2;
				miny = miny < y1 ? miny : y1;
				maxy = maxy > y2 ? maxy : y2;
			}
			int width = (int)((maxx - minx) * 100) + 10,
					height = (int)((maxy - miny) * 100) + 20;
			Vector2d startvec = new Vector2d(-(int)(minx * 100 - 5),
				-(int)(miny * 100 - 5));
			StreamWriter sw = new StreamWriter(filename);

			sw.WriteLine("<</PageSize[" + width.ToString() + " " + height.ToString() + "]>>setpagedevice");
			sw.WriteLine("0 0 translate");
			//edges
			Vector3d lineColor = new Vector3d(0.65, 0.65, 0.65);//
			foreach (ModelGraphEdge me in this.modelGraph.graphEdges)
			{
				Vector3d v1 = me.conntectingNodes[0].meshComponent.boundingBox.center.pos,
					v2 = me.conntectingNodes[1].meshComponent.boundingBox.center.pos;
				Vector2d v = new Vector2d(v1.x, v1.y);
				v *= 100;
				v += startvec;
				sw.WriteLine("newpath");
				// turn direction
				v.x = width - v.x;
				//
				sw.Write(v.x.ToString() + " ");
				sw.Write(v.y.ToString() + " ");
				sw.Write("moveto ");
				v = new Vector2d(v2.x, v2.y);
				v *= 100;
				v += startvec;
				// turn direction
				v.x = width - v.x;
				//
				sw.Write(v.x.ToString() + " ");
				sw.Write(v.y.ToString() + " ");
				sw.Write("lineto ");
				sw.WriteLine("{0} {1} {2} {3}", lineColor[0].ToString(), lineColor[1].ToString(), lineColor[2].ToString(), "setrgbcolor");
				sw.WriteLine("1 setlinewidth");

				sw.WriteLine("stroke");
			}
			// nodes
			int radius = 3;
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				Vector3d v3 = mn.meshComponent.boundingBox.center.pos;
				Vector2d v = new Vector2d(v3.x, v3.y);
				sw.WriteLine("newpath");
				Color color = colorMall[mn.nodeType];
				sw.WriteLine("{0} {1} {2} {3}", lineColor[0].ToString(), lineColor[1].ToString(), lineColor[2].ToString(), "setrgbcolor");
				sw.WriteLine("0.3 setlinewidth");
				v *= 100;
				v += startvec;
				//v = new Vector2d((int)v.x, (int)v.y);
				// turn direction
				v.x = width - v.x;
				//
				sw.Write(v.x.ToString() + " ");
				sw.Write(v.y.ToString() + " ");
				sw.Write(radius.ToString() + " ");
				int d1 = 0, d2 = 360;
				sw.Write(d1.ToString() + " " + d2.ToString() + " ");
				sw.WriteLine("arc closepath");
				sw.WriteLine("gsave");
				Vector3d rgb =  new Vector3d((double)color.R / 255.0, (double)color.G / 255.0, (double)color.B / 255.0);
				sw.Write("{0} {1} {2} {3}", rgb[0].ToString(), rgb[1].ToString(), rgb[2].ToString(), "setrgbcolor ");
				sw.WriteLine("fill");
				sw.WriteLine("grestore");
				sw.WriteLine("stroke");
			}
			sw.Close();
		}

		public void ExportSubgGraphPairToPSFiles(string foldername, int sgIndex)
		{
			ModelGraph a_mg = this.sourceGraph, b_mg = this.targetGraph;
			double minx = double.MaxValue, miny = double.MaxValue,
			maxx = double.MinValue, maxy = double.MinValue;
			foreach (ModelGraphNode mn in a_mg.graphNodes)
			{
				double x1 = mn.meshComponent.boundingBox.points3[0].pos.x,
					y1  =mn.meshComponent.boundingBox.points3[0].pos.y,
					x2 = mn.meshComponent.boundingBox.points3[6].pos.x,
					y2  =mn.meshComponent.boundingBox.points3[6].pos.y;
				minx = minx < x1 ? minx : x1;
				maxx = maxx > x2 ? maxx : x2;
				miny = miny < y1 ? miny : y1;
				maxy = maxy > y2 ? maxy : y2;
			}
			foreach (ModelGraphNode mn in b_mg.graphNodes)
			{
				double x1 = mn.meshComponent.boundingBox.points3[0].pos.x,
					y1 = mn.meshComponent.boundingBox.points3[0].pos.y,
					x2 = mn.meshComponent.boundingBox.points3[6].pos.x,
					y2 = mn.meshComponent.boundingBox.points3[6].pos.y;
				minx = minx < x1 ? minx : x1;
				maxx = maxx > x2 ? maxx : x2;
				miny = miny < y1 ? miny : y1;
				maxy = maxy > y2 ? maxy : y2;
			}
			int wid = (int)((maxx - minx) * 100) + 10,
					height = (int)((maxy - miny) * 100) + 10;
			Vector2d startvec = new Vector2d( -(int)(minx * 100 - 5),
				-(int)(miny * 100 - 5));

			//for (int sidx = 0; sidx < this.sourceGraph.equSubgraphs.Count; ++sidx)
			//{
			int sidx = sgIndex;
				string filename = foldername + @"subgraph_" + sidx.ToString();
				
				Subgraph a_sg = this.sourceGraph.equSubgraphs[sidx];
				Subgraph b_sg = this.targetGraph.equSubgraphs[sidx];
				string subfilename = filename + @"_a.ps";
				WriteASubgraphToPSFile(subfilename, a_mg, a_sg, wid, height, startvec);
				subfilename = filename + @"_b.ps";
				WriteASubgraphToPSFile(subfilename, b_mg, b_sg, wid, height, startvec);
			//}
		}

		public void WriteASubgraphToPSFileUsingTwoColors(string filename, ModelGraph mg, Subgraph sg, 
			int width, int height, Vector2d startvec)
		{
			StreamWriter sw = new StreamWriter(filename);

			sw.WriteLine("<</PageSize[" + width.ToString() + " " + height.ToString() + "]>>setpagedevice");
			sw.WriteLine("0 0 translate");
			//edges
			Vector3d globalColor = new Vector3d(0.65, 0.65, 0.65),
				hlinecolor = new Vector3d(0.8, 0, 0);
			foreach (ModelGraphEdge me in mg.graphEdges)
			{
				Vector3d v1 = me.conntectingNodes[0].meshComponent.boundingBox.center.pos,
					v2 = me.conntectingNodes[1].meshComponent.boundingBox.center.pos;
				Vector2d v = new Vector2d(v1.x, v1.y);
				v *= 100;
				v += startvec;
				sw.WriteLine("newpath");
				sw.Write(v.x.ToString() + " ");
				sw.Write(v.y.ToString() + " ");
				sw.Write("moveto ");
				v = new Vector2d(v2.x, v2.y);
				v *= 100;
				v += startvec;
				sw.Write(v.x.ToString() + " ");
				sw.Write(v.y.ToString() + " ");
				sw.Write("lineto ");
				if (sg.outgoingEdges.Contains(me))
				{
					sw.WriteLine("{0} {1} {2} {3}", hlinecolor[0].ToString(), hlinecolor[1].ToString(), hlinecolor[2].ToString(), "setrgbcolor");
					sw.WriteLine("1 setlinewidth");
				}
				else
				{
					sw.WriteLine("{0} {1} {2} {3}", globalColor[0].ToString(), globalColor[1].ToString(), globalColor[2].ToString(), "setrgbcolor");
					sw.WriteLine("0.5 setlinewidth");
				}
				sw.WriteLine("stroke");
			}
			// nodes
			int radius = 3;
			Vector3d hfillcolor = new Vector3d(0.1, 0.1, 0.8);
			foreach (ModelGraphNode mn in mg.graphNodes)
			{
				Vector3d v3 = mn.meshComponent.boundingBox.center.pos;
				Vector2d v = new Vector2d(v3.x, v3.y);
				sw.WriteLine("newpath");
				Vector3d rgb = globalColor;
				if (sg.nodeSets.Contains(mn))
					rgb = hfillcolor;
				sw.WriteLine("{0} {1} {2} {3}", globalColor[0].ToString(), globalColor[1].ToString(), globalColor[2].ToString(), "setrgbcolor");
				sw.WriteLine("0.3 setlinewidth");
				v *= 100;
				v += startvec;
				//v = new Vector2d((int)v.x, (int)v.y);
				sw.Write(v.x.ToString() + " ");
				sw.Write(v.y.ToString() + " ");
				if (sg.nodeSets.Contains(mn))
					sw.Write((radius * 1.2).ToString() + " ");
				else
					sw.Write(radius.ToString() + " ");
				int d1 = 0, d2 = 360;
				sw.Write(d1.ToString() + " " + d2.ToString() + " ");
				sw.WriteLine("arc closepath");
				sw.WriteLine("gsave");
				sw.Write("{0} {1} {2} {3}", rgb[0].ToString(), rgb[1].ToString(), rgb[2].ToString(), "setrgbcolor ");
				sw.WriteLine("fill");
				sw.WriteLine("grestore");
				sw.WriteLine("stroke");
			}
			sw.Close();
		}

		public void WriteASubgraphToPSFile(string filename, ModelGraph mg, Subgraph sg,
			int width, int height, Vector2d startvec)
		{
			StreamWriter sw = new StreamWriter(filename);

			sw.WriteLine("<</PageSize[" + width.ToString() + " " + height.ToString() + "]>>setpagedevice");
			sw.WriteLine("0 0 translate");
			//edges
			Vector3d lineColor = new Vector3d(0.65, 0.65, 0.65),
				highlightLineColor = new Vector3d(0.8, 0, 0);
			foreach (ModelGraphEdge me in mg.graphEdges)
			{
				Vector3d v1 = me.conntectingNodes[0].meshComponent.boundingBox.center.pos,
					v2 = me.conntectingNodes[1].meshComponent.boundingBox.center.pos;
				Vector2d v = new Vector2d(v1.x, v1.y);
				v *= 100;
				v += startvec;

				// turn direction
				v.x = width - v.x;

				sw.WriteLine("newpath");
				sw.Write(v.x.ToString() + " ");
				sw.Write(v.y.ToString() + " ");
				sw.Write("moveto ");
				v = new Vector2d(v2.x, v2.y);
				v *= 100;
				v += startvec;

				// turn direction
				v.x = width - v.x;

				sw.Write(v.x.ToString() + " ");
				sw.Write(v.y.ToString() + " ");
				sw.Write("lineto ");
				if (sg.outgoingEdges.Contains(me))
				{
					sw.WriteLine("{0} {1} {2} {3}", highlightLineColor[0].ToString(), highlightLineColor[1].ToString(), highlightLineColor[2].ToString(), "setrgbcolor");
					sw.WriteLine("1 setlinewidth");
				}
				else
				{
					sw.WriteLine("{0} {1} {2} {3}", lineColor[0].ToString(), lineColor[1].ToString(), lineColor[2].ToString(), "setrgbcolor");
					sw.WriteLine("0.5 setlinewidth");
				}
				sw.WriteLine("stroke");
			}
			// nodes
			int radius = 3;
			Vector3d hfillcolor = new Vector3d(0.1, 0.1, 0.8);
			foreach (ModelGraphNode mn in mg.graphNodes)
			{
				Vector3d v3 = mn.meshComponent.boundingBox.center.pos;
				Vector2d v = new Vector2d(v3.x, v3.y);
				sw.WriteLine("newpath");
				Vector3d rgb = lineColor;
				if (sg.nodeSets.Contains(mn))
					rgb = hfillcolor;
				sw.WriteLine("{0} {1} {2} {3}", lineColor[0].ToString(), lineColor[1].ToString(), lineColor[2].ToString(), "setrgbcolor");
				sw.WriteLine("0.3 setlinewidth");
				v *= 100;
				v += startvec;

				// turn direction
				v.x = width - v.x;

				//v = new Vector2d((int)v.x, (int)v.y);
				sw.Write(v.x.ToString() + " ");
				sw.Write(v.y.ToString() + " ");
				if (sg.nodeSets.Contains(mn))
					sw.Write((radius * 1.2).ToString() + " ");
				else
					sw.Write(radius.ToString() + " ");
				int d1 = 0, d2 = 360;
				sw.Write(d1.ToString() + " " + d2.ToString() + " ");
				sw.WriteLine("arc closepath");
				sw.WriteLine("gsave");
				Color color = colorMall[mn.nodeType];
				rgb = new Vector3d((double)color.R / 255.0, (double)color.G / 255.0, (double)color.B / 255.0);
				sw.Write("{0} {1} {2} {3}", rgb[0].ToString(), rgb[1].ToString(), rgb[2].ToString(), "setrgbcolor ");
				sw.WriteLine("fill");
				sw.WriteLine("grestore");
				sw.WriteLine("stroke");
			}
			sw.Close();
		}

		public void ExportEdgeOrder(string pathname)
		{
			string foldername = pathname;
			if (!Directory.Exists(foldername))
				Directory.CreateDirectory(foldername);
			string filename = foldername + this.modelGraph.objfilename + @"_edgeOrder_index.txt";
			StreamWriter sw = new StreamWriter(filename);
			int index = 1;
			this.modelGraph.CompleteAdjEdgesByCycles(false);
			foreach (ModelGraphEdge me in this.modelGraph.graphEdges)
			{
				sw.WriteLine("Edge " + index.ToString() + ": " +
					me.conntectingNodes[0].name + " " +
					me.conntectingNodes[1].name);
				sw.WriteLine("Neighoring edgs:");
				foreach (ModelGraphEdge adjme in me.adjEdges)
				{
					sw.WriteLine(adjme.edgeIdx.ToString());
					//sw.WriteLine(
					//adjme.conntectingNodes[0].name + " " +
					//adjme.conntectingNodes[1].name);
				}
				index++;
			}
			sw.Close();
		}
		// end_****** Save & Load ****** //

		public void InitializeModelGraph()
		{
			if (this.modelGraph != null && this.modelGraph.meshComponents.Count > 0)
			{
				this.modelGraph.FormGraph(); //with node labels
			}
			else
			{
				// After grouping components of a model, construct graph
				this.modelGraph = new ModelGraph(this.meshComponents);
				//**Mesh reuse**//
				this.modelGraph.mesh = this.meshRecord.Mesh;
				this.visContactPoints = new List<Vector3d>();
			}
			if (this.modelGraph.graphEdges != null)
			{
				for (int i = 0; i < this.modelGraph.graphEdges.Count; i++)
					this.visContactPoints.Add(this.modelGraph.graphEdges[i].pos);
			}
		}
		//******* Model Analysis *******//
		//Chairs segments: #5 colors
		//irreplicable: seat(1), legs(4), handrest(0,2)
		//replicable: backrest(including bars), leg bars
		public void ConstructGraphGroup()
		{
			// 1. Construct a graph group from all loaded model graph
			//		including extracting shared components
			if (this.modelGraphs.Count == 0) return;
			this.graphGroup = new GraphGroup(this.modelGraphs);
			//// - make self copy 
			//// - find ground touching
			//// - group meta components
			//// - create new replaceable graph for deform update
			//if (this.modelType == 1 || this.modelType == 3)
			//{
			//    for (int i = 0; i < this.modelGraphs.Count; i++)
			//    {
			//        this.modelGraphs[i].BuildDualGraph(0);
			//    }
			//}
			// copy a reference, including replaceable subgraph info
			// Note the added copy of current edit graph in "SwitchModel" is not included in the orignal graph
			for (int i = 0; i < this.graphGroup.kidModelGraphs.Count; i++)
			{
				this.graphGroup.kidModelGraphs[i].MakeSelfCopy();
			}
		}

		public void SwitchModel(int mIndex)
		{
			// user select a models from the given list for deformation
			//if (this.modelGraph!=null && this.modelGraph.originalGraph != null)
			//{
			//    // restore current edit graph
			//    this.modelGraph.ResetGraph();
			//}
			// switch model
			this.modelGraph = this.graphGroup.kidModelGraphs[mIndex];
			MeshRecord rec = new MeshRecord(this.modelGraph.mesh);
			rec.Filename = this.modelGraph.meshfile;
			rec.objName = this.modelGraph.objfilename;
			this.ChangeCurrentModel(rec);
			this.meshComponents = this.modelGraph.meshComponents;
			// ground touching -- Y-axis
			this.FindGroundTouching(this.modelGraph);

			this.mcIdx = 80;
			this.visContactPoints = new List<Vector3d>();
			for (int i = 0; i < this.modelGraph.graphEdges.Count; i++)
				this.visContactPoints.Add(this.modelGraph.graphEdges[i].pos);

			this.Refresh();
		}

		public MeshComponent MCMap(List<MeshComponent> mcs, int index)
		{
			MeshComponent mappedMC = null;
			foreach (MeshComponent mc in mcs)
			{
				if (mc.componentIndex == index)
				{
					mappedMC = mc;
					break;
				}
			}
			return mappedMC;
		}
		public void UpdateModel()
		{
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				mn.meshComponent.mesh.UpdateVertexPos();
				mn.meshComponent.boundingBox.Update();
				mn.meshComponent.handled = false;
				if (mn.edges == null) continue;
				foreach (ModelGraphEdge e in mn.edges)
				{
					e.UpdatePos();
					e.updated = false;
				}
			}
		}

		public void SearchEquivalentSubgraph(int maxdistanceOfdummyedges, int mincut, bool sym, bool useHeight)
		{
			bool isIterative = false;
			ModelGraph a_mg = this.sourceGraph, b_mg = this.targetGraph;
			if (this.iterationIndex > 0)
				isIterative = true;
			this.graphGroup.minmumCutedges = mincut;
			if (a_mg.graphIndex == b_mg.graphIndex && !isIterative)
			{
				if (a_mg.modelType == 3)
					this.graphGroup.SearchReplaceableSubgraphsWithLevel(a_mg, b_mg, useHeight);
				else if (a_mg.modelType == 1)
				{
					//this.graphGroup.numCutThresh = mincut;
					this.graphGroup.SearchReplaceableSubgraphs(a_mg, b_mg,
							0, isIterative);
					//Program.GetFormMain().GetTargetGraphView().modelGraph = b_mg;
				}
				else
					this.graphGroup.SearchReplaceableSubgraphs_old(a_mg, b_mg, 0);
				return;
			}
			if (sym)
			{
				this.graphGroup.GetSymmetryCuts(a_mg, b_mg);
				return;
			}
			if (a_mg != null && b_mg != null)
			{
				if (a_mg.modelType == 1)
				{
					//this.graphGroup.SearchReplaceableSubgraphs(b_mg, a_mg,
					//        maxdistanceOfdummyedges, isIterative);
					if (b_mg.graphEdges.Count < a_mg.graphEdges.Count)//&& !isIterative)
						this.graphGroup.SearchReplaceableSubgraphs(b_mg, a_mg,
							maxdistanceOfdummyedges, isIterative);
					else
						this.graphGroup.SearchReplaceableSubgraphs(a_mg, b_mg,
							maxdistanceOfdummyedges, isIterative);
				}
				else if (a_mg.modelType == 3)
					this.graphGroup.SearchReplaceableSubgraphsWithLevel(a_mg, b_mg, useHeight);
				else
				{
					if (b_mg.graphEdges.Count < a_mg.graphEdges.Count)
						this.graphGroup.SearchReplaceableSubgraphs_old(b_mg,
							a_mg, maxdistanceOfdummyedges);
					else
						this.graphGroup.SearchReplaceableSubgraphs_old(a_mg,
							b_mg, maxdistanceOfdummyedges);
				}
			}

			//this.graphGroup.SelectExpandGraph(a_mg, b_mg);
		}//end-search-EquivalentSubgraph


		public int iterationIndex = 0;
		public void SaveCurrentSubgraphReplacementInfo(string folderName, int repGraph)
		{
			ModelGraph a_mg = this.sourceGraph, b_mg = this.targetGraph;
			if (repGraph == 1)
			this.SaveSubgraphSequnce(a_mg, b_mg, this.mcIdx, iterationIndex, folderName);
			else
				this.SaveSubgraphSequnce(b_mg, a_mg, this.mcIdx, iterationIndex, folderName);
			this.iterationIndex++;
			//Program.GetFormMain().GetSourceGraphView().modelGraph = a_mg;
			//if (a_mg.graphIndex == b_mg.graphIndex)
			//{
			//    //this.targetGraph = a_mg.Clone() as ModelGraph;
			//    Program.GetFormMain().GetTargetGraphView().modelGraph = a_mg.Clone() as ModelGraph;
			//}
		}

		public void SaveSubgraphSequnce(ModelGraph a_mg, ModelGraph b_mg, 
			int sgIndx, int iterIndex, string folderName)
		{
			// Subgraph Info
			string updatePrefix = @"iter_" + (iterIndex+1).ToString() + @"_";

			string subgraphFilename = folderName + updatePrefix + @"subgraph";
			subgraphFilename += @".subgraph";
			StreamWriter sw = new StreamWriter(subgraphFilename);
			List<ModelGraphNode> mns_1 = a_mg.equSubgraphs[sgIndx].nodeSets;
			sw.WriteLine("#" + a_mg.objfilename);
			sw.WriteLine("#Nodes in Subgraph 1");
			foreach (ModelGraphNode mn in mns_1)
			{
				sw.WriteLine(mn.name);
			}
			// edges
			sw.WriteLine("#Edges in Subgraph 1");
			int nedges = a_mg.equSubgraphs[sgIndx].edgeNodes.Count;
			if (a_mg.equSubgraphs[sgIndx].edgeNodes.Count !=
				a_mg.equSubgraphs[sgIndx].connNodes.Count)
			{
				Program.OutputText("Not Matched Node Correspondence!", true);
			}
			for (int i = 0; i < nedges; ++i)
			{
				sw.Write(a_mg.equSubgraphs[sgIndx].edgeNodes[i].name);
				sw.Write(" ");
				sw.WriteLine(a_mg.equSubgraphs[sgIndx].connNodes[i].name);
			}
			List<ModelGraphNode> mns_2 = b_mg.equSubgraphs[sgIndx].nodeSets;
			sw.WriteLine("#" + b_mg.objfilename);
			sw.WriteLine("#Nodes in Subgraph 2");
			foreach (ModelGraphNode mn in mns_2)
			{
				sw.WriteLine(mn.name);
			}
			sw.WriteLine("#Edges in Subgraph 2");

			if (b_mg.equSubgraphs[sgIndx].edgeNodes.Count !=
				b_mg.equSubgraphs[sgIndx].connNodes.Count)
			{
				Program.OutputText("Not Matched Node Correspondence!", true);
			}
			for (int i = 0; i < nedges; ++i)
			{
				sw.Write(b_mg.equSubgraphs[sgIndx].edgeNodes[i].name);
				sw.Write(" ");
				sw.WriteLine(b_mg.equSubgraphs[sgIndx].connNodes[i].name);
			}
			sw.Close();

			//********************Replace********************//
			ReplaceASubgraph(a_mg, b_mg, a_mg.equSubgraphs[sgIndx], b_mg.equSubgraphs[sgIndx], false);
			//********************Replace********************//

			// new graph file after replacement
			
			a_mg.objfilename =  updatePrefix + a_mg.objfilename;
			string newGraphFilename = folderName + a_mg.objfilename;
			newGraphFilename += @".graph";
			
			sw = new StreamWriter(newGraphFilename);
			sw.WriteLine("#node number");
			sw.WriteLine(a_mg.graphNodes.Count);
			List<string> names = new List<string>();
			foreach (ModelGraphNode mn in a_mg.graphNodes)
			{
				if (names.Contains(mn.name))
					mn.name += @"_graphB";
				else
					names.Add(mn.name);
				mn.name = updatePrefix + mn.name;
				sw.WriteLine(mn.name);
			}
			sw.WriteLine();
			sw.WriteLine("#edge number --- node 1/node 2");
			sw.WriteLine(a_mg.graphEdges.Count);
			foreach (ModelGraphEdge me in a_mg.graphEdges)
			{
				if (me.conntectingNodes.Count != 2)
				{
					MessageBox.Show("Wrong number of contacts components!");
					return;
				}
				ModelGraphNode mn1 = me.conntectingNodes[0];
				ModelGraphNode mn2 = me.conntectingNodes[1];
				sw.Write(mn1.name + " " + mn2.name + " ");
				sw.WriteLine();
			}
			sw.Close();
			Program.OutputText("saved graph file to" + newGraphFilename, true);

			
		}//End-save-sequence

		public int SelectMaxValidSubgraph(ModelGraph a_mg, ModelGraph b_mg)
		{
			int index = -1;
			int maxNodeDiff = 0;
			for (int sg = 0;sg < a_mg.equSubgraphs.Count; ++sg)
			{	
				int num = b_mg.equSubgraphs[sg].nodeSets.Count -
					a_mg.equSubgraphs[sg].nodeSets.Count;
				if (num > maxNodeDiff)
				{
					maxNodeDiff = num;
					index = sg;
				}
			}
			return index;
		}
		public int SelectMaxNodeSetsSubgraph(ModelGraph a_mg, ModelGraph b_mg)
		{
			int index = -1;
			int maxNodeNum = 0;
			for (int sg = 0; sg < a_mg.equSubgraphs.Count; ++sg)
			{
				// check whether they are the same
				int num = b_mg.equSubgraphs[sg].nodeSets.Count;
				List<string> a_mg_names = new List<string>(),
				b_mg_names = new List<string>();
				for (int i = 0; i < num; ++i)
				{
					a_mg_names.Add(a_mg.equSubgraphs[sg].nodeSets[i].name);
					b_mg_names.Add(b_mg.equSubgraphs[sg].nodeSets[i].name);
				}
				bool same = true;
				for (int i = 0; i < num; ++i)
				{
					if (!b_mg_names.Contains(a_mg_names[i]))
					{
						same = false;
						break;
					}
				}
				if (same) continue;
				if (num > maxNodeNum)
				{
					maxNodeNum = num;
					index = sg;
				}
			}
			return index;
		}

		public void RemoveDummyEdges()
		{
 			// for 2 kinds of search, with/out dummy edges
			for (int i = 0; i < this.modelGraph.graphEdges.Count; ++i )
			{
				ModelGraphEdge mg = this.modelGraph.graphEdges[i];
				if (!mg.isDummyEdge) continue;
				mg.conntectingNodes[0].edges.Remove(mg);
				mg.conntectingNodes[0].adjNodes.Remove(mg.conntectingNodes[1]);
				mg.conntectingNodes[1].edges.Remove(mg);
				mg.conntectingNodes[1].adjNodes.Remove(mg.conntectingNodes[0]);
				this.modelGraph.graphEdges.Remove(mg);
				--i;
			}
			// for added *dummy edges* by distance
			foreach (ModelGraphEdge me in this.modelGraph.graphEdges)
			{
				if (me.trueAdjEdges == null || me.adjEdges.Count != me.trueAdjEdges.Count)
					continue;

				for (int i = 0; i < me.adjEdges.Count; ++i)
				{
					if (!me.trueAdjEdges[i])
					{
						me.trueAdjEdges.RemoveAt(i);
						me.adjEdges.RemoveAt(i);
						--i;
					}
				}
			}
		}//End-Remove-dummy-edges

		public void SetComponentView(int tag)
		{
			MeshView mv = null;
			Mesh mesh_mv = null;
			List<ModelGraphNode> mns = null;
			if (tag == 0) //source
			{
				mv = Program.GetFormMain().GetSourceComponentView();
				mesh_mv = this.sourceGraph.mesh;// new Mesh(this.sourceGraph.mesh);
				if (this.mcIdx >= 0)
					mns = this.sourceGraph.equSubgraphs[this.mcIdx].nodeSets;
			}
			else 
			{
				mv = Program.GetFormMain().GetTargetComponentView();
				mesh_mv = this.targetGraph.mesh;// new Mesh(this.targetGraph.mesh);
				if (this.mcIdx >= 0)
					mns = this.targetGraph.equSubgraphs[this.mcIdx].nodeSets;
			}
			mv.meshRecord = new MeshRecord(mesh_mv);
			if (this.mcIdx >= 0)
			{
				mv.meshComponents = new List<MeshComponent>();
				for (int i = 0; i < mns.Count; ++i)
				{
					mv.meshComponents.Add(mns[i].meshComponent.Clone() as MeshComponent);
				}
			}
		}//End-set-component-view

		public ModelGraph sourceGraph = null, targetGraph = null;

		public void InitializeGraphMatch(ModelGraph sGraph, ModelGraph tGraph)
		{
			// Intialize an edge to begin match
			this.sourceGraph = sGraph;
			this.targetGraph = tGraph;
		}

		public void FindMatchingSubgraphs()
		{
			ModelGraph a_mg = this.sourceGraph, b_mg = this.targetGraph;
			this.graphGroup.FindMatchingSubgraphs(a_mg, b_mg);
		}

		public void LocateContactFaces()
		{ 
			// after preparing all edges, calcluate the contact face number
			// y--upright direction
			// 0(x+), 1(x-), 2(y+), 3(y-), 4(z+), 5(z-)
			// --calculate box size for each node
			// -----MODIFY: only support direction is calculated
			foreach (ModelGraphEdge mge in this.modelGraph.graphEdges)
			{
				ModelGraphNode mn1 = mge.conntectingNodes[0];
				ModelGraphNode mn2 = mge.conntectingNodes[1];
				if (mn1.meshComponent.boundingBox.points3[0].pos.y >
					mn2.meshComponent.boundingBox.center.pos.y)
				{
					mge.start = mn2;
					mge.end = mn1;
				}
				else if (mn2.meshComponent.boundingBox.points3[0].pos.y >
					mn1.meshComponent.boundingBox.center.pos.y)
				{
					mge.start = mn1;
					mge.end = mn2;
				}
			}
		}//End-Locate-contact-face

		public bool VerifyPath(ModelGraph mg)
		{
			// after replacing "sg" of "mg" by "repSG"
			// verify whether the new model is valid
			bool isvalid = true;
			// in playgrounds, actually, the entry(ladder) is also the exit,
			// but we do not verify this to save time, only verify entry-exit
			List<ModelGraphNode> entries = new List<ModelGraphNode>(),
				exits = new List<ModelGraphNode>();
			int nodesCount = mg.graphNodes.Count;
			int[] exitNodeIdMap = new int[nodesCount];
			int index = 0;
			for (int i = 0; i < nodesCount; ++i)
			{
				exitNodeIdMap[i] = -1;
				mg.graphNodes[i].nodeIdx = index++;
			}
			index = 0;
			foreach (ModelGraphNode mn in mg.graphNodes)
			{
				if (mn.isEntry)
					entries.Add(mn);
				if (mn.isExit)
				{
					exits.Add(mn);
					exitNodeIdMap[mn.nodeIdx] = index++;
				}
			}
			// path matrix
			int m = entries.Count, n = exits.Count;
			MatrixNd pathMat = new MatrixNd(m,n); //initialize 0
			// true = 1
			for (int row = 0; row < m; ++row)
			{
				List<ModelGraphNode> queue = new List<ModelGraphNode>();
				queue.Add(entries[row]);
				bool[] visited = new bool[nodesCount];
				visited[queue[0].nodeIdx] = true;
				while(queue.Count > 0)
				{
					ModelGraphNode currNode = queue[0];
					queue.RemoveAt(0);
					foreach (ModelGraphEdge mge in currNode.edges)
					{
						ModelGraphNode node = mge.conntectingNodes[0] == currNode ?
							mge.conntectingNodes[1] : mge.conntectingNodes[0];
						if (visited[node.nodeIdx]) continue;
						if (node.isExit)
						{
							if (mge.start != currNode) continue; // cannot access
							int col = exitNodeIdMap[node.nodeIdx];
							pathMat[row, col] = 1;
						}
						else
							queue.Add(node);
						visited[node.nodeIdx] = true;
					}
				}//end-while
				// if any exit cannot be accessed, wrong
				bool access = true;
				for (int i = 0; i < n; ++i)
				{
					if (pathMat[row, i] == 0)
					{
						access = false;
						break;
					}
				}
				if (!access)
				{
					isvalid = false;
					break;
				}
			}//end-for
			return isvalid;
		}//End-Verify-validity-of-subgraph-replacement

		public void UpdateGraph(int tag)
		{
			int sgIdx = this.mcIdx;
			ModelGraph mg = null;
			if (tag == 0)
			{
				mg = this.sourceGraph;
				this.ReplaceASubgraph(this.sourceGraph,
					this.targetGraph,
				this.sourceGraph.equSubgraphs[sgIdx],
                this.targetGraph.equSubgraphs[sgIdx], false);
			}
			else
			{
				mg = this.targetGraph;
				this.ReplaceASubgraph(this.targetGraph,
					this.sourceGraph,
				this.targetGraph.equSubgraphs[sgIdx],
                this.sourceGraph.equSubgraphs[sgIdx], false);
			}
			// 
			//int mnIdx = 0;
			//foreach (ModelGraphNode mn in mg.graphNodes)
			//{
			//    mn.nodeIdx = mnIdx++;
			//    mn.adjNodes = new List<ModelGraphNode>();
			//}
			//foreach (ModelGraphEdge me in mg.graphEdges)
			//{
			//    me.conntectingNodes[0].adjNodes.Add(me.conntectingNodes[1]);
			//    me.conntectingNodes[1].adjNodes.Add(me.conntectingNodes[0]);
			//}
			//mg.CompleteAdjEdgesByCycles(true);
		}// End-update-graph

        private int[] GetVertexIndices(int dir)
        {
            int[] indices = new int[4];
            switch (dir)
            {
                case 1:
                    indices = new int[4] { 2, 6, 7, 3 };
                    break;
                case 2:
                    indices = new int[4] { 1, 2, 3, 0 };
                    break;
                case 3:
                    indices = new int[4] { 5, 1, 0, 4 };
                    break;
                case 4:
                    indices = new int[4] { 6, 5, 4, 7 };
                    break;
                case 5:
                    indices = new int[4] { 0, 3, 4, 7 };
                    break;
                case 6:
                    indices = new int[4] { 2, 1, 6, 5 };
                    break;
                default:
                    break;
            }
            return indices;
        }//GetVertexIndices

		public int ValidateRaceTrack(ModelGraph mg)
		{
			// valid: 1
			// sg1<->sg2 valid: 2
			// invalid: -1
			int valid = 1;
			// check at boundary
			int nsection = 0, narc = 0;
			int startIdx = 0;
			for (; startIdx < mg.graphNodes.Count; ++startIdx)
				if (mg.graphNodes[startIdx].name.Contains("plane") &&
					!mg.graphNodes[startIdx].name.Contains("up"))
					break;
			for (int i = 0; i < mg.graphNodes.Count; ++i)
			{
				ModelGraphNode mn = mg.graphNodes[(i + startIdx) % mg.graphNodes.Count];
				if (mn.name.Contains("plane") && !mn.name.Contains("up"))
				{
					if (narc > 0)
					{
						if (narc > nsection * 4)
							return -1;
						nsection = 0;
						narc = 0;
					}
					++nsection;
				}
				if (mn.name.Contains("arc"))
					++narc;
			}
			return valid;
		}//ValidateRaceTrack

		private Matrix4d TranslationNodes(Subgraph tarSG, Subgraph srcSG, int eidx, int sfaceIdx, int tfaceIdx)
		{
			double thresh = 0.01;
			int[] sdir = GetVertexIndices(sfaceIdx), tdir = GetVertexIndices(tfaceIdx);
			int axis = -1;
			if (sfaceIdx == 1 || sfaceIdx == 3) axis = 0;
			else if (sfaceIdx == 2 || sfaceIdx == 4) axis = 2;
			else axis = 1;
			Vector3d sv = srcSG.edgeNodes[eidx].meshComponent.boundingBox.points3[sdir[0]].pos,
				tv = tarSG.edgeNodes[eidx].meshComponent.boundingBox.points3[tdir[0]].pos;
			Vector3d sc = new Vector3d(), tc = new Vector3d();
			int nums = 0, numt = 0, num = 20;
			for (int i = 0, j = 0; i < srcSG.edgeNodes[eidx].meshComponent.mesh.VertexCount; ++i, j += 3)
			{
				Vector3d v = new Vector3d(srcSG.edgeNodes[eidx].meshComponent.mesh.VertexPos[j],
					srcSG.edgeNodes[eidx].meshComponent.mesh.VertexPos[j + 1],
					srcSG.edgeNodes[eidx].meshComponent.mesh.VertexPos[j + 2]);
				if (Math.Abs(v[axis] - sv[axis]) < 0.03)
				{
					sc += v;
					++nums;
					if (nums >= num) break;
				}
			}
			sc /= nums;
			for (int i = 0, j = 0; i < tarSG.edgeNodes[eidx].meshComponent.mesh.VertexCount; ++i, j += 3)
			{
				Vector3d v = new Vector3d(tarSG.edgeNodes[eidx].meshComponent.mesh.VertexPos[j],
					tarSG.edgeNodes[eidx].meshComponent.mesh.VertexPos[j + 1],
					tarSG.edgeNodes[eidx].meshComponent.mesh.VertexPos[j + 2]);
				if (Math.Abs(v[axis] - tv[axis]) < thresh)
				{
					tc += v;
					++numt;
					if (numt >= num) break;
				}
			}
			tc /= numt;

			Matrix4d T = Matrix4d.TranslationMatrix(tc - sc);
			if (nums == 0 || numt == 0)
				T = Matrix4d.TranslationMatrix(tarSG.edgeNodes[eidx].meshComponent.boundingBox.center.pos
					- srcSG.edgeNodes[eidx].meshComponent.boundingBox.center.pos);

			return T;
		}//TranslationNodes

		//private static int iterIdx = 0;
		public  int iterIdx = 0;
		private void TransformSubgraph(ModelGraph mg, Subgraph tarSG, ref Subgraph srcSG,
			out List<ModelGraphNode> editNodes, bool isThreeD)
		{
			Matrix4d T = null, R = null;

			int tfaceIdx = 0, sfaceIdx = 0;

			for (int i = 0; i < tarSG.outgoingEdges.Count; ++i)
			{ 
				tfaceIdx = (tarSG.outgoingEdges[i].conntectingNodes[0] == tarSG.edgeNodes[i] ?
					tarSG.outgoingEdges[i].faceIndex[0] : tarSG.outgoingEdges[i].faceIndex[1]);
				sfaceIdx = (srcSG.outgoingEdges[i].conntectingNodes[0] == srcSG.edgeNodes[0] ?
					srcSG.outgoingEdges[i].faceIndex[0] : srcSG.outgoingEdges[i].faceIndex[1]);
				if (tfaceIdx < 5 && sfaceIdx < 5)
					break;
			}
			// rotation
			int diffFidx = tfaceIdx - sfaceIdx;
			double rotAngle = diffFidx * Math.PI / 2;


			//R = Matrix4d.RotationMatrix(new Vector3d(1, 0, 0), rotAngle);
			R = Matrix4d.RotationMatrix(new Vector3d(0, 1, 0), rotAngle);

			Vector3d minv = new Vector3d(double.MaxValue, double.MaxValue, double.MaxValue),
				maxv = new Vector3d(double.MinValue, double.MinValue, double.MinValue);
			//int flipAxis = -1;
			
			//if (srcSG.flip)
			//{
			//    foreach (ModelGraphNode mn in srcSG.nodeSets)
			//    {
			//        maxv = Vector3d.Max(mn.meshComponent.mesh.MaxCoord(), maxv);
			//        minv = Vector3d.Min(mn.meshComponent.mesh.MinCoord(), minv);
			//    }
			//    if (Math.Abs(srcSG.outgoingEdges[0].pos[0] - srcSG.outgoingEdges[1].pos[0]) >
			//        Math.Abs(srcSG.outgoingEdges[0].pos[2] - srcSG.outgoingEdges[1].pos[2]))
			//    {
			//        flipAxis = 2;
			//    }
			//    else flipAxis = 0;
			//}
			//flipAxis = 2;

			int[] nodeIdMap = new int[srcSG.nodeSets.Count];
			int nidx = 0;
			editNodes = new List<ModelGraphNode>();
			foreach (ModelGraphNode mn in srcSG.nodeSets)
			{
				nodeIdMap[nidx++] = mg.graphNodes.Count;
				ModelGraphNode repnode = mn.Clone() as ModelGraphNode;
                repnode.name = @"iter_" + iterIdx.ToString() + @"_" + mn.name;
                if (!isThreeD)
                {
                    repnode.meshComponent.mesh.Transform(R);
                    repnode.meshComponent.boundingBox.Transform(R);
                }
				repnode.pos3D = repnode.meshComponent.boundingBox.center.pos;
				repnode.adjNodes = new List<ModelGraphNode>();
				editNodes.Add(repnode); //node index will be reset in search				
			}
           // return;

			//foreach (ModelGraphNode mn in editNodes)
			//{
			//    if (srcSG.flip)
			//    {
			//        for (int i = 0, j = 0; i < mn.meshComponent.mesh.VertexCount; ++i, j += 3)
			//        {
			//            mn.meshComponent.mesh.VertexPos[j + flipAxis] = maxv[flipAxis] -
			//                mn.meshComponent.mesh.VertexPos[j + flipAxis];
			//        }
			//        mn.meshComponent.FindBoundingBox();
			//    }
			//}

            List<ModelGraphEdge> inEdges = new List<ModelGraphEdge>();
			foreach (ModelGraphEdge me in srcSG.ingoingEdges)
			{               
				me.faceIndex[0] = (me.faceIndex[0] + diffFidx + 4) % 4;
				if (me.faceIndex[0] == 0) me.faceIndex[0] = 4;
				me.faceIndex[1] = (me.faceIndex[1] + diffFidx + 4) % 4;
				if (me.faceIndex[1] == 0) me.faceIndex[1] = 4;
				if (srcSG.flip)
				{
					if (me.faceIndex[0] == 2 || me.faceIndex[0] == 4)
					{
						me.faceIndex[0] = 4 - me.faceIndex[0];
						if (me.faceIndex[0] == 0) me.faceIndex[0] = 4;
					}
					if (me.faceIndex[1] == 2 || me.faceIndex[1] == 4)
					{
						me.faceIndex[1] = 4 - me.faceIndex[1];
						if (me.faceIndex[1] == 0) me.faceIndex[1] = 4;
					}
				}
			}			

			// translation
			
			sfaceIdx = (sfaceIdx + diffFidx + 4) % 4;
			if (sfaceIdx == 0) sfaceIdx = 4;

			//T = TranslationNodes(tarSG, srcSG, 0, sfaceIdx, tfaceIdx);
			Vector3d tv = tarSG.edgeNodes[0].meshComponent.boundingBox.center.pos
				- srcSG.edgeNodes[0].meshComponent.boundingBox.center.pos;
			T = Matrix4d.TranslationMatrix(tv);
			
			foreach (ModelGraphNode mn in editNodes)
			{
				mn.meshComponent.mesh.Transform(T);
				mn.meshComponent.boundingBox.Transform(T);
				mn.pos3D = mn.meshComponent.boundingBox.center.pos;
			}

		}//FindTransformation

		public void ReplaceASubgraph(ModelGraph mg, ModelGraph repMG, Subgraph sg, Subgraph repSG,
            bool isThreeD)
		{
			// replace subgraph sg of mg by repSG
			// outgoing edges should be already corresponded
			// add *repSG*
			++iterIdx;
			List<ModelGraphNode> editNodes = null;

			if(!isThreeD)
				TransformSubgraph(mg, sg, ref repSG, out editNodes, isThreeD);

			int diffLevel = 0;
            if (isThreeD)
            {
				editNodes = new List<ModelGraphNode>();
                Vector3d sg_center = new Vector3d(),
                    repSG_center = new Vector3d();
                List<Vector3d> srcpts = new List<Vector3d>(),
                    tarpts = new List<Vector3d>();
                foreach (ModelGraphNode mn in sg.nodeSets)
                    sg_center += mn.meshComponent.boundingBox.center.pos;
                sg_center /= sg.nodeSets.Count;
                tarpts.Add(sg_center);
                foreach (ModelGraphNode mn in repSG.nodeSets)
                    repSG_center += mn.meshComponent.boundingBox.center.pos;
                repSG_center /= repSG.nodeSets.Count;
                srcpts.Add(repSG_center);

                Matrix4d T = null;			

                // rigid transformation
				double thresh = 0.3;
                Vector3d tv = sg.connNodes[0].meshComponent.boundingBox.center.pos
                    - sg.edgeNodes[0].meshComponent.boundingBox.center.pos,
                    sv = repSG.connNodes[0].meshComponent.boundingBox.center.pos
                    - repSG.edgeNodes[0].meshComponent.boundingBox.center.pos;
				tv = tv.Normalize(); sv = sv.Normalize();
				double acos = tv.Dot(sv);
				double angle = Math.Acos(acos);
				diffLevel = sg.edgeNodes[0].level - repSG.edgeNodes[0].level;
				if (sg.edgeNodes.Count > 1 && angle < thresh)
				{
					Vector3d tvec = new Vector3d(), svec = new Vector3d();
					for (int i = 1; i < sg.edgeNodes.Count; ++i)
					{
						if (sg.edgeNodes[1] != sg.edgeNodes[0] && repSG.edgeNodes[1] != repSG.edgeNodes[0])
						{
							tvec = sg.edgeNodes[1].meshComponent.boundingBox.center.pos
							- sg.edgeNodes[0].meshComponent.boundingBox.center.pos;
							svec = repSG.edgeNodes[1].meshComponent.boundingBox.center.pos
							- repSG.edgeNodes[0].meshComponent.boundingBox.center.pos;
						}
						else if (sg.connNodes[1] != sg.connNodes[0] && repSG.connNodes[1] != repSG.connNodes[0])
						{
							tvec = sg.connNodes[1].meshComponent.boundingBox.center.pos
							- sg.connNodes[0].meshComponent.boundingBox.center.pos;
							svec = repSG.connNodes[1].meshComponent.boundingBox.center.pos
							- repSG.connNodes[0].meshComponent.boundingBox.center.pos;
						}
						tvec = tvec.Normalize(); svec = svec.Normalize();
						acos = tvec.Dot(svec);
						double ag = Math.Acos(acos);
						if (ag > angle)
						{
							angle = ag;
							sv = svec;
							tv = tvec;
						}
					}
				}
                

				Vector3d vz = tv.Cross(sv);
				
				
				//angle = Math.PI;
                Matrix4d rotMat = new Matrix4d();
                rotMat[0, 0] = Math.Cos(angle);
                rotMat[0, 2] = -1 *Math.Sin(angle);
                rotMat[2, 0] = Math.Sin(angle);
				if (vz.y < 0)
				{
					rotMat[0, 2] = Math.Sin(angle);
					rotMat[2, 0] = -1*Math.Sin(angle);
				}
                rotMat[2, 2] = Math.Cos(angle);
                rotMat[1, 1] = 1;

                //this.FindBestTransform(srcpts, tarpts, out S, out T, out Q);

				//T = Matrix4d.TranslationMatrix(sg.edgeNodes[0].meshComponent.boundingBox.center.pos
				//    - repSG.edgeNodes[0].meshComponent.boundingBox.center.pos);

				Vector3d edge_pos = repSG.edgeNodes[0].meshComponent.boundingBox.center.pos;
                foreach (ModelGraphNode mn in repSG.nodeSets)
                {
					ModelGraphNode repnode = mn.Clone() as ModelGraphNode;
					repnode.name = @"iter_" + iterIdx.ToString() + @"_" + mn.name;
					repnode.pos3D = repnode.meshComponent.boundingBox.center.pos;
					repnode.level = mn.level + diffLevel;
					repnode.adjNodes = new List<ModelGraphNode>();
					repnode.meshComponent.mesh.Transform(rotMat);
					repnode.meshComponent.mesh.UpdateNormal();
					repnode.meshComponent.boundingBox.Transform(rotMat);
					repnode.pos3D = mn.meshComponent.boundingBox.center.pos;
					editNodes.Add(repnode); //node index will be reset 
					if (mn == repSG.edgeNodes[0]) edge_pos = repnode.meshComponent.boundingBox.center.pos;
                }

				T = Matrix4d.TranslationMatrix(sg.edgeNodes[0].meshComponent.boundingBox.center.pos
					- edge_pos);

				foreach (ModelGraphNode mn in editNodes)
				{
					mn.meshComponent.mesh.Transform(T);
					mn.meshComponent.FindBoundingBox();
				}
            }

			// 1. add nodes
			int[] nodeIdMap = new int[repSG.nodeSets.Count];
            int nidx = 0;
            List<ModelGraphNode> edgeNodes = new List<ModelGraphNode>();
            List<Vector3d> edgeNodePos = new List<Vector3d>();
            List<int> edgeNodeIdx = new List<int>();
            for (int i = 0; i < repSG.edgeNodes.Count; ++i)
            {
                int index = repSG.nodeSets.IndexOf(repSG.edgeNodes[i]);
                edgeNodeIdx.Add(index);
            }

            Subgraph ssg = repSG.Clone() as Subgraph;
            edgeNodes = new List<ModelGraphNode>();
			//for (int i = 0; i < edgeNodeIdx.Count; ++i)
			//{
			//    edgeNodes.Add(ssg.nodeSets[edgeNodeIdx[i]]);
			//    edgeNodePos.Add(sg.edgeNodes[i].meshComponent.boundingBox.center.pos);
			//}
            for (int i = 0; i < editNodes.Count; ++i)
            {
                ModelGraphNode mn = editNodes[i];
                nodeIdMap[nidx++] = mg.graphNodes.Count;
                mg.graphNodes.Add(mn);
            }
			for (int i = 0; i < edgeNodeIdx.Count; ++i)
			{
				edgeNodes.Add(editNodes[edgeNodeIdx[i]]);
				edgeNodePos.Add(sg.edgeNodes[i].meshComponent.boundingBox.center.pos);
			}
			
            // verify orders
			if (mg.modelType == 9)
			{
				if (sg.outgoingEdges.Count == 2 &&
					this.graphGroup.IsEqualEdge(sg.outgoingEdges[0], sg.outgoingEdges[1]))
				{
					// avoid intersection
					Vector3d v1 = editNodes[0].meshComponent.boundingBox.center.pos -
						sg.connNodes[0].meshComponent.boundingBox.center.pos,
						v2 = editNodes[1].meshComponent.boundingBox.center.pos -
						sg.connNodes[1].meshComponent.boundingBox.center.pos;
					double acosv = v1.Dot(v2);
					if (Math.Abs(1 - Math.Abs(acosv)) < 0.2)
					{
						ModelGraphEdge me = repSG.outgoingEdges[0];
						repSG.outgoingEdges.RemoveAt(0);
						repSG.outgoingEdges.Add(me);
						ModelGraphNode mn = repSG.edgeNodes[0];
						repSG.edgeNodes.RemoveAt(0);
						repSG.edgeNodes.Add(mn);
						mn = repSG.connNodes[0];
						repSG.connNodes.RemoveAt(0);
						repSG.connNodes.Add(mn);
					}
				}
				// update level info

				List<int> restGraphNodeLevel = new List<int>(),
					sgNodeLevel = new List<int>(),
					repSGNodeLevel = new List<int>();
				int minRL = int.MaxValue, minSL = int.MaxValue, minRSL = int.MaxValue,
					maxRL = 0, maxSL = 0, maxRSL = 0;
				foreach (ModelGraphNode mn in sg.connNodes)
				{
					restGraphNodeLevel.Add(mn.level);
					if (mn.level > maxRL)
						maxRL = mn.level;
					if (mn.level < minRL)
						minRL = mn.level;
				}
				foreach (ModelGraphNode mn in sg.edgeNodes)
				{
					sgNodeLevel.Add(mn.level);
					if (mn.level > maxSL)
						maxSL = mn.level;
					if (mn.level < minSL)
						minSL = mn.level;
				}
				foreach (ModelGraphNode mn in repSG.edgeNodes)
				{
					repSGNodeLevel.Add(mn.level);
					if (mn.level > maxRSL)
						maxRSL = mn.level;
					if (mn.level < minRSL)
						minRSL = mn.level;
				}
				int levelDiff = minSL - minRSL;
				int trans = maxRL - (maxRSL + levelDiff) - 1;
				if (maxSL < minRL)
				{
					levelDiff = 0;
					trans = maxRSL - maxSL;
				}
				foreach (ModelGraphNode mn in editNodes)
					mn.level += levelDiff;
				List<ModelGraphNode> spanNodes = new List<ModelGraphNode>();
				foreach (ModelGraphNode mn in sg.connNodes)
				{
					if (mn.level > maxSL)
						spanNodes.Add(mn);
				}
				bool[] updated = new bool[mg.graphNodes.Count];
				while (true)
				{
					if (spanNodes.Count == 0) break;
					int nnodes = spanNodes.Count;
					for (int i = 0; i < nnodes; ++i)
					{
						foreach (ModelGraphEdge me in spanNodes[0].edges)
						{
							if (sg.outgoingEdges.Contains(me)) continue;
							ModelGraphNode other = me.conntectingNodes[0] == spanNodes[0] ?
								me.conntectingNodes[1] : me.conntectingNodes[0];
							if (!updated[other.nodeIdx])
								spanNodes.Add(other);
						}
						spanNodes[0].level += trans;
						updated[spanNodes[0].nodeIdx] = true;
						spanNodes.RemoveAt(0);
					}
				}
			}

            if (mg.faceCycles != null)
            {
                foreach (ModelGraphEdge me in sg.ingoingEdges)
                {
                    for (int i = 0; i < mg.faceCycles.Count; ++i)
                    {
                        if (mg.faceCycles[i].edges.Contains(me))
                            mg.faceCycles[i].edges.Remove(me);
                    }
                }

                foreach (ModelGraphNode mn in sg.nodeSets)
                {
                    for (int i = 0; i < mg.faceCycles.Count; ++i)
                    {
                        if (mg.faceCycles[i].vertices != null &&
                            mg.faceCycles[i].vertices.Contains(mn))
                            mg.faceCycles[i].vertices.Remove(mn);
                    }
                }
            }
			// 2. ingoing edges --- adj edges&nodes are already ordered
			int nEditEdges = repSG.ingoingEdges.Count + repSG.outgoingEdges.Count;
			int[] edgeIdMap = new int[nEditEdges];
			int eidx = 0;
			List<ModelGraphEdge> oriEdges = new List<ModelGraphEdge>();
			oriEdges.AddRange(repSG.ingoingEdges);
			oriEdges.AddRange(repSG.outgoingEdges);
			List<int> inedgeCycleIdxs = new List<int>();
			List<int> shareCycleWithCutEdge = new List<int>();
			List<int> inedgeNewIdx = new List<int>();
			foreach (ModelGraphEdge me in repSG.ingoingEdges)
			{
				int foundcycle = -1;
				for (int i = 0; i < repSG.outgoingEdges.Count; ++i )
				{
					ModelGraphEdge ome = repSG.outgoingEdges[i];
					List<int> sects = me.cycleIndices.Intersect(ome.cycleIndices).ToList();
					if (sects.Count() == 1)
					{
						foundcycle = i;
						break;
					}
				}
				shareCycleWithCutEdge.Add(foundcycle);

				ModelGraphNode mn1 = me.conntectingNodes[0];
				ModelGraphNode mn2 = me.conntectingNodes[1];
				int nidx1 = repSG.nodeSets.IndexOf(mn1);
				int nidx2 = repSG.nodeSets.IndexOf(mn2);
				ModelGraphEdge mge = new ModelGraphEdge(me.pos, mg.graphIndex, mg.graphEdges.Count);
                mge.faceIndex[0] = me.faceIndex[0];
                mge.faceIndex[1] = me.faceIndex[1];
				ModelGraphNode newmn1 = mg.graphNodes[nodeIdMap[nidx1]]; 
				ModelGraphNode newmn2 = mg.graphNodes[nodeIdMap[nidx2]];
				//mge.TransformFromCurrent(transMat);
				mge.AddConnectingNode(newmn1);
				mge.AddConnectingNode(newmn2);
				//newmn1.adjNodes.Add(newmn2);
				//newmn2.adjNodes.Add(newmn1);
				edgeIdMap[eidx++] = mge.edgeIdx;
				if(me.start != null)
				{
					if(me.start == mn1)
					{
						mge.start = newmn1;
						mge.end = newmn2;
					}
					else
					{
						mge.start = newmn2;
						mge.end =  newmn1;
					}
				}
				mge.cycleIndices = new List<int>();
				inedgeNewIdx.Add(mg.graphEdges.Count);
				mg.graphEdges.Add(mge);
				
				if (me.cycleIndices != null && me.cycleIndices.Count > 0)
				{
					foreach (int idx in me.cycleIndices)
						if (!inedgeCycleIdxs.Contains(idx))
							inedgeCycleIdxs.Add(idx);
				}
			}//ingoing edges
			List<int> refercycleIdx = new List<int>();
			foreach (ModelGraphEdge me in sg.outgoingEdges)
			{
				int foundcycle = -1;
				foreach (ModelGraphEdge ime in sg.ingoingEdges)
				{
					List<int> sects = me.cycleIndices.Intersect(ime.cycleIndices).ToList();
					if (sects.Count() == 1)
					{
						foundcycle = sects[0];
						break;
					}
				}
				refercycleIdx.Add(foundcycle);
			}
			for (int i = 0; i < shareCycleWithCutEdge.Count; ++i)
			{
				if (shareCycleWithCutEdge[i] == -1) continue;
				int cidx = refercycleIdx[shareCycleWithCutEdge[i]];
				if ( cidx != -1)
				{
					mg.graphEdges[inedgeNewIdx[i]].cycleIndices.Add(cidx);
					mg.faceCycles[cidx].AddElement(null, mg.graphEdges[inedgeNewIdx[i]]);
				}
			}
			// 2.5 cycles
			// if the ingoing edges of the inserted subgraph can form a cycle
			foreach (int idx in inedgeCycleIdxs)
			{
 				// if the cycle exsit in the ingoing edges, add, otherwise not
				bool exist = true;
				foreach (ModelGraphEdge me in repMG.faceCycles[idx].edges)
				{
					if (!repSG.ingoingEdges.Contains(me))
					{
						exist = false;
						break;
					}
				}
				if (!exist)
				{
					continue;
				}
				// add 
				Face face = new Face();
				int faceId = mg.faceCycles.Count;
				mg.faceCycles.Add(face);
				foreach (ModelGraphEdge me in repMG.faceCycles[idx].edges)
				{
					int index = repSG.ingoingEdges.IndexOf(me);
					ModelGraphEdge newMe = mg.graphEdges[edgeIdMap[index]];
					newMe.cycleIndices.Add(faceId);
					face.edges.Add(newMe);
				}
			}
			//if the ingoing edges of the removed subgraph form a cycle
			inedgeCycleIdxs = new List<int>();
			foreach (ModelGraphEdge me in sg.ingoingEdges)
			{
				if (me.cycleIndices != null && me.cycleIndices.Count > 0)
				{
					foreach (int cidx in me.cycleIndices)
					{
						if (!inedgeCycleIdxs.Contains(cidx)) continue;
						int loc = 0;
						for (int i = 0; i < inedgeCycleIdxs.Count; ++i)
						{
							if (cidx > inedgeCycleIdxs[i])
							{
								loc = i;
								break;
							}
						}
						inedgeCycleIdxs.Insert(loc, cidx);
					}
				}
			}
			// remove cycles, will re-build cycle index for each edge in the end
			foreach (int idx in inedgeCycleIdxs)
				mg.faceCycles.RemoveAt(idx);
			// 3. outgoing edges, just exchange edge nodes
			// the outgoing edges have to be corresponded correctly
			// each outgoing edge corresponds to one node and 
			// one associated node left in the graph (just update the connection, too)
			
			for(int i = 0; i < sg.outgoingEdges.Count; i++)
			{
				ModelGraphNode edgenode = sg.edgeNodes[i];
				nidx = repSG.nodeSets.IndexOf(repSG.edgeNodes[i]);
				ModelGraphNode repnode = mg.graphNodes[nodeIdMap[nidx]];
				ModelGraphEdge me = sg.outgoingEdges[i];
				me.conntectingNodes.Remove(edgenode);
				// change the old connected node to the one in the new subgraph
				ModelGraphNode nodeInLeftGraph = (me.conntectingNodes[0] == edgenode?
					me.conntectingNodes[1]:me.conntectingNodes[0]);
				if (nodeInLeftGraph != sg.connNodes[i])
					break;
				int nindex = nodeInLeftGraph.adjNodes.IndexOf(edgenode);
				nodeInLeftGraph.adjNodes.Remove(edgenode);
				//edgenode.adjNodes.Remove(nodeInLeftGraph);
				me.conntectingNodes.Add(repnode);
				repnode.edges.Add(me);
				repnode.adjNodes.Add(nodeInLeftGraph);
				nodeInLeftGraph.adjNodes.Insert(nindex, repnode);
				if (me.start != null)
				{
					if (me.start == edgenode)
						me.start = repnode;
					else
						me.end = repnode;
				}
				edgeIdMap[eidx++] = me.edgeIdx;
			}

			// 4. corrent order of adj nodes
			for (int i = 0; i < repSG.nodeSets.Count(); ++i )
			{
				ModelGraphNode mn = repSG.nodeSets[i];
				editNodes[i].adjNodes = new List<ModelGraphNode>();
				editNodes[i].edges = new List<ModelGraphEdge>();
				for (int j = 0; j < mn.adjNodes.Count(); ++j )
				{
					ModelGraphNode adjnode = mn.adjNodes[j];
					int nindex = repSG.nodeSets.IndexOf(adjnode);
					// neighboring node in the subgraph
					if (nindex != -1)
					{
						editNodes[i].adjNodes.Add(editNodes[nindex]);
						continue;
					}
					
					int eindex = oriEdges.IndexOf(mn.edges[j]);
					if (eindex != -1)
					{
						// corresponding neighboring edge in the subgraph or 
						// the docking sites (matched outgoing edges)
						eindex = edgeIdMap[eindex];
						ModelGraphEdge editedge = mg.graphEdges[eindex];
						//editNodes[i].edges.Add(editedge);
						if (nindex == -1)
						{
							// node in *mg*
							ModelGraphNode othernode = editedge.conntectingNodes[0] == editNodes[i] ?
								editedge.conntectingNodes[1] : editedge.conntectingNodes[0];
							editNodes[i].adjNodes.Add(othernode);
						}
					}
				}
			}//ordered-nodes-and-edges

			// 5. correct order of adj edgs (after collect adj nodes)
			foreach(ModelGraphNode mn in editNodes)
			{
				int nneigs = mn.adjNodes.Count;
				List<ModelGraphEdge> orderedEdges = new List<ModelGraphEdge>();
				for(int  i = 0; i < nneigs; ++i)
				{
					bool found = false;
					ModelGraphNode ni = mn.adjNodes[i];
					foreach(ModelGraphEdge mge in mg.graphEdges)
					{
						if(mge.conntectingNodes.Contains(mn) && 
							mge.conntectingNodes.Contains(ni))
						{
							found = true;
							orderedEdges.Add(mge);
							break;
						}
					}
					if(!found)
						break;
				}
				for(int  i = 0; i < nneigs; ++i)
				{
					ModelGraphEdge ei = orderedEdges[i];
					ModelGraphEdge ei1 = orderedEdges[(i+1)%nneigs];
					if(ei.adjEdges == null)
						ei.adjEdges = new List<ModelGraphEdge>();
					if(ei1.adjEdges == null)
						ei1.adjEdges = new List<ModelGraphEdge>();
					if(!ei.adjEdges.Contains(ei1))
						ei.adjEdges.Add(ei1);
					if(!ei1.adjEdges.Contains(ei))
						ei1.adjEdges.Add(ei);
				}
			}

			// 7. remove sugbraph from mg
			foreach(ModelGraphEdge mge in sg.ingoingEdges)
			{
				mg.graphEdges.Remove(mge);
				mge.conntectingNodes[0].edges.Remove(mge);
				mge.conntectingNodes[0].adjNodes.Remove(mge.conntectingNodes[1]);
				mge.conntectingNodes[1].edges.Remove(mge);
				mge.conntectingNodes[1].adjNodes.Remove(mge.conntectingNodes[0]);
			}
			foreach(ModelGraphNode mgn in sg.nodeSets)
				mg.graphNodes.Remove(mgn);

			// 8. re-build cycle inde
			foreach (ModelGraphEdge me in mg.graphEdges)
			{
				me.trueAdjEdges = new List<bool>();
				me.cycleIndices = new List<int>();
				foreach (ModelGraphEdge mge in me.adjEdges)
					me.trueAdjEdges.Add(true);
			}

			//reset index
			foreach (ModelGraphNode mn in mg.graphNodes)
			{
				mn.edges = new List<ModelGraphEdge>();
				foreach (ModelGraphNode adjmn in mn.adjNodes)
				{
					bool found = false;
					foreach (ModelGraphEdge me in mg.graphEdges)
					{
						if (me.conntectingNodes.Contains(mn) &&
							me.conntectingNodes.Contains(adjmn))
						{
							mn.edges.Add(me);
							found = true;
							break;
						}
					}
					if (!found)
						break;
				}
			}
			// reset cycle edges
			int faceIdx = 0;
			if (mg.faceCycles != null)
			{
				foreach (Face face in mg.faceCycles)
				{
					int nedges = face.edges.Count;
					for (int i = 0; i < nedges - 1; ++i)
					{
						ModelGraphEdge e1 = face.edges[i];
						for (int j = i + 1; j < nedges; ++j)
						{
							ModelGraphEdge e2 = face.edges[j];
							if (!e1.adjEdges.Contains(e2) && e1 != e2)
							{
								e1.adjEdges.Add(e2);
								e1.trueAdjEdges.Add(true);
							}
							if (!e2.adjEdges.Contains(e1) && e1 != e2)
							{
								e2.adjEdges.Add(e1);
								e2.trueAdjEdges.Add(true);
							}
						}
					}
					foreach (ModelGraphEdge me in face.edges)
					{
						if (!me.cycleIndices.Contains(faceIdx))
							me.cycleIndices.Add(faceIdx);
					}
					++faceIdx;
				}//end-face-iteration
			}
			mg.AssignIndex();

	
				this.BalanceHeight();

			//BoundbingBoxOptimizer boxopt = new BoundbingBoxOptimizer();

			//boxopt.Init(this.modelGraph, edgeNodes, edgeNodePos);

			////boxopt.Init(ssg, edgeNodes, edgeNodePos);

			//double[] x = boxopt.LMOptimize(false);

			//for (int i = 0, j = 0; i < x.Length / 6; ++i, j += 6)
			//{
			//    ModelGraphNode mn = mg.graphNodes[i];
			//    //ModelGraphNode mn = editNodes[i];
			//    Vector3d t = new Vector3d(x[j], x[j + 1], x[j + 2]) -
			//        mn.meshComponent.boundingBox.center.pos;
			//    Vector3d scale = mn.meshComponent.boundingBox.scale;
			//    Vector3d s = new Vector3d(x[j + 3] / scale[0],
			//        x[j + 4] / scale[1], x[j + 5] / scale[2]);
			//    Matrix4d S = Matrix4d.ScalingMatrix(s),
			//        T = Matrix4d.TranslationMatrix(t);

			//    mn.meshComponent.mesh.Transform(T);
			//    mn.meshComponent.boundingBox.Transform(T);
			//    mn.meshComponent.mesh.Transform(S);
			//    mn.meshComponent.boundingBox.Transform(S);
			//    mn.meshComponent.FindBoundingBox();
			//}

		}//End-replace-asubgraph

		public void VaryGeometryForSubgraph(ModelGraph a_mg, ModelGraph b_mg,
			Subgraph a_sg, Subgraph b_sg, out Subgraph new_asg, out Subgraph new_bsg)
		{ 
			// replace a_sg by b_sg, use the geometry from a_mg
			new_asg = new Subgraph();
			new_bsg = new Subgraph();
			
			for (int nSG = 0; nSG < 2; ++nSG)
			{
				Subgraph nsg = new_bsg, sg = b_sg;
				ModelGraph mg = a_mg;
				if (nSG == 1)
				{
					nsg = new_asg;
					sg = a_sg;
					mg = b_mg;
				}
				nsg.nodeSets = new List<ModelGraphNode>();
				nsg.edgeNodes = new List<ModelGraphNode>(sg.edgeNodes);
				foreach (ModelGraphNode mn in sg.nodeSets)
				{
					// find from a_mg
					ModelGraphNode mn_Graph1 = null, temp_mn = null;
					// connected edge / node type in order
					List<int> connNodeTypes = new List<int>();
					foreach (ModelGraphNode adjmn in mn.adjNodes)
						connNodeTypes.Add(adjmn.nodeType);
					int maxMatchdocking = 0;
					foreach (ModelGraphNode a_mn in mg.graphNodes)
					{
						if (a_mn.nodeType != mn.nodeType) continue;
						temp_mn = a_mn;
						//check if the topology satisfies best
						int a_index = 0, b_index = 0, b_start = 0, a_start = 0;
						int nmatch = 0;
						while (true)
						{
							a_index = a_start;
							b_index = b_start;
							nmatch = 0;
							while (true)
							{
								if (connNodeTypes[b_index] == a_mn.adjNodes[a_index].nodeType)
								{
									b_index++;
									nmatch++;
									if (b_index == 0)
										a_start = a_index;
								}
								else if (a_index > a_start || a_mn.adjNodes.Count == 1)
									break;
								if (b_index >= connNodeTypes.Count)
									break;
								a_index = (a_index + 1) % a_mn.adjNodes.Count;
							}
							if (b_index == b_start) // not found
							{
								++b_start;
								a_start = 0;
							}
							else break;
							if (b_start >= connNodeTypes.Count)
								break;
						}
						if (nmatch > maxMatchdocking)
						{
							maxMatchdocking = b_index;
							temp_mn = a_mn;
						}
						if (b_index >= connNodeTypes.Count) // found one match
						{
							mn_Graph1 = a_mn;
							break;
						}
					}//end-search-node-with-same-geometry-in-original-graph
					if (mn_Graph1 == null)
					{
						if (temp_mn != null)
							mn_Graph1 = temp_mn;
						else
							mn_Graph1 = mn;
					}
					nsg.nodeSets.Add(mn_Graph1);
					int edgeNodeIdx = b_sg.edgeNodes.IndexOf(mn);
					if (edgeNodeIdx != -1)
					{
						nsg.edgeNodes.RemoveAt(edgeNodeIdx);
						nsg.edgeNodes.Insert(edgeNodeIdx, mn_Graph1);
					}
				}//end-for-each-node-in-the-set
			}
		}


		public void FindCorrespondingNodes(ModelGraph a_mg, ModelGraph b_mg)
		{			
			for (int ng = 0; ng < 2; ++ng)
			{
				ModelGraph mg1 = a_mg, mg2 = b_mg;
				if (ng == 1)
				{
					mg1 = b_mg;
					mg2 = a_mg;
				}
				List<ModelGraphNode> corrNodes = new List<ModelGraphNode>();
				foreach (ModelGraphNode mn1 in mg1.graphNodes)
				{
					ModelGraphNode mn_Graph1 = null, temp_mn = null;
					List<int> connNodeTypes = new List<int>();
					foreach (ModelGraphNode adjmn in mn1.adjNodes)
						connNodeTypes.Add(adjmn.nodeType);
					int maxMatchdocking = 0;
					foreach (ModelGraphNode mn2 in mg2.graphNodes)
					{
						if (mn2.nodeType != mn1.nodeType) continue;
						//check if the topology satisfies best
						int a_index = 0, b_index = 0, b_start = 0, a_start = 0;
						int nmatch = 0;
						while (true)
						{
							a_index = a_start;
							b_index = b_start;
							nmatch = 0;
							while (true)
							{
								if (connNodeTypes[b_index] == mn2.adjNodes[a_index].nodeType)
								{
									b_index++;
									nmatch++;
									if (b_index == 0)
										a_start = a_index;
								}
								else if (a_index > a_start || mn2.adjNodes.Count == 1)
									break;
								if (b_index >= connNodeTypes.Count)
									break;
								a_index = (a_index + 1) % mn2.adjNodes.Count;
							}
							if (b_index == b_start) // not found
							{
								++b_start;
								a_start = 0;
							}
							else break;
							if (b_start >= connNodeTypes.Count)
								break;
						}
						if (nmatch > maxMatchdocking)
						{
							maxMatchdocking = b_index;
							temp_mn = mn2;
						}
						if (b_index >= connNodeTypes.Count) // found one match
						{
							mn_Graph1 = mn2;
							break;
						}
					}//end-search-node-with-same-geometry-in-original-graph
					if (mn_Graph1 == null)
					{
						if (temp_mn != null)
							mn_Graph1 = temp_mn;
						else
							mn_Graph1 = mn1;
					}
					corrNodes.Add(mn_Graph1);
				}//end-for-each-node-in-the-set
				// save
				string filename = @"F:\Projects\GraphEditing\models\replaceableSubgraphs\playgrounds\";
				filename += mg1.objfilename + "_updateGeometry.txt";
				StreamWriter sw = new StreamWriter(filename);
				for (int i = 0; i < mg1.graphNodes.Count; ++i)
				{
					sw.WriteLine(mg1.graphNodes[i].name + " " +
						corrNodes[i].name);
				}
				sw.Close();
			}
		}

		

		//////***User assistance***//////
		public void SetNodeType(int typeIdx)
		{
			if (this.modelGraph == null || this.modelGraph.graphNodes == null)
				return;
			// Assign node type to each component
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				if (this.selectedMeshComponents.Contains(mn.meshComponent))
				{
					mn.nodeType = typeIdx;
					mn.meshComponent.colorIdx = typeIdx;
				}
			}
		}

		public void AddAEdge()
		{
			// delete an edge with minor intersection (unexpected) between selected nodes
			if (this.selectedMeshComponents.Count != 2)
			{
				MessageBox.Show(" Please select two edges!");
				return;
			}
			ModelGraphNode n1 = this.selectedMeshComponents[0].graphNode;
			ModelGraphNode n2 = this.selectedMeshComponents[1].graphNode;
			bool existed = false;
			if (this.modelGraph.graphEdges == null)
				this.modelGraph.graphEdges = new List<ModelGraphEdge>();
			if (n1.edges != null)
			{
				foreach (ModelGraphEdge e in n1.edges)
				{
					if (e.conntectingNodes.Contains(n2))
					{
						existed = true;
						break;
					}
				}
			}
			if (!existed)
			{
				Vector3d center1 = (n1.meshComponent.mesh.MinCoord() + n1.meshComponent.mesh.MaxCoord()) / 2;
				Vector3d center2 = (n2.meshComponent.mesh.MinCoord() + n2.meshComponent.mesh.MaxCoord()) / 2;
				Vector3d center = (center1 + center2) / 2;
				ModelGraphEdge e = new ModelGraphEdge(center, this.modelGraph.graphIndex, this.modelGraph.graphEdges.Count);
				e.AddConnectingNode(n1);
				e.AddConnectingNode(n2);
				this.modelGraph.graphEdges.Add(e);
			}
		}// End-add-an-edge

		public void DeleteAEdge()
		{
			// delete an edge with minor intersection (unexpected) between selected nodes
			if (this.selectedMeshComponents.Count != 2)
			{
				MessageBox.Show(" Please select two edges!");
				return;
			}
			ModelGraphNode deln1 = this.selectedMeshComponents[0].graphNode;
			ModelGraphNode deln2 = this.selectedMeshComponents[1].graphNode;
			foreach (ModelGraphEdge e in this.modelGraph.graphEdges)
			{
				if (e.conntectingNodes.Contains(deln1)
					&& e.conntectingNodes.Contains(deln2))
				{
					deln1.edges.Remove(e);
					deln2.edges.Remove(e);
					this.modelGraph.graphEdges.Remove(e);
					break;
				}
			}
		}// End-delete-an-edge

		public void MarkEntry()
		{
			// mark entry
			// playgrouds: ladder, hand-ring?
			// facade: door
			if (this.selectedMeshComponents.Count == 0)
			{
				MessageBox.Show("Please select at least one node!");
				return;
			}
			foreach (MeshComponent mc in this.selectedMeshComponents)
			{
				ModelGraphNode mn = mc.graphNode;
				mn.isEntry = true;
				// make *mn* the *start*
				foreach (ModelGraphEdge mge in mn.edges)
				{
					mge.start = mn;
					mge.end = mge.conntectingNodes[0] == mn ?
						mge.conntectingNodes[1] : mge.conntectingNodes[0];
				}
			}
		}// End-mark-entry

		public void MarkExit()
		{
			// mark exit
			// playgrouds: slider, ladder
			// facade: balcony
			if (this.selectedMeshComponents.Count == 0)
			{
				MessageBox.Show("Please select at least one node!");
				return;
			}
			foreach (MeshComponent mc in this.selectedMeshComponents)
			{
				ModelGraphNode mn = mc.graphNode;
				mn.isExit = true;
				// make *mn* the *end*
				foreach (ModelGraphEdge mge in mn.edges)
				{
					mge.end = mn;
					mge.start = mge.conntectingNodes[0] == mn ?
						mge.conntectingNodes[1] : mge.conntectingNodes[0];
					if (mge.end.meshComponent.boundingBox.points3[0].pos.y >
						mge.start.meshComponent.boundingBox.center.pos.y)
					{
						ModelGraphNode temp = mge.end;
						mge.end = mge.start;
						mge.start = temp;
					}
				}
			}
		}// End-mark-exit

		public void UpdateEdges()
		{
			// for re-set edge positions
			foreach(ModelGraphEdge e in this.modelGraph.graphEdges)
			{
				ModelGraphNode n1 = e.conntectingNodes[0];
				ModelGraphNode n2 = e.conntectingNodes[1];
				Vector3d center1 = (n1.meshComponent.mesh.MinCoord() + n1.meshComponent.mesh.MaxCoord()) / 2;
				Vector3d center2 = (n2.meshComponent.mesh.MinCoord() + n2.meshComponent.mesh.MaxCoord()) / 2;
				Vector3d center = (center1 + center2) / 2;
				e.pos = center;
				e.UpdatePos();
			}
			this.visContactPoints = new List<Vector3d>();
			foreach (ModelGraphEdge e in this.modelGraph.graphEdges)
				this.visContactPoints.Add(e.pos);
			this.Refresh();
		}

		public void CalculateContactFace()
		{
			// approximate the touching faces of two bounding box by comparing their extreme vertice
			double[] maxLength = new double[this.modelGraph.graphNodes.Count];
			for(int j = 0; j < this.modelGraph.graphNodes.Count; ++j)
			{
				ModelGraphNode mn = this.modelGraph.graphNodes[j];
				double maxval = double.MinValue;
				for(int i = 0; i < 3;++i)
				{
					double val = mn.meshComponent.boundingBox.points3[6].pos[i] - 
						mn.meshComponent.boundingBox.points3[0].pos[i];
					if( val > maxval)
						maxval = val;
				}
				maxLength[j] = maxval;
			}
			List<Vector3d> axes = new List<Vector3d>{new Vector3d(1, 0, 0), new Vector3d(-1, 0, 0),
				new Vector3d(0, 1, 0), new Vector3d(0, -1, 0), new Vector3d(0, 0, 1), new Vector3d(0, 0, -1)};
			foreach (ModelGraphEdge e in this.modelGraph.graphEdges)
			{
				ModelGraphNode n1 = e.conntectingNodes[0];
				ModelGraphNode n2 = e.conntectingNodes[1];
				int label = 0; // +-1:x, +-2:y, +-3:z
                // simple test on bounding box
                double mindiff = double.MaxValue;
                for (int i = 0; i < 3; ++i)
                {
                    double diff1 = Math.Abs(n1.meshComponent.boundingBox.points3[0].pos[i]
                        - n2.meshComponent.boundingBox.points3[6].pos[i]);
                    double diff2 = Math.Abs(n2.meshComponent.boundingBox.points3[0].pos[i]
                    - n1.meshComponent.boundingBox.points3[6].pos[i]);
                    if (diff1 < diff2 && diff1 < mindiff)
                    {
                        mindiff = diff1;
                        label = -(i + 1);
                    }
                    else if (diff2 < diff1 && diff2 < mindiff)
                    {
                        mindiff = diff2;
                        label = i + 1;
                    }
                }
                if (label == 0)
                {
                    Program.OutputText("Wrong calculation!", true);
                    break;
                }
                e.faceIndex[0] = label;
                e.faceIndex[1] = -label;
				// fit a plane
                //// collect a set of nearest points
                //double threshold = 1e-3;// Math.Min(maxLength[n1.nodeIdx], maxLength[n2.nodeIdx]) / 10;
                //if (n1.meshComponent.vertexCount == 0)
                //    n1.meshComponent.vertexCount = n1.meshComponent.vertexIndex.Length;
                //if (n2.meshComponent.vertexCount == 0)
                //    n2.meshComponent.vertexCount = n2.meshComponent.vertexIndex.Length;
                //bool[] i_visited = new bool[n1.meshComponent.vertexCount],
                //    j_visited = new bool[n2.meshComponent.mesh.VertexCount];
                //List<Vector3d> points = new List<Vector3d>();
                //for (int i = 0; i < n1.meshComponent.mesh.VertexCount; ++i)
                //{
                //    Vector3d vi = new Vector3d(n1.meshComponent.mesh.VertexPos, i * 3);
                //    for (int j = 0; j < n2.meshComponent.mesh.VertexCount; ++j)
                //    {
                //        Vector3d vj = new Vector3d(n2.meshComponent.mesh.VertexPos, j * 3);
                //        if ((vi - vj).Length() < threshold)
                //        {
                //            if (!i_visited[i])
                //            {
                //                points.Add(vi);
                //                i_visited[i] = true;
                //            }
                //            if (!j_visited[j])
                //            {
                //                points.Add(vj);
                //                j_visited[j] = true;
                //            }
                //        }
                //    }
                //}
                //if (points.Count < 3)
                //{
                //    double mindiff = double.MaxValue;
                //    for (int i = 0; i < 3; ++i)
                //    {
                //        double diff1 = Math.Abs(n1.meshComponent.boundingBox.points3[0].pos[i]
                //            - n2.meshComponent.boundingBox.points3[6].pos[i]);
                //        double diff2 = Math.Abs(n2.meshComponent.boundingBox.points3[0].pos[i]
                //        - n1.meshComponent.boundingBox.points3[6].pos[i]);
                //        if (diff1 < diff2 && diff1 < mindiff)
                //        {
                //            mindiff = diff1;
                //            label = -(i + 1);
                //        }
                //        else if (diff2 < diff1 && diff2 < mindiff)
                //        {
                //            mindiff = diff2;
                //            label = i + 1;
                //        }
                //    }
                //    if (label == 0)
                //    {
                //        Program.OutputText("Wrong calculation!", true);
                //        break;
                //    }
                //    e.faceIndex[0] = label;
                //    e.faceIndex[1] = -label;
                //}
                //else
                //{
                //    Vector3d centroid = new Vector3d();
                //    foreach (Vector3d v in points)
                //        centroid += v;
                //    centroid /= points.Count;
                //    Vector3d normal = FittingPlane(points, centroid);
                //    double minangle = double.MaxValue;
                //    int axis = -1, dir = 0;
                //    for (int i = 0; i < 6; ++i)
                //    {
                //        double angle = Math.Acos(normal.Dot(axes[i]) / normal.Length());
                //        if (angle < minangle)
                //        {
                //            minangle = angle;
                //            axis = i;
                //        }
                //    }
                //    if (axis < 2)
                //        dir = 1;
                //    else if (axis == 2 || axis == 3)
                //        dir = 2;
                //    else
                //        dir = 3;
                //    if (Math.Abs(n1.meshComponent.boundingBox.points3[6].pos[dir - 1] - centroid[dir - 1]) <
                //            Math.Abs(n1.meshComponent.boundingBox.points3[0].pos[dir - 1] - centroid[dir - 1]))
                //        e.faceIndex[0] = dir;
                //    else
                //        e.faceIndex[0] = -dir;

                //    e.faceIndex[1] = -e.faceIndex[0];
                //}
			}
			foreach (ModelGraphEdge e in this.modelGraph.graphEdges)
			{
				e.faceIndex[0] = MapSignToOrder(e.faceIndex[0]);
				e.faceIndex[1] = MapSignToOrder(e.faceIndex[1]);
			}
		}// Calculate Edge face index

		private Vector3d FittingPlane(List<Vector3d> points, Vector3d centroid)
		{
			int npoints = points.Count;
			double[] distances = new double[npoints * 3];
			
			int idx = 0;
			foreach (Vector3d v in points)
			{
				Vector3d vd = v - centroid;
				for (int i = 0; i < 3; ++i)
					distances[idx++] = vd[i];
			}
			SVD svd = new SVD(distances, npoints, 3);
			// find smalest singular value
			double minVal = double.MaxValue;
			idx = -1;
			for (int i = 1; i < 4; ++i)
			{
				if (svd.w[i] < minVal)
				{
					minVal = svd.w[i];
					idx = i;
				}
			}
			return new Vector3d(svd.u[1, idx], svd.u[2, idx], svd.u[3, idx]);
		}

		private int MapPolygonIndex(int direction)
		{
			// map the direction (x,y,z) to the index of polygons in a boundingbox
			int index = -1;
			switch (direction)
			{
				case 1:
					index = 4;
					break;
				case 3:
					index = 2;
					break;
				case 6:
					index = 3;
					break;
				case 5:
					index = 5;
					break;
				case 4:
					index = 1;
					break;
				case 2:
					index = 0;
					break;
			}
			return index;
		}

		private int MapSignToPolygonIndex(int direction)
		{
			// map the direction (x,y,z) to the index of polygons in a boundingbox
			int index = -1;
			switch (direction)
			{
				case 1:
					index = 4;
					break;
				case -1:
					index = 2;
					break;
				case 2:
					index = 3;
					break;
				case -2:
					index = 5;
					break;
				case 3:
					index = 1;
					break;
				case -3:
					index = 0;
					break;
			}
			return index;
		}

		private int MapSignToOrder(int direction)
		{
			// map the direction (x,y,z) to the index of polygons in a boundingbox
			int index = -1;
			switch (direction)
			{
				case 1:
					index = 1;
					break;
				case -1:
					index = 3;
					break;
				case 2:
					index = 6;
					break;
				case -2:
					index = 5;
					break;
				case 3:
					index = 4;
					break;
				case -3:
					index = 2;
					break;
			}
			return index;
		}

		private int InverseMapPolygonIndex(int direction)
		{
			// map the direction (x,y,z) to the index of polygons in a boundingbox
			int index = -1;
			switch (direction)
			{
				case 4:
					index = 1;
					break;
				case 2:
					index = -1;
					break;
				case 3:
					index = 2;
					break;
				case 5:
					index = -2;
					break;
				case 1:
					index = 3;
					break;
				case 0:
					index = -3;
					break;
			}
			return index;
		}

        private int InverseMapPolygonIndex2(int direction)
        {
            // map the direction (x,y,z) to the index of polygons in a boundingbox
            int index = -1;
            switch (direction)
            {
                case 4:
                    index = 1;
                    break;
                case 2:
                    index = 3;
                    break;
                case 3:
                    index = 6;
                    break;
                case 5:
                    index = 5;
                    break;
                case 1:
                    index = 4;
                    break;
                case 0:
                    index = 2;
                    break;
            }
            return index;
        }

		public Vector3d GetBoundingboxSize(PolyProxy bb)
		{
			Vector3d size = new Vector3d();
			size.x = bb.points3[6].pos.x - bb.points3[0].pos.x;
			size.y = bb.points3[6].pos.y - bb.points3[0].pos.y;
			size.z = bb.points3[6].pos.z - bb.points3[0].pos.z;
			return size;
		}

		// the two selected faces to correct
		List<MeshComponent> selectedMCs = new List<MeshComponent>();
		List<int> selectedFaceIdx = new List<int>();

		public void SelectBoxFacesByComponent()
		{
			if (this.selectedMeshComponents.Count() != 1)
			{
				MessageBox.Show("Please choose only one bounding box.");
				return;
			}
			int currMCidx = -1;
			for (int i = 0; i < this.selectedMCs.Count; ++i)
				if (this.selectedMCs[i] == this.selectedMeshComponents[0])
					currMCidx = i;
			if (selectedMCs.Count() >= 2 && currMCidx == -1)
			{
				this.selectedMCs.RemoveAt(0);
				this.selectedFaceIdx.RemoveAt(0);
			}
			if (currMCidx == -1)
			{
				this.selectedMCs.Add(this.selectedMeshComponents[0]);
				this.selectedFaceIdx.Add(-1);
				currMCidx = this.selectedMCs.Count() - 1;
			}
			this.selectedFaceIdx[currMCidx] = (this.selectedFaceIdx[currMCidx] + 1) % 6;
		}//SelectBoxFacesByComponent

		public void CorrectContactFaceIndex()
		{
			if (this.selectedMCs.Count() != 2)
			{
				MessageBox.Show("Please choose two bounding boxes.");
				return;
			}
			ModelGraphNode node1 = this.selectedMCs[0].graphNode;
			ModelGraphNode node2 = this.selectedMCs[1].graphNode;
			foreach (ModelGraphEdge me in node1.edges)
			{
				if (me.conntectingNodes.Contains(node2))
				{
					if (me.conntectingNodes[0] == node1)
					{
						me.faceIndex[0] = InverseMapPolygonIndex2(this.selectedFaceIdx[0]);
						me.faceIndex[1] = InverseMapPolygonIndex2(this.selectedFaceIdx[1]);
					}
					else
					{
						me.faceIndex[0] = InverseMapPolygonIndex2(this.selectedFaceIdx[1]);
						me.faceIndex[1] = InverseMapPolygonIndex2(this.selectedFaceIdx[0]);
					}
					break;
				}
			}
			this.selectedMCs.Clear();
			this.selectedFaceIdx.Clear();
		}//CorrectContactFaceIndex

        public void Labelcycle()
        {
            if (this.selectedMeshComponents.Count == 0) return;
            List<ModelGraphNode> nodes = new List<ModelGraphNode>();
            foreach (MeshComponent mc in this.selectedMeshComponents)
            {
                nodes.Add(mc.graphNode);
            }
            Subgraph sg = this.modelGraph.GetSubgraph(nodes);
			if(this.modelGraph.faceCycles == null)
				this.modelGraph.faceCycles = new List<Face>();
            Face face = new Face();
			face.vertices.AddRange(nodes);
            for (int i = 0; i < sg.ingoingEdges.Count; ++i)
            {
                face.edges.Add(sg.ingoingEdges[i]);
            }
			this.modelGraph.faceCycles.Add(face);
        }
		

		//********* draw *********//
		public void DrawModelGraph()
		{
			if (modelGraph.graphEdges == null || modelGraph.graphEdges.Count == 0) return;
			// draw 
			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			float radius = 6f;
			int circle_points = 100;
			float PI2 = 2.0f * 3.1416f;
			float angle = PI2 / circle_points;
			Gl.glPointSize(2.0f);
			foreach (MeshComponent mc in this.modelGraph.meshComponents)
			{
				Point3 center3 = mc.boundingBox.center;
				Color cc = colorMall[mc.colorIdx];
				Vector2d center = modelGraph.nodePos[mc.componentIndex];
				Gl.glColor3ub(cc.R, cc.G, cc.B);
				Gl.glBegin(Gl.GL_POLYGON);
				for (float ag = 0; ag < PI2; ag += angle)
				{
					Gl.glVertex2d(center.x + radius * Math.Cos(ag), center.y + radius * Math.Sin(ag));
				}
				Gl.glEnd();
			}

			Color linec = Color.DimGray;
			Gl.glColor3ub(linec.R, linec.G, linec.B);
			Gl.glLineWidth(2.0f);
			Gl.glBegin(Gl.GL_LINES);
			for (int i = 0; i < this.modelGraph.graphEdges.Count; ++i)
			{
				MeshComponent mc1 = this.modelGraph.graphEdges[i].conntectingNodes[0].meshComponent;
				MeshComponent mc2 = this.modelGraph.graphEdges[i].conntectingNodes[1].meshComponent;
				Vector2d center1 = modelGraph.nodePos[mc1.componentIndex];
				Vector2d center2 = modelGraph.nodePos[mc2.componentIndex];
				Gl.glVertex2d(center1.x, center1.y);
				Gl.glVertex2d(center2.x, center2.y);
			}
			Gl.glEnd();

			Gl.glEnable(Gl.GL_LIGHTING);
		}//end-draw model graph

		public void DrawModelGraph3D(int tag)
		{
			// tag = 0: meshView; tag = 1: graphView (small window)
			if (this.modelGraph == null || modelGraph.graphEdges == null || modelGraph.graphEdges.Count == 0) return;
			// draw 
			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			double radius = 0.03;
			if (tag == 1)
				radius = 0.04;
			// Draw nodes			
			Glu.GLUquadric quad = Glu.gluNewQuadric();
			foreach (ModelGraphNode mc in this.modelGraph.graphNodes)
			{
				Point3 center3 = mc.meshComponent.boundingBox.center;
				Color cc = colorMall[mc.nodeType];
				Gl.glColor3ub(cc.R, cc.G, cc.B);
				//if (mc.level != -1 && mc.level == this.mcIdx)
				//{
				//    Gl.glColor3ub(0, 0, 0);
				//}
				Gl.glPushMatrix();
				Gl.glTranslated(center3.pos.x, center3.pos.y, center3.pos.z);
				//Gl.glTranslated(mc.pos3D.x, mc.pos3D.y, mc.pos3D.z);
				Glu.gluSphere(quad, radius, 40, 40);
				Gl.glPopMatrix();
			}
			Glu.gluDeleteQuadric(quad);

            Gl.glLineWidth(4.0f);
            if (this.modelGraph.equCutpairs != null && this.modelGraph.equCutpairs.Count > 0 && this.mcIdx >= 0)
            {
                Gl.glBegin(Gl.GL_LINES);
                Color color1 = Color.Red, color2 = Color.Green, color = Color.Black;
                cutSet = new List<ModelGraphEdge>();
                cutSet.AddRange(this.modelGraph.equCutpairs[mcIdx * 2]);
                cutSet.AddRange(this.modelGraph.equCutpairs[mcIdx * 2 + 1]);
                for (int i = 0; i < 2; ++i)
                {
                    if (i % 2 == 0) color = color1;
                    else color = color2;
                    foreach (ModelGraphEdge c in this.modelGraph.equCutpairs[mcIdx * 2 + i])
                    {
                        ModelGraphNode mc1 = c.conntectingNodes[0];
                        ModelGraphNode mc2 = c.conntectingNodes[1];
                        Point3 center1 = mc1.meshComponent.boundingBox.center;
                        Point3 center2 = mc2.meshComponent.boundingBox.center;
                        Gl.glColor3ub(color.R, color.G, color.B);
                        Gl.glVertex3d(center1.pos.x, center1.pos.y, center1.pos.z);
                        Gl.glVertex3d(center2.pos.x, center2.pos.y, center2.pos.z);
                    }
                }
                Gl.glEnd();
            }

			// Draw Edges
			Color linec = Color.Black;
			Gl.glColor3ub(linec.R, linec.G, linec.B);
			Gl.glLineWidth(3.0f);
			if (tag == 1)
				Gl.glLineWidth(2.0f);
			Gl.glBegin(Gl.GL_LINES);
			//***TEST*****// current dual edge
			ModelGraphEdge edge = null;
			//end***TEST*****// 
			//if (this.mcIdx >= 0 && this.mcIdx < this.modelGraph.graphEdges.Count)
			//    edge = this.modelGraph.graphEdges[this.mcIdx];
			foreach (ModelGraphEdge e in this.modelGraph.graphEdges)
			{
				if (cutSet != null && cutSet.Contains(e)) continue;
				ModelGraphNode mc1 = e.conntectingNodes[0];
				ModelGraphNode mc2 = e.conntectingNodes[1];
				//Point3 center1 = mc1.meshComponent.boundingBox.center;
				//Point3 center2 = mc2.meshComponent.boundingBox.center;
				//***TEST*****// 
				if ((edge != null && e == edge) || e.start != null)
				{
					Gl.glColor3ub(255, 0, 0);
				}
				else
					Gl.glColor3ub(linec.R, linec.G, linec.B);
				//end***TEST*****// 
				Gl.glVertex3d(mc1.meshComponent.boundingBox.center.pos.x,
                    mc1.meshComponent.boundingBox.center.pos.y,
                    mc1.meshComponent.boundingBox.center.pos.z);
                Gl.glVertex3d(mc2.meshComponent.boundingBox.center.pos.x,
                    mc2.meshComponent.boundingBox.center.pos.y,
                    mc2.meshComponent.boundingBox.center.pos.z);
			}
			Gl.glEnd();

			if (edge != null)
			{
				if (edge.faceIndex[0] != 0)
				{
					int polyIndex_0 = MapPolygonIndex(edge.faceIndex[0]),
						polyIndex_1 = MapPolygonIndex(edge.faceIndex[1]);
					DrawProxy_Highlight(edge.conntectingNodes[0].meshComponent.boundingBox, polyIndex_0, 1);
					DrawProxy_Highlight(edge.conntectingNodes[1].meshComponent.boundingBox, polyIndex_1, 1);
				}
			}

			//***Equivalent cut***// 
			if (this.ShowComponentProxy && this.selectedMCs != null)
			{
				for (int i = 0; i < this.selectedMCs.Count; ++i)
				{
					DrawProxy_Highlight(this.selectedMCs[i].boundingBox, this.selectedFaceIdx[i], 2);
				}
				//if(this.highlight_componentIdx != -1)
				//    DrawProxy_Highlight(this.modelGraph.meshComponents[highlight_componentIdx].boundingBox,
				//            this.highlight_faceIdx, 2);
			}

			//***Equivalent cut***// 
           

			Gl.glEnable(Gl.GL_LIGHTING);
		}//end-draw model graph

        public int searchMODE = 1;
		public void DrawMatchedModelGraph()
		{
			//if (this.sourceGraph == null || this.targetGraph == null) return;
			Gl.glDisable(Gl.GL_LIGHTING);
			Gl.glEnable(Gl.GL_LINE_SMOOTH);
			double radius = 0.015;
			// Draw nodes			
			List<ModelGraphNode> labeledNodes = new List<ModelGraphNode>();
			if (this.modelGraph.equSubgraphs != null && this.mcIdx > -1)
			{
				foreach (ModelGraphNode mn in this.modelGraph.equSubgraphs[this.mcIdx].nodeSets)
					labeledNodes.Add(mn);
			}
			Glu.GLUquadric quad = Glu.gluNewQuadric();
			foreach (ModelGraphNode mc in this.modelGraph.graphNodes)
			{
				//Point3 center3 = mc.meshComponent.boundingBox.center;
				if (mc.nodeType == -1)
					mc.nodeType = 0;
				Color cc = colorMall[mc.nodeType];
				Gl.glColor3ub(cc.R, cc.G, cc.B);
				Gl.glPushMatrix();
				//Gl.glTranslated(center3.pos.x, center3.pos.y, center3.pos.z);
				Gl.glTranslated(mc.pos3D.x, mc.pos3D.y, mc.pos3D.z);
				//if(labeledNodes.Contains(mc))
				//    Glu.gluSphere(quad, 0.03, 40, 40);
				//else
					Glu.gluSphere(quad, radius, 40, 40);
				Gl.glPopMatrix();
			}
			Glu.gluDeleteQuadric(quad);
			// Draw Edges		

			//***Equivalent cut***// 
			List<ModelGraphEdge> labeledEdges = new List<ModelGraphEdge>();
			
			Gl.glLineWidth(0.8f);

                if (this.cutPairs != null && this.cutPairs.Count > 0 && this.mcIdx >= 0)
                {
                    Gl.glBegin(Gl.GL_LINES);
                    Color color = Color.Red, colord = Color.Green;
                    foreach (ModelGraphEdge e in this.cutPairs[mcIdx])
                    {
                        ModelGraphNode mc1 = e.conntectingNodes[0];
                        ModelGraphNode mc2 = e.conntectingNodes[1];
                        if (e.isDummyEdge)
                            Gl.glColor3ub(colord.R, colord.G, colord.B);
                        else
                            Gl.glColor3ub(color.R, color.G, color.B);
                        Gl.glVertex3d(mc1.pos3D.x, mc1.pos3D.y, mc1.pos3D.z);
                        Gl.glVertex3d(mc2.pos3D.x, mc2.pos3D.y, mc2.pos3D.z);
                        labeledEdges.Add(e);
                    }
                    Gl.glEnd();
                }
          

			Color linec = Color.Black;
			Gl.glColor3ub(linec.R, linec.G, linec.B);
			Gl.glLineWidth(1.0f);
			Gl.glBegin(Gl.GL_LINES);
			foreach (ModelGraphEdge e in this.modelGraph.graphEdges)
			{
				//if (!e.isDummyEdge) continue;
				if (labeledEdges.Contains(e)) continue;
				ModelGraphNode mc1 = e.conntectingNodes[0];
				ModelGraphNode mc2 = e.conntectingNodes[1];
				if (mc1 == null || mc2 == null)
					linec = Color.Black;
				if (e.isDummyEdge)
				{
					Color adde = Color.Gray;
					Gl.glColor3ub(adde.R, adde.G, adde.B);
				}
				else
					Gl.glColor3ub(linec.R, linec.G, linec.B);
				Gl.glVertex3d(mc1.pos3D.x, mc1.pos3D.y, mc1.pos3D.z);
				Gl.glVertex3d(mc2.pos3D.x, mc2.pos3D.y, mc2.pos3D.z);
			}
			Gl.glEnd();

			Gl.glEnable(Gl.GL_LIGHTING);
		}//end-draw model graph

		//*********end_draw *********//

		//***REUSE CODE***//
		public void FindGroundTouching(ModelGraph g)
		{
			if (g == null || g.meshComponents == null) return;

			List<MeshComponent> mesh_components = g.meshComponents;

			// ground contacts
			double miny = double.MaxValue; Polygon plg = null;
			foreach (MeshComponent mc in mesh_components)
			{
				foreach (Polygon p in mc.boundingBox.polygons)
				{
					double yy = p.center.pos.y;
					if (yy < miny)
					{
						miny = yy;
						plg = p;
					}
				}
			}

			double thres2 = 1.0 / 64;
			foreach (MeshComponent mc in mesh_components)
			{
				foreach (Polygon p in mc.boundingBox.polygons)
				{
					double yy = p.center.pos.y;
					if (Math.Abs(yy - miny) < thres2)
					{
						mc.isgroundcontact = true;
						break;
					}
				}
			}

			g.ground_touch_ycoord = miny;
		}

		public void BuildMetaComponents(ModelGraph g)
		{
			g.metaComponents = new List<MeshComponent>();

			int numComponnts = g.meshComponents.Count;
			bool[] tag = new bool[numComponnts];
			for (int i = 0; i < numComponnts; ++i)
			{
				if (tag[i]) continue;

				// find components with same tag name
				List<MeshComponent> components = new List<MeshComponent>();
				components.Add(g.meshComponents[i]);
				tag[i] = true;
				string name = g.meshComponents[i].name;
				if (name == "Unknown")
					continue;
				for (int j = i + 1; j < numComponnts; ++j)
				{
					if (!tag[j] && g.meshComponents[j].name == name)
					{
						components.Add(g.meshComponents[j]);
						tag[j] = true;
					}
				}

				// group components 
				MeshComponent d = new MeshComponent(null);

				// add parent-kid relation
				List<MeshComponent> kid_components = new List<MeshComponent>();
				foreach (MeshComponent c in components)
				{
					if (c.kids == null)
						kid_components.Add(c);
					else
						kid_components.AddRange(c.kids);
				}
				d.kids = kid_components;
				foreach (MeshComponent c in kid_components)
				{
					c.parent = d;
				}
				d.CreateMesh();

				g.metaComponents.Add(d);

			}

			// assume that the contacts are updated with the constrained components
			// first tag the ground-touching ones
			foreach (MeshComponent mc in g.metaComponents)
			{
				foreach (MeshComponent kid in mc.kids)
				{
					if (kid.isgroundcontact)
					{
						mc.isgroundcontact = true;
						break;
					}
				}
			}

		}//end-BuildMetaComponents

		public Mesh trackball = null;

		//********* animation *********//

		public void RollingBall()
		{
			if (this.trackball == null)
			{
				MessageBox.Show("Please Load a ball!");
				return;
			}
			//SelectARandomBallPos();
			List<ModelGraphNode> bridges = new List<ModelGraphNode>();
			List<ModelGraphNode> outputs = new List<ModelGraphNode>();
			ModelGraphNode output = null;
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				if ((mn.name.Length >= 6 && mn.name.Contains("bridge")) ||
					(mn.name.Length >= 4 && mn.name.Contains("fork")))
					bridges.Add(mn);
				if (mn.name.Contains("output")) // for check convinence
				{
					output = mn;
					bridges.Add(mn);
					outputs.Add(mn);
				}
			}

			double speed = (this.modelGraph.mesh.MaxCoord().y -
				this.modelGraph.mesh.MinCoord().y) / 400; //can adjust
			bool stopRolling = false;
			Vector3d groundDir = new Vector3d(0, -1, 0);
			Vector3d fall = new Vector3d(0, -speed, 0);
			// fall matrix
			Matrix4d fallMat = Matrix4d.TranslationMatrix(fall);
			Vector3d ballCenter = (this.trackball.MaxCoord() +
					this.trackball.MinCoord()) / 2, ballMin = new Vector3d();
			double thresh = 0.04;
			while (true)
			{
				if (stopRolling) break;
				if (Math.Abs(ballCenter.y) < thresh) break;
				this.trackball.Transform(fallMat);
				this.Refresh();
				// whetehr meet a bridge				
				ballCenter = (this.trackball.MaxCoord() + this.trackball.MinCoord()) / 2;
				ModelGraphNode aBridge = null;
				double ymin = this.trackball.MinCoord().y;
				bool hashole = true;
				foreach (ModelGraphNode mn in bridges)
				{
					bool inbox = false;
					if (ballCenter.x > mn.meshComponent.boundingBox.points3[0].pos.x &&
						ballCenter.x < mn.meshComponent.boundingBox.points3[6].pos.x &&
						ballCenter.y > mn.meshComponent.boundingBox.points3[0].pos.y &&
						ballCenter.y < mn.meshComponent.boundingBox.points3[6].pos.y &&
						ballCenter.z > mn.meshComponent.boundingBox.points3[0].pos.z &&
						ballCenter.z < mn.meshComponent.boundingBox.points3[6].pos.z)
						inbox = true;
					if (!inbox) continue;
					// judge whether there is a hole

					ballMin = ballCenter;
					ballMin.y = ymin;
					for (int i = 0; i < mn.meshComponent.mesh.VertexCount; ++i)
					{
						Vector3d posi = new Vector3d(mn.meshComponent.mesh.VertexPos, i * 3);
						if ((ballMin - posi).Length() > 0.05) continue;
						double cosval = (posi - ballCenter).Dot(groundDir) / (posi - ballCenter).Length();
						if (Math.Abs(Math.Abs(cosval) - 1) < 0.03)
						{
							hashole = false;
							break;
						}
					}
					aBridge = mn;
					if (!hashole) // if blocked, continue falling
					{
						if (aBridge.name.Contains("output"))
						{
							stopRolling = true;
							//aBridge = null;
						}
					}
					break;
				}//check bridge
				if (aBridge == null || (aBridge != null && hashole))
				{
					bridges.Remove(aBridge);
					continue;
				}

				// meet a bridge
				double lenToMin, lenToMax;
				Vector3d horiDir = new Vector3d();
				double maxdist = double.MinValue;
				Vector3d[] maxDistPoints = new Vector3d[2];
				for (int i = 0; i < aBridge.meshComponent.mesh.VertexCount - 1; ++i)
				{
					Vector3d point_i = new Vector3d(aBridge.meshComponent.mesh.VertexPos, i * 3);
					if (stopRolling && point_i.y > thresh) continue;
					for (int j = i + 1; j < aBridge.meshComponent.mesh.VertexCount; ++j)
					{
						Vector3d point_j = new Vector3d(aBridge.meshComponent.mesh.VertexPos, j * 3);
						if (stopRolling && point_j.y > thresh) continue;
						double dist = (point_i - point_j).Length();
						if (dist > maxdist)
						{
							horiDir = point_i - point_j;
							maxdist = dist;
							maxDistPoints[0] = point_i;
							maxDistPoints[1] = point_j;
						}
					}
				}

				lenToMax = (ballCenter - maxDistPoints[0]).Length();
				lenToMin = (ballCenter - maxDistPoints[1]).Length();
				if (lenToMax < lenToMin)
					horiDir = -1 * horiDir;

				double bridgeLength = horiDir.Length();
				horiDir[1] = 0; // horizontal direction
				horiDir = horiDir.Normalize();
				if (aBridge.name.Contains("fork"))
					bridgeLength /= 2;
				int rollHoriTimes = 100;
				double horiStep = bridgeLength / rollHoriTimes;
				horiDir *= horiStep;
				Matrix4d horiTransMat = Matrix4d.TranslationMatrix(horiDir);

				int nhoriRoll = 0;
				int stopHoriRoll = (int)(rollHoriTimes * 0.85);
				if (stopRolling)
				{
					stopHoriRoll = (int)(rollHoriTimes * 0.35);
					while (true)
					{
						if (Math.Abs(ballCenter.y) < thresh) break;
						this.trackball.Transform(fallMat);
						ballCenter = (this.trackball.MaxCoord() +
							this.trackball.MinCoord()) / 2;
						this.Refresh();
					}
				}
				while (true)
				{
					this.trackball.Transform(horiTransMat);
					Matrix4d rotMat = Matrix4d.RotationMatrix((this.trackball.MaxCoord()
						- this.trackball.MinCoord()).Normalize(), Math.PI / 30);
					this.trackball.TransformCenter(rotMat);
					this.Refresh();

					ballCenter = (this.trackball.MaxCoord() +
					this.trackball.MinCoord()) / 2;
					nhoriRoll++;
					if (nhoriRoll > stopHoriRoll)
						break;
				}// horizontal rolling
				bridges.Remove(aBridge);
			}// rolling
		}//RollingBall

		public void RotateWheels()
		{
			List<ModelGraphNode> rotNodes = new List<ModelGraphNode>();
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				if (mn.name.Length >= 5 &&
					(mn.name.Contains("wheel") ||
					mn.name.Contains("pedal")))
					rotNodes.Add(mn);
			}
			// rotate the involved nodes
			double angle = Math.PI / 20;
			int rotTimes = 1000;
			int begin = 0;
			Matrix4d rotMat = new Matrix4d();
			rotMat[0, 0] = Math.Cos(angle);
			rotMat[0, 1] = -1 * Math.Sin(angle);
			rotMat[1, 0] = Math.Sin(angle);
			rotMat[1, 1] = Math.Cos(angle);
			rotMat[2, 2] = 1;
			rotMat[3, 3] = 1;
			Vector3d trans = new Vector3d(-0.005, 0, 0);
			Matrix4d transMat = Matrix4d.TranslationMatrix(trans);
			while (begin < rotTimes)
			{
				foreach (ModelGraphNode mn in rotNodes)
				{
					mn.meshComponent.mesh.TransformCenter(rotMat);
				}
				foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
				{
					Vector3d center = mn.meshComponent.boundingBox.center.pos;
					mn.meshComponent.mesh.TransformCenter(transMat);
				}
				this.Refresh();
				Thread.Sleep(10);
				begin++;
			}
		}//RotateWheels

		public void RaceTrack()
		{ 
			// path
			List<ModelGraphNode> pathNodes = new List<ModelGraphNode>();
			List<ModelGraphEdge> pathEdges = new List<ModelGraphEdge>();
			ModelGraph mg = this.modelGraph;
			int nNodes = mg.graphNodes.Count;

			foreach(ModelGraphNode mn in mg.graphNodes)
			{
				if (mn.name.Contains("plane") && !mn.name.Contains("up"))
				{
					bool find = true;
					foreach (ModelGraphEdge me in mn.edges)
					{
						ModelGraphNode adjmn = me.conntectingNodes[0] == mn ?
							me.conntectingNodes[1] : me.conntectingNodes[0];
						if(adjmn.name.Contains("upplane"))
						{
							find = false;
							break;
						}
					}
					if (find)
					{
						foreach (ModelGraphEdge me in mn.edges)
						{
							if (me.faceIndex[0] % 2 != 0 && me.faceIndex[1] % 2 != 0)
							{
								pathNodes.Add(mn);
								break;
							}
						}
					}
					if (pathNodes.Count > 0) break;
				}
			}
			// load a car
			string fullPath = @"F:\Projects\GraphEditing\models\raceTrack\car.obj";
            //string fullPath = @"F:\Projects\GraphEditing\models\trackingballs\ball\ball.obj";
			// load mesh model
			StreamReader sr = new StreamReader(fullPath);
			string extension = "obj";
			this.trackball = new Mesh(sr, extension, true);
			sr.Close();

			Vector3d pos = new Vector3d();
			ModelGraphNode startNode = pathNodes[0];
			double smallest = double.MaxValue;

			pos = startNode.meshComponent.mesh.MaxCoord() - startNode.meshComponent.mesh.MinCoord();
			smallest = Math.Min(pos.x, pos.z);

			
			pos = startNode.meshComponent.boundingBox.center.pos;

			double ratio = smallest / (this.trackball.MaxCoord() - this.trackball.MinCoord()).Length();
			Vector3d scale = new Vector3d(ratio, ratio, ratio);
			Matrix4d scaleMat = Matrix4d.ScalingMatrix(scale);
			this.trackball.Transform(scaleMat);

			pos = startNode.meshComponent.boundingBox.center.pos;
			pos.x = startNode.meshComponent.mesh.MinCoord().x;
			Vector3d center = (this.trackball.MaxCoord() + this.trackball.MinCoord()) / 2;
			Vector3d translation = pos - center;
			Matrix4d transMat = Matrix4d.TranslationMatrix(translation);
			this.trackball.Transform(transMat);

			this.Refresh();


			// start from clockwise direction

			foreach (ModelGraphEdge me in pathNodes[0].edges)
			{
				ModelGraphNode adjmn = (me.conntectingNodes[0] == pathNodes[0] ?
						me.conntectingNodes[1] : me.conntectingNodes[0]);
				int fidx = me.conntectingNodes[0] == pathNodes[0]? me.faceIndex[0]:me.faceIndex[1];
				if (!adjmn.name.Contains("base") && fidx == 1)
				{
					pathNodes.Add(adjmn);
					pathEdges.Add(me);
					break;
				}
			}
		
			int start = 1;
			while (start < pathNodes.Count)
			{
				ModelGraphNode mn = pathNodes[start++];
				foreach(ModelGraphEdge me in mn.edges)
				{
					ModelGraphNode adjmn = (me.conntectingNodes[0] == mn ?
						me.conntectingNodes[1] : me.conntectingNodes[0]);
					if (!adjmn.name.Contains("base") && !pathNodes.Contains(adjmn))
					{
						pathNodes.Add(adjmn);
						pathEdges.Add(me);
						break;
					}
				}
			}// collect path

			foreach(ModelGraphEdge me in pathNodes[pathNodes.Count-1].edges)
				if(me.conntectingNodes.Contains(pathNodes[0]))
				{
					pathEdges.Add(me);
					break;
				}
			if (pathEdges.Count != pathNodes.Count)
			{
				MessageBox.Show("Wrong loop!");
				return;
			}

			// run
			Vector3d carCenter = (this.trackball.MaxCoord() + this.trackball.MinCoord()) / 2;
			// arc
			Vector3d arcScale = new Vector3d();
			Vector3d upplaneScale = new Vector3d();
			nNodes = pathNodes.Count;
			for (int i = 0; i < nNodes; ++i)
			{
				ModelGraphNode mn = pathNodes[i];
				if (mn.name.Contains("arc") && pathEdges[i].faceIndex[0] < 5)
				{
					arcScale = mn.meshComponent.mesh.MaxCoord() - mn.meshComponent.mesh.MinCoord();
					break;
				}
			}
			for (int i = 0; i < nNodes; ++i)
			{
				ModelGraphNode mn = pathNodes[i];
				if (mn.name.Contains("upplane"))
				{
					upplaneScale = mn.meshComponent.mesh.MaxCoord() - mn.meshComponent.mesh.MinCoord();
					break;
				}
			}
			double arcRadius = Math.Min(arcScale[0], arcScale[2]);
            arcRadius -= arcRadius / 3;
			double upplaneRadius = Math.Min(arcScale[0], arcScale[1]);
           // upplaneRadius -= upplaneRadius / 4;

			int cur = 0;
			
			while (cur < nNodes)
			{
				ModelGraphNode curNode = pathNodes[cur];
                //break;
				ModelGraphEdge curEdge = pathEdges[cur];
				ModelGraphEdge prevEdge = pathEdges[(cur - 1 + nNodes) % nNodes];
				int curFaceIdx = curEdge.conntectingNodes[0] == curNode ? 
                    curEdge.faceIndex[0] : curEdge.faceIndex[1];
				int prevcurFaceIdx = prevEdge.conntectingNodes[0] == curNode ? 
                    prevEdge.faceIndex[0] : prevEdge.faceIndex[1];
				int dir = MapPostivieToNeg(curFaceIdx);
                int prevDir = MapPostivieToNeg(prevcurFaceIdx);
                int prevAxis = Math.Abs(prevDir) - 1;

                scale = curNode.meshComponent.mesh.MaxCoord() - curNode.meshComponent.mesh.MinCoord();

				double slope = curNode.meshComponent.mesh.MaxCoord().y - curNode.meshComponent.mesh.MinCoord().y;
				double slopeStart = curNode.meshComponent.mesh.MaxCoord().y ;
				double slopeSign = 1;

                Vector3d arcCenter = new Vector3d();
                double radius = arcRadius;
				int mode = 0;
				int XYZ = 0; //XZ
                bool counterclockwise = true;
                double startAngle = 0;
                int sign = 1;
				if (curNode.name.Contains("upplane"))
				{
					mode = 2;
                    if ((curFaceIdx < 5 && curFaceIdx % 2 == 0) ||
                        (prevcurFaceIdx < 5 && prevcurFaceIdx % 2 == 0))//YZ
                    {
                        XYZ = 2;
                        if ((prevcurFaceIdx == 4 && curFaceIdx == 6) ||
                                (prevcurFaceIdx == 6 && curFaceIdx == 4))
                            arcCenter = new Vector3d(
                                    (curNode.meshComponent.mesh.MinCoord().x +
                                    curNode.meshComponent.mesh.MaxCoord().x) / 2,
                                    curNode.meshComponent.mesh.MaxCoord().y,
                                    curNode.meshComponent.mesh.MaxCoord().z);
                        else if ((prevcurFaceIdx == 2 && curFaceIdx == 6) ||
                                (prevcurFaceIdx == 6 && curFaceIdx == 2))
                            arcCenter = new Vector3d(
                                    (curNode.meshComponent.mesh.MinCoord().x +
                                    curNode.meshComponent.mesh.MaxCoord().x) / 2,
                                    curNode.meshComponent.mesh.MaxCoord().y,
                                    curNode.meshComponent.mesh.MinCoord().z);
                        if ((prevcurFaceIdx == 6 && curFaceIdx == 4) ||
                                (prevcurFaceIdx == 2 && curFaceIdx == 6))
                        {
                            counterclockwise = false;
                        }
                        if ((prevcurFaceIdx == 2 && curFaceIdx == 6))
                            startAngle = Math.PI * 3 / 2;
                        if ((prevcurFaceIdx == 6 && curFaceIdx == 2))
                            sign = -1;
						
                        radius = upplaneRadius;
                    }
                    else if ((curFaceIdx < 5 && curFaceIdx % 2 != 0) ||
                        (prevcurFaceIdx < 5 && prevcurFaceIdx % 2 != 0)) //XY
                    {
                        XYZ = 1;

                        if ((prevcurFaceIdx == 1 && curFaceIdx == 6) ||
                                (prevcurFaceIdx == 6 && curFaceIdx == 1))
                            arcCenter = new Vector3d(curNode.meshComponent.mesh.MaxCoord().x,
                                curNode.meshComponent.mesh.MaxCoord().y,
                                (curNode.meshComponent.mesh.MinCoord().z +
                                curNode.meshComponent.mesh.MaxCoord().z) / 2);
                        else if ((prevcurFaceIdx == 3 && curFaceIdx == 6) ||
                                (prevcurFaceIdx == 6 && curFaceIdx == 3))
                            arcCenter = new Vector3d(curNode.meshComponent.mesh.MinCoord().x,
                                curNode.meshComponent.mesh.MaxCoord().y,
                                (curNode.meshComponent.mesh.MinCoord().z +
                                curNode.meshComponent.mesh.MaxCoord().z) / 2);
                        if ((prevcurFaceIdx == 6 && curFaceIdx == 3) ||
                                (prevcurFaceIdx == 1 && curFaceIdx == 6))
                        {
                            counterclockwise = false;
                        }
                        if ((prevcurFaceIdx == 3 && curFaceIdx == 6))
                            startAngle = Math.PI * 3 / 2;
                        if ((prevcurFaceIdx == 6 && curFaceIdx == 3))
                            sign = -1;
                    }
				}
				else if (curNode.name.Contains("arc"))
				{
					mode = 1; 

                    if ((prevcurFaceIdx == 1 && curFaceIdx == 2) ||
                        (prevcurFaceIdx == 2 && curFaceIdx == 1))
                        arcCenter = new Vector3d(curNode.meshComponent.mesh.MaxCoord().x,
                            (curNode.meshComponent.mesh.MinCoord().y +
                            curNode.meshComponent.mesh.MaxCoord().y)/2,
                            curNode.meshComponent.mesh.MinCoord().z);
                    else if ((prevcurFaceIdx == 2 && curFaceIdx == 3) ||
                        (prevcurFaceIdx == 3 && curFaceIdx == 2))
                        arcCenter = new Vector3d(curNode.meshComponent.mesh.MinCoord().x,
                            (curNode.meshComponent.mesh.MinCoord().y +
                            curNode.meshComponent.mesh.MaxCoord().y) / 2,
                            curNode.meshComponent.mesh.MinCoord().z);
                    else if ((prevcurFaceIdx == 3 && curFaceIdx == 4) ||
                        (prevcurFaceIdx == 4 && curFaceIdx == 3))
                        arcCenter = new Vector3d(curNode.meshComponent.mesh.MinCoord().x,
                            (curNode.meshComponent.mesh.MinCoord().y +
                            curNode.meshComponent.mesh.MaxCoord().y) / 2,
                            curNode.meshComponent.mesh.MaxCoord().z);
                    else if ((prevcurFaceIdx == 4 && curFaceIdx == 1) ||
                        (prevcurFaceIdx == 1 && curFaceIdx == 4))
                        arcCenter = new Vector3d(curNode.meshComponent.mesh.MaxCoord().x,
                            (curNode.meshComponent.mesh.MinCoord().y +
                            curNode.meshComponent.mesh.MaxCoord().y) / 2,
                            curNode.meshComponent.mesh.MaxCoord().z);

                    if ((prevcurFaceIdx == 2 && curFaceIdx == 1) ||
                        (prevcurFaceIdx == 3 && curFaceIdx == 2) ||
                         (prevcurFaceIdx == 4 && curFaceIdx == 3) ||
                        (prevcurFaceIdx == 1 && curFaceIdx == 4))
                        counterclockwise = false;

					if ((prevcurFaceIdx == 4 && curFaceIdx == 3))
					{
						startAngle = Math.PI / 2;
						//slopeSign = -1;
					}
                    if ((prevcurFaceIdx == 3 && curFaceIdx == 4))
                    {
                        startAngle = -Math.PI;
                        sign = -1;
						
                    }
                    if ((prevcurFaceIdx == 2 && curFaceIdx == 3))
                    {
                        startAngle = Math.PI / 2;
                        sign = -1;
                    }
					if ((prevcurFaceIdx == 3 && curFaceIdx == 2))
					{
						startAngle = 0;
						//slopeSign = -1;
					}
					if ((prevcurFaceIdx == 2 && curFaceIdx == 1))
					{
						startAngle = Math.PI * 3 / 2;
						//slopeSign = -1;
					}
                    if ((prevcurFaceIdx == 1 && curFaceIdx == 2))
                    {
                        startAngle = 0;
                        sign = -1;
						//slopeSign = 1;
                    }
                    if ((prevcurFaceIdx == 4 && curFaceIdx == 1))
                    {
                        startAngle = -Math.PI / 2;
                        sign = -1;
                    }
					if ((prevcurFaceIdx == 1 && curFaceIdx == 4))
					{
						startAngle = Math.PI;
						//slopeSign = -1;
					}

					if (curFaceIdx > 4 || prevcurFaceIdx > 4)
					{
                        mode = 2;
                        if ((curFaceIdx < 5 && curFaceIdx % 2 == 0) ||
                            (prevcurFaceIdx < 5 && prevcurFaceIdx % 2 == 0)) //YZ
                        {
                            XYZ = 2;

                            if ((prevcurFaceIdx == 5 && curFaceIdx == 4) ||
                                (prevcurFaceIdx == 4 && curFaceIdx == 5))
                                arcCenter = new Vector3d(
                                    (curNode.meshComponent.mesh.MinCoord().x +
                                    curNode.meshComponent.mesh.MaxCoord().x) / 2,
                                    curNode.meshComponent.mesh.MinCoord().y,
                                    curNode.meshComponent.mesh.MaxCoord().z);
                            else if ((prevcurFaceIdx == 5 && curFaceIdx == 2)||
                                (prevcurFaceIdx == 2 && curFaceIdx == 5))
                                arcCenter = new Vector3d(
                                    (curNode.meshComponent.mesh.MinCoord().x +
                                    curNode.meshComponent.mesh.MaxCoord().x) / 2,
                                    curNode.meshComponent.mesh.MinCoord().y,
                                    curNode.meshComponent.mesh.MinCoord().z);
                            if ((prevcurFaceIdx == 5 && curFaceIdx == 2) ||
                                (prevcurFaceIdx == 4 && curFaceIdx == 5))
                                counterclockwise = false;

                            if ((prevcurFaceIdx == 5 && curFaceIdx == 4))
                                startAngle = Math.PI * 3 /2;
                            if ((prevcurFaceIdx == 4 && curFaceIdx == 5))
                                sign = -1;

                            if ((prevcurFaceIdx == 5 && curFaceIdx == 2))
                            {
                                startAngle = Math.PI / 2;
                                sign = -1;
                            }
                            
                        }
                        else if ((curFaceIdx < 5 && curFaceIdx % 2 != 0) ||
                            (prevcurFaceIdx < 5 && prevcurFaceIdx % 2 != 0)) //XY
                        {
                            XYZ = 1;

                            if ((prevcurFaceIdx == 5 && curFaceIdx == 1)||
                                (prevcurFaceIdx == 1 && curFaceIdx == 5))
                                arcCenter = new Vector3d(curNode.meshComponent.mesh.MaxCoord().x,
                                    curNode.meshComponent.mesh.MinCoord().y,
                                    (curNode.meshComponent.mesh.MinCoord().z +
                                    curNode.meshComponent.mesh.MaxCoord().z) / 2);
                            else if ((prevcurFaceIdx == 5 && curFaceIdx == 3)||
                                (prevcurFaceIdx == 3 && curFaceIdx == 5))
                                arcCenter = new Vector3d(curNode.meshComponent.mesh.MinCoord().x,
                                    curNode.meshComponent.mesh.MinCoord().y,
                                    (curNode.meshComponent.mesh.MinCoord().z +
                                    curNode.meshComponent.mesh.MaxCoord().z) / 2);
                            if ((prevcurFaceIdx == 5 && curFaceIdx == 1) ||
                                (prevcurFaceIdx == 3 && curFaceIdx == 5))
                                counterclockwise = false;
                        }
					}
					if (pathNodes[(cur - 1 + nNodes) % nNodes].meshComponent.MaxCoord().y >
						curNode.meshComponent.MaxCoord().y)
						slopeSign = -1;
				}//arc

				if (XYZ == 1)
				{
					slopeStart = curNode.meshComponent.mesh.MaxCoord().z;
					slope = curNode.meshComponent.mesh.MaxCoord().z - curNode.meshComponent.mesh.MinCoord().z;
				}
				else if (XYZ == 2)
				{
					slopeStart = curNode.meshComponent.mesh.MaxCoord().x;
					slope = curNode.meshComponent.mesh.MaxCoord().x - curNode.meshComponent.mesh.MinCoord().x;
				}
				slope /= 2;
				// length
				Vector3d movedir = new Vector3d();
				int axis = Math.Abs(dir) - 1;
				movedir[axis] = 1;
                if (dir < 0) movedir[axis] = -1;
				
				double length = Math.Abs(scale[axis]);
                if(mode == 0)
                    length *= 0.8;
                if (mode == 1)
                {
                    length = Math.PI * arcRadius / 2;
                    length *= 0.5;
                }
                if (mode == 2)
                {
                    length = Math.PI * upplaneRadius / 2;
                    //length *= 0.7;
                }
                
				// speed
				int rollTimes = 10;
                if (mode == 0) rollTimes = 8;
                if (mode == 2) rollTimes = 6;
				double transStep = length / rollTimes;

				double angleStep = Math.PI / rollTimes / 2;

				double slopeStep = slope / rollTimes;

				movedir *= transStep;
				Matrix4d translationMat = Matrix4d.TranslationMatrix(movedir);
                if (mode == 0)
                {
					pos =  (curNode.meshComponent.mesh.MaxCoord() + curNode.meshComponent.mesh.MinCoord()) / 2;
                    pos -= carCenter;
					pos[1] = slopeStart-carCenter.y;
                    pos[0] = 0; pos[2] = 0;
                    //pos /= ntrans;
                    translationMat = Matrix4d.TranslationMatrix(pos);
                    this.trackball.Transform(translationMat);
                    carCenter = (this.trackball.MaxCoord() + this.trackball.MinCoord()) / 2;
                    this.Refresh();
                }
				if (XYZ == 2)
				{
					pos = (curNode.meshComponent.mesh.MaxCoord() + curNode.meshComponent.mesh.MinCoord()) / 2;
					pos[0] = curNode.meshComponent.mesh.MinCoord().x - carCenter.x;
					pos[1] = 0; pos[2] = 0;
					//pos /= ntrans;
					translationMat = Matrix4d.TranslationMatrix(pos);
					this.trackball.Transform(translationMat);
					carCenter = (this.trackball.MaxCoord() + this.trackball.MinCoord()) / 2;
					this.Refresh();
				}
                translationMat = Matrix4d.TranslationMatrix(movedir);

				Matrix4d rotMat = new Matrix4d();

				if (XYZ == 0)//XZ
				{
					rotMat[0, 0] = Math.Cos(angleStep);
					rotMat[0, 2] = -1*Math.Sin(angleStep);
					rotMat[2, 0] = Math.Sin(angleStep);
                    if (!counterclockwise)
                    {
                        rotMat[0, 2] = Math.Sin(angleStep);
                        rotMat[2, 0] = -1 * Math.Sin(angleStep);
                    }
					rotMat[2, 2] = Math.Cos(angleStep);
					rotMat[1, 1] = 1;
				}
				else if (XYZ == 1) //XY
				{
					rotMat[0, 0] = Math.Cos(angleStep);
					rotMat[0, 1] = -1 * Math.Sin(angleStep);
					rotMat[1, 0] = Math.Sin(angleStep);
                    if (!counterclockwise)
                    {
                        rotMat[0, 1] = Math.Sin(angleStep);
                        rotMat[1, 0] = -1*Math.Sin(angleStep);
                    }
					rotMat[1, 1] = Math.Cos(angleStep);
					rotMat[2, 2] = 1;
				}
				else //YZ
				{
					rotMat[1, 1] = Math.Cos(angleStep);
					rotMat[1, 2] = -1 * Math.Sin(angleStep);
					rotMat[2, 1] = Math.Sin(angleStep);
                    if (!counterclockwise)
                    {
                        rotMat[1, 2] = Math.Sin(angleStep);
                        rotMat[2, 1] = -1 * Math.Sin(angleStep);
                    }
					rotMat[2, 2] = Math.Cos(angleStep);
					rotMat[0, 0] = 1;
				}
				rotMat[3, 3] = 1;

                // from last pos to current
                
                Vector3d currpos = new Vector3d();
				int startroll = 0;
				while (startroll < rollTimes)
				{
					
					switch (mode)
					{
						case 0: //trans	
                           
                            this.trackball.Transform(translationMat);
							break;
						case 1: //rot XZ plane

                            currpos = new Vector3d(arcCenter.x +
                                radius * Math.Sin(startAngle + sign * angleStep * startroll),
								carCenter.y + slopeSign * slopeStep, arcCenter.z +
                                radius * Math.Cos(startAngle + sign * angleStep * startroll));
                           
                            //currpos += diff;
                            movedir = currpos - carCenter;                            
                            translationMat = Matrix4d.TranslationMatrix(movedir);
                            this.trackball.Transform(translationMat);
							this.trackball.TransformCenter(rotMat);
							break;
						case 2: //rot XY or YZ plane
                                if (XYZ == 1)
                                {
                                    currpos = new Vector3d(arcCenter.x +
									radius * Math.Cos(startAngle + sign * angleStep * startroll) * 1.5,
                                    arcCenter.y +
									radius * Math.Sin(startAngle + sign * angleStep * startroll) * 1.5,
									carCenter.z);// + slopeSign * slopeStep);
                                }
                                else
                                {
									if (curNode.name.Contains("arc"))
									{
										currpos = new Vector3d(carCenter.x,// + slopeSign * slopeStep,
										arcCenter.y +
										radius * Math.Cos(startAngle + sign * angleStep * startroll),
										arcCenter.z +
										radius * Math.Sin(startAngle + sign * angleStep * startroll)
										);
									}
									else
									{
										currpos = new Vector3d(carCenter.x + slopeStep,
										arcCenter.y +
										radius * Math.Cos(startAngle + sign * angleStep * startroll) * 1.5,
										arcCenter.z +
										radius * Math.Sin(startAngle + sign * angleStep * startroll) * 1.5
										);
									}
                                }
							movedir = currpos - carCenter;
                            translationMat = Matrix4d.TranslationMatrix(movedir);
                            this.trackball.Transform(translationMat);
							this.trackball.TransformCenter(rotMat);
							break;
					}
					carCenter = (this.trackball.MaxCoord() + this.trackball.MinCoord()) / 2;
					++startroll;
					this.Refresh();
				}// movement

				++cur;

			}//end-run

		}//RaceTrack

		private void BalanceHeight()
		{
			ModelGraph mg = this.modelGraph;
			List<ModelGraphNode> shortNodes = new List<ModelGraphNode>(),
				tallNodes = new List<ModelGraphNode>();
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				if (mn.edges.Count == 1 && mn.adjNodes[0].level > mn.level)
				{
					if (mn.level > 0)
						tallNodes.Add(mn);
					else if (mn.level < 0)
						shortNodes.Add(mn);
				}
			}
			if (tallNodes.Count > 0 || shortNodes.Count > 0)
				CorrectGraph();
		}//BalanceHeight


		public void CorrectGraph()
		{
 			// add support nodes
			ModelGraph mg = this.modelGraph;
			double minHeight = 0;
			List<ModelGraphNode> errorBaseNodes = new List<ModelGraphNode>();
			//this.modelGraph.CalculateNodeLvelByDirections();
			int minLevel = 0;
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
				minLevel = (minLevel < mn.level ? minLevel : mn.level);
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
				mn.level -= minLevel;
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				if (mn.edges.Count == 1 && mn.adjNodes[0].level > mn.level && mn.level > 0)
					//if ((mn.name.Contains("base") || mn.name.Contains("out")) && mn.level != 0)
					errorBaseNodes.Add(mn);
			}

			foreach (ModelGraphNode baseNode in errorBaseNodes)
			{
				int numToAdd = baseNode.level;
				//if (baseNode.adjNodes[0].edges.Count > 2 )
				//{
				//    MessageBox.Show("Cannot fix the graph!");
				//    return;
				//}
				// add nodes between base node and source node
				ModelGraphNode sourceNode = baseNode.adjNodes[0];

				ModelGraphNode adjNode = sourceNode.adjNodes[0] == baseNode ?
					sourceNode.adjNodes[1] : sourceNode.adjNodes[0];

                Vector3d heigVec = new Vector3d();
                double H = sourceNode.meshComponent.mesh.MaxCoord().y
                    - sourceNode.meshComponent.mesh.MinCoord().y;
                double h12 = adjNode.meshComponent.mesh.MaxCoord().y
                    - sourceNode.meshComponent.mesh.MinCoord().y;
                heigVec.y = (H * 2 - h12) / 2 -
                    sourceNode.meshComponent.boundingBox.center.pos.y
                    + adjNode.meshComponent.boundingBox.center.pos.y; //!from top to bottom
                heigVec[0] = 0; heigVec[2] = 0; 
                heigVec[1] = -heigVec[1];
                if (baseNode.name.Contains("base"))
                    heigVec[1] *= 0.9;
				double height = heigVec[1];

                int n = (int)(baseNode.meshComponent.boundingBox.center.pos.y / H);
                //numToAdd = numToAdd < n ? numToAdd : n;
                
				Matrix4d T = Matrix4d.TranslationMatrix(heigVec);
				
				sourceNode.level += numToAdd;
				baseNode.level = 0;
				ModelGraphNode prevNode = sourceNode;
				
				for (int i = 0; i < numToAdd; ++i)
				{
					ModelGraphNode copyNode = sourceNode.Clone() as ModelGraphNode;
					copyNode.name = @"add_" + (i + 1).ToString() +@"_" + copyNode.name;
					copyNode.level = numToAdd - i;
					copyNode.adjNodes = new List<ModelGraphNode>();
					T = Matrix4d.TranslationMatrix(heigVec);
					copyNode.meshComponent.mesh.Transform(T);
					copyNode.meshComponent.boundingBox.Transform(T);
					mg.graphNodes.Add(copyNode);
					// topology
					Vector3d _pos = (copyNode.meshComponent.boundingBox.center.pos +
						prevNode.meshComponent.boundingBox.center.pos)/2;
					ModelGraphEdge me = new ModelGraphEdge(_pos, mg.graphIndex, mg.graphEdges.Count);
					me.AddConnectingNode(prevNode);
					me.AddConnectingNode(copyNode);
					copyNode.adjNodes.Add(prevNode);
					prevNode.adjNodes.Add(copyNode);
					mg.graphEdges.Add(me);
					prevNode = copyNode; // at each time, only add one edge to the previous node
					this.meshComponents.Add(copyNode.meshComponent);

					heigVec.y += height;

					if (i < numToAdd - 1) continue;

					ModelGraphEdge baseEdge = baseNode.edges[0];
					sourceNode.edges.Remove(baseEdge);
					baseNode.edges.Remove(baseEdge);
					mg.graphEdges.Remove(baseEdge);
					sourceNode.adjNodes.Remove(baseNode);
					baseNode.adjNodes.Remove(sourceNode);

					heigVec.y -= height;
					T = Matrix4d.TranslationMatrix(heigVec);
					baseNode.meshComponent.mesh.Transform(T);
					baseNode.meshComponent.boundingBox.Transform(T);

					_pos = (copyNode.meshComponent.boundingBox.center.pos +
						baseNode.meshComponent.boundingBox.center.pos) / 2;
					me = new ModelGraphEdge(_pos, mg.graphIndex, mg.graphEdges.Count);
					me.AddConnectingNode(copyNode);
					me.AddConnectingNode(baseNode);
					copyNode.adjNodes.Add(baseNode);
					baseNode.adjNodes.Add(copyNode);
					mg.graphEdges.Add(me);
				}

				minHeight = baseNode.meshComponent.MinCoord().y < minHeight ?
					baseNode.meshComponent.MinCoord().y : minHeight;
			}//correct each string
			this.Refresh();
			if (minHeight != 0 )
			{
				Matrix4d T = Matrix4d.TranslationMatrix(new Vector3d(0,-minHeight,0));
				foreach (ModelGraphNode mn in mg.graphNodes)
				{
					mn.meshComponent.mesh.Transform(T);
					mn.meshComponent.boundingBox.Transform(T);
				}
			}
		}//CorrectGraph


		public void CheckValidity()
		{
			int nInput = 0, nOutput = 0;
			foreach (ModelGraphNode mn in this.modelGraph.graphNodes)
			{
				if (mn.name.Contains("input"))
					nInput++;
				if (mn.name.Contains("output"))
					nOutput++;
			}
			if (nInput == 0 && nOutput == 0)
				MessageBox.Show("No entry & exit!");
			else if (nInput == 0)
				MessageBox.Show("No entry!");
			else if (nOutput == 0)
				MessageBox.Show("No exit!");
		}
    }//Class-MeshView

}//Namespace-ShapeGraph
